#include <math.h>
#include "Say.h"
#include "AIMNetThread.h"
#include "Globals.h"

//---------------------------------------------------------------------

// Send message function
void AIMNetThread::SendMessage( BMessage* msg ) {

	char Message[2048];
	SNAC_Object sendMsg;
	bool isAutoResponse = msg->FindBool( "autorespond" );

	// Wrap the message in some generic HTML (AOL style)
	strcpy( Message, "<HTML><BODY BGCOLOR=\"#ffffff\">" );
	strcat( Message, msg->FindString("message") );
	strcat( Message, "</HTML>" );
	
	// Make the SNAC object for the message
	sendMsg.Update( 0x04, 0x06, 0x00, 0x00, requestID );
	sendMsg.data << ToAIMWord(0x00) << ToAIMWord(0x00)
				 << ToAIMWord(0x00) << ToAIMWord(0x00)
				 << ToAIMWord(0x01)
				 << char(strlen((char*)msg->FindString("send_to")))		// sn length
				 << (char*)msg->FindString("send_to");					// sn
				 
	// handle the auto-respond case and stuff
	if( isAutoResponse ) {
		sendMsg.data << ToAIMWord(0x04)
					 << ToAIMWord(0x00)
					 << ToAIMWord(0x02);
	} else
		sendMsg.data << ToAIMWord(0x02);
				 
	// finish up with the packet
	sendMsg.data << ToAIMWord(strlen(Message) + 0x0D)					// msg len + 0x0D
				 << char(0x05) << ToAIMWord(0x0100)
				 << ToAIMWord(0x0101) << ToAIMWord(0x0101)
				 << ToAIMWord(strlen(Message) + 0x04)					// msg len + 0x04
				 << ToAIMWord(0x00) << ToAIMWord(0x00)
				 << Message;
				 
	// log it, so we know what happened in case the message never gets there
	LogMessageReqID( BString((char*)msg->FindString("send_to")), RID_SEND_MESSAGE );

	// Now that we've gone to all the trouble of making that, send it
	SendSNACPacket( sendMsg );
}

//---------------------------------------------------------------------

void AIMNetThread::DecodeBuddyStuff( DataContainer data, short& i, BMessage* values ) {

	char snLength;
	char* sn;
	unsigned short warningLevel, numTLVs, typeCode,
				   tlvLen, userClass, idleTime, temp;
	time_t memberSince = 0, onSince = 0;

	struct tm* t_s; // to get localtime(time_t t)

	// Get the screen name
	snLength = data[i++];
	sn = new char[snLength+1];
	strncpy( sn, data.getrange(i,snLength).c_ptr(), snLength );
	sn[snLength] = '\0';
	i += snLength;
	if( values )
		values->AddString( "userid", sn );	
	
	printf( "BUDDY_TLV... screen name (%d chars): %s\n", (int)snLength, sn );
	delete sn;
	
	// Get the sender's warning level
	warningLevel = (unsigned short)( rint( (double)GetWord(data,i) / 10.0 ) );
	i += 2;
	if( values )
		values->AddInt32( "warninglevel", (int32)warningLevel );
	
	printf( "BUDDY_TLV... warning level %d\n", warningLevel );
	
	// Get the number of TLV's in the packet
	numTLVs = (unsigned short)GetWord( data, i );
	i += 2;

	printf( "BUDDY_TLV... %d TLV's in this packet\n", numTLVs );
	
	// Go through and grab the rest of the TLV's
	for( unsigned short j = 0; j < numTLVs; ++j ) {
	
		// Grab the type code and length
		typeCode =	(unsigned short)GetWord( data, i );
		tlvLen = (unsigned short)GetWord( data, i+2 );
		i += 4;

		// grab data based on the type code
		switch( typeCode ) {
		
			// classes... 0x10=free, 0x04=AOL... 0x11 is the "new" AIM
			case 0x01:	
				userClass = (unsigned short)GetWord( data, i );
				i += 2;
				if( userClass == 0x10 )
					values->AddString( "userclass", "Internet" );
				else if( values && userClass == 0x11 )
					values->AddString( "userclass", "Trial Internet" );
				else if( values && userClass == 0x04 )
					values->AddString( "userclass", "AOL" );
				else if( userClass == 0x30 || userClass == 0x31 ) {
					values->AddString( "userclass", "Internet" );
					values->AddBool( "away", true );
				} else if( values )
					values->AddString( "userclass", "Unknown" );

				if( userClass == 0x10 || userClass == 0x11 )
					printf( "BUDDY_TLV... Class = Non-AOL (probably AIM)\n" );
				else if( userClass == 0x04 )
					printf( "BUDDY_TLV... Class = AOL user\n" );
				else if( userClass == 0x30 || userClass == 0x31 )
					printf( "BUDDY_TLV... Class = Away??\n" );
				else
					printf( "BUDDY_TLV... Class = Huh? (unknown, val=0x%X)\n", userClass );
				break;
				
			case 0x02:	// member since date
				temp = (unsigned short)GetWord( data, i );
				memberSince = (temp << 16) + (unsigned short)GetWord( data, i+2 );
				{DataContainer tempDC = data.getrange( i, 4 );
				printf( "membersince: " );
				Dump( tempDC );}
				i += 4;
				if( values )
					values->AddData( "membersince", B_TIME_TYPE, (void*)&memberSince, sizeof(time_t) );
				
				t_s = localtime( &memberSince );
				printf( "BUDDY_TLV: Member since date: %s", asctime(t_s) );				
				break;
			
			case 0x03:	// online since date
				temp = (unsigned short)GetWord( data, i );
				onSince = (temp << 16) + (unsigned short)GetWord( data, i+2 );
				{DataContainer tempDC = data.getrange( i, 4 );
				printf( "onsince: " );
				Dump( tempDC );}
				i += 4;
				if( values )
					values->AddData( "onsince", B_TIME_TYPE, (void*)&onSince, sizeof(time_t) );
				
				t_s = localtime( &onSince );
				printf( "BUDDY_TLV: Online since: %s", asctime(t_s) );				
				break;			
			
			case 0x04:	// idle time
				idleTime = (unsigned short)GetWord( data, i );
				i += 2;
				if( values )
					values->AddInt32( "idletime", (int32)idleTime );
				
				printf( "BUDDY_TLV: Idle time: %u\n", idleTime );
				break;
		
			default:	// something unknown... just skip it
				printf( "BUDDY_TLV: Unknown TLV: type 0x%X length (%d)\n", typeCode, tlvLen );
				i += tlvLen;
		}	
	}
}

//---------------------------------------------------------------------

void AIMNetThread::BuddyOnline( SNAC_Object& snac ) {

	short i = 0;
	BMessage* newBuddy = new BMessage( BEAIM_ONCOMING_BUDDY );
	
	// The buddy info comes at the beginnning... yank it out
	DecodeBuddyStuff( snac.data, i, newBuddy );
	
	// Send the message
	SendAppMessage( newBuddy );
}

//---------------------------------------------------------------------

void AIMNetThread::BuddyOffline( SNAC_Object& snac ) {

	char name[DISPLAY_NAME_MAX];
	BMessage* oldBuddy = new BMessage( BEAIM_OFFGOING_BUDDY );
	char len;
	
	// get the screen name
	len = snac.data[0];
	printf( "logoff name len: %d\n", int(len) );

	snac.data << char(0);	
	strcpy( name, snac.data.c_ptr() + 1 );
	printf( "offgoing name: %s\n", name );
	
	
	oldBuddy->AddString( "userid", name );
	
	SendAppMessage( oldBuddy );	
}

//---------------------------------------------------------------------

// Receive message function
void AIMNetThread::ReceiveMessage( SNAC_Object& snac ) {

	short i = 10;
	unsigned short temp, temp2;
	unsigned int languageValue;
	bool autoRespond = false;
	BMessage* incomingMsg = new BMessage(BEAIM_INCOMING_IM);
	DataContainer msg;
	
	// Grab the other person's info
	DecodeBuddyStuff( snac.data, i, incomingMsg );

	// This will be either 0x0002 or 0x0004 (autorespond mode?)
	temp = (unsigned short)GetWord( snac.data, i );
	i += 2;
	if( temp == 0x0004 ) {
		temp = (unsigned short)GetWord( snac.data, i );		// 0x0000
		temp = (unsigned short)GetWord( snac.data, i+2 );	// 0x0002
		autoRespond = true;
		i += 4;
	}

	if( autoRespond )
		printf( "AutoResponded IM...\n" );
	incomingMsg->AddBool( "autorespond", autoRespond );

	// Reality check
	if( temp != 0x0002 )
		printf( "RECV_MESSAGE ERROR: temp should be 0x0002, not 0x%X!\n", temp );
	
	// Get the message length (with 0x0D tacked on), and fixed value 0x0501
	temp = (unsigned short)GetWord( snac.data, i );
	temp = (unsigned short)GetWord( snac.data, i+2 );
	i += 4;
	
	// Another reality check
	if( temp != 0x0501 )
		printf( "RECV_MESSAGE ERROR: temp should be 0x0501, not 0x%X!\n", temp );
	
	// Grab the mysterious language value
	temp = (unsigned short)GetWord( snac.data, i );
	temp2 = (unsigned short)GetWord( snac.data, temp + i + 8 );
	languageValue = (temp2 << 16) + (unsigned short)GetWord( snac.data, temp + i + 6 );
	incomingMsg->AddInt32( "languagevalue", (int32)languageValue );
	i += 2;	
	
	printf( "Mysterious Language value: 0x%X\n", (unsigned)languageValue );
	
	// Get the message itself
	msg = snac.data.getrange( temp + i + 8 );
	msg << char(0);
	incomingMsg->AddString( "message", (char*)msg.c_ptr() );
	
	printf( "actual message: %s\n", msg.c_ptr() );
	
	// Send the message off its final resting place... the great bit-bucket in the sky
	incomingMsg->AddInt32( "wtype", (int32)USER_MESSAGE_TYPE );
	SendAppMessage( incomingMsg );
}

//---------------------------------------------------------------------

// Requests the info and profile for a member
void AIMNetThread::RequestPersonInfo( BMessage* msg ) {

	printf( "requesting person info... %s\n", msg->FindString("userid") );

	// Make the SNAC object
	SNAC_Object snac( 0x02, 0x05, 0, 0, requestID );
	snac.data << ToAIMWord(0x0001)
			  << char(strlen(msg->FindString("userid")))
			  << msg->FindString("userid");
			  
	// add an object to the pending list
	userRequestID rid;
	strcpy( rid.userid, msg->FindString("userid") );
	rid.reqID = requestID++;
	rid.type = RID_GET_INFO;
	pendingRequests.Add( rid );

	// and send it!
	SendSNACPacket( snac );
}

//---------------------------------------------------------------------

// Requests the info and profile for a member
void AIMNetThread::RequestAwayInfo( BMessage* msg ) {

	printf( "requesting away info... %s\n", msg->FindString("userid") );

	// Make the SNAC object
	SNAC_Object snac( 0x02, 0x05, 0, 0, requestID );
	snac.data << ToAIMWord(0x03)
			  << char(strlen(msg->FindString("userid")))
			  << msg->FindString("userid");
			  
	// add an object to the pending list
	userRequestID rid;
	strcpy( rid.userid, msg->FindString("userid") );
	rid.reqID = requestID++;
	rid.type = RID_GET_AWAY_MSG;
	pendingRequests.Add( rid );

	// and send it!
	SendSNACPacket( snac );
}

//---------------------------------------------------------------------

// Requests the info and profile for a member
void AIMNetThread::ReceivePersonInfo( SNAC_Object& snac ) {

	// vars
	BMessage* incomingInfo = new BMessage(BEAIM_UPDATE_INFO);
	unsigned short type, length;
	DataContainer temp;	
	short i = 0;
 
	// Grab the other person's info (from the beginning)
	DecodeBuddyStuff( snac.data, i, incomingInfo );
	
	// get the encoding value and the profile (if they are there)
	if( i < snac.data.length() ) {
		
		// There should be two TLV's in here...
		for( int j = 0; j < 2; ++j ) {
		
			// get the type and the length
			type = (unsigned short)GetWord( snac.data, i );
			length = (unsigned short)GetWord( snac.data, i+2 );
			i += 4;
			
			// get the data
			temp = snac.data.getrange( i, length );
			temp << char(0);
			i += length;

			// the profile encoding
			if( type == 0x01 ) {
				printf( "> Profile encoding:\n%s\n", temp.c_ptr() );
				incomingInfo->AddString( "encoding", temp.c_ptr() );
			}

			// the profile itself
			else if( type == 0x02 ) {
				printf( "> Profile:\n%s\n", temp.c_ptr() );
				incomingInfo->AddString( "profile", temp.c_ptr() );
			}
			
			// away encoding?
			else if( type == 0x03 ) {
				printf( "> Away Encoding:\n%s\n", temp.c_ptr() );
				incomingInfo->AddString( "encoding", temp.c_ptr() );
			}
			
			// away?
			else if( type == 0x04 ) {
				printf( "> Away Message:\n%s\n", temp.c_ptr() );
				incomingInfo->AddString( "away_message", temp.c_ptr() );
			}
		}	
	}
	
	// Find this request in the pending list and delete it
	userRequestID btemp;
	int curPos = 0;
	bool keepGoing = pendingRequests.First(btemp);
	while( keepGoing ) {
		if( snac.GetRequestID() == btemp.reqID ) {
			pendingRequests.Delete(curPos);
			printf( "--- Found and Nuked ReqID %d ---\n", btemp.reqID );
			if( btemp.type == RID_GET_AWAY_MSG )
				incomingInfo->what = BEAIM_UPDATE_AWAY_INFO;
			break;
		}
		keepGoing = pendingRequests.Next(btemp);
		++curPos;
	}

	// bombs away...
	incomingInfo->AddInt32( "wtype", (int32)USER_INFO_TYPE );
	SendAppMessage( incomingInfo );
}

//---------------------------------------------------------------------

void AIMNetThread::UpdateServerBuddyList( BMessage* msg ) {

	// Set up a SNAC object, with the correct fields to ADD (not remove) a person
	SNAC_Object snac( 0x03, 0x04, 0, 0, 0x1015 );
	
	// other vars we're gonna need (mostly to parse the list)
	int l = 1;
	char* i = (char*)msg->FindString("list");
	char* q = i;
	char name[50];
	
	if( msg->FindBool("add") )
		printf( "ADDING to server buddy list\n" );
	else
		printf( "REMOVING from server buddy list\n" );
	

	// are we adding to or removing from the server's buddy list?
	bool add = msg->FindBool("add");
	
	// the add command apparently has three sets of zeroes before the name list
	if( add )
		snac.data << ToAIMWord(0x00) << ToAIMWord(0x00) << ToAIMWord(0x00);

	// Go through and pull out all the names
	if( i ) {
		while( *i ) {
			if( *i == 9 ) {
				strncpy( name, q, l );
				name[l-1] = '\0';
				if( add ) snac.data << char(l-1) << name;
				else SendRemovePersonMessage( name, int(l-1) );
				printf( "(%d): %s\n", int(l-1), name );
				l = 0;
				q = i+1;
			}
			++i; ++l;
		}
		if( i != q ) {
			printf( "(%d): %s\n", int(l-1), q );
			if( add ) snac.data << char(l-1) << q;
			else SendRemovePersonMessage( q, int(l-1) );
		}
	}
	
	// if we are adding screen names, go ahead and send them
	if( add )
		SendSNACPacket( snac );
}

//---------------------------------------------------------------------

void AIMNetThread::SendRemovePersonMessage( char* name, int len ) {

	printf( "REMOVINK: %s\n", name );

	// Set up a SNAC object, with the correct fields to remove a person
	SNAC_Object snac( 0x03, 0x05, 0, 0, 0x1015 );
	snac.data << ToAIMWord(0x00) << ToAIMWord(0x00) << ToAIMWord(0x00);
	snac.data << char(len) << name;
	SendSNACPacket( snac );
}

//---------------------------------------------------------------------

void AIMNetThread::SearchByEmail( BMessage* msg ) {

	// calculate the next request ID
	emailSearchID = requestID++;

	// Set up a SNAC object, with the correct fields to search for an address
	SNAC_Object snac( 0x0a, 0x02, 0x00, 0x00, emailSearchID );
	snac.data << (char*)msg->FindString( "email" );
	SendSNACPacket( snac );
}

//---------------------------------------------------------------------

void AIMNetThread::ReceiveSearchResults( SNAC_Object& snac ) {

	DataContainer name;
	char actualName[50];	
	unsigned short length;
	BMessage* sndMessage;
	short i = 0;

	// is this the current search? If not, ignore it
	if( snac.GetRequestID() != emailSearchID )
		return;

	// go through and suck out all the search results
	while( (snac.data.length() - i) > 4 ) {

		// get 0x0001 (a constant), and the length of the screen name
		GetWord( snac.data, i );
		length = (unsigned short)GetWord( snac.data, i+2 );
		i += 4;
		
		printf( "SR---> SN length: %d chars\n", length );
		
		// get the screen name itself
		name = snac.data.getrange( i, length );
		strncpy( actualName, name.c_ptr(), length );
		actualName[length] = '\0';
		i += length;
		
		printf( "SR---> SN name: %s\n", name.c_ptr() );
		
		// send a message with this match
		sndMessage = new BMessage( BEAIM_EMAIL_SEARCH_RESULTS );
		sndMessage->AddString( "userid", actualName );
		SendAppMessage( sndMessage );
	}
}

//---------------------------------------------------------------------

void AIMNetThread::ReceiveFailedSearch( SNAC_Object& snac ) {

	// is this the current search? If not, ignore it
	if( snac.GetRequestID() != emailSearchID )
		return;
		
	// post a message saying that the search failed
	BMessage* failMessage = new BMessage( BEAIM_EMAIL_SEARCH_RESULTS );
	failMessage->AddBool( "failed", true );
	SendAppMessage( failMessage );	
}

//---------------------------------------------------------------------

void AIMNetThread::FailedPersonInfo( SNAC_Object& snac ) {

	printf( "Failed packet:\n" );

	// Find this request in the pending list and delete it
	userRequestID btemp;
	int curPos = 0;
	bool keepGoing = pendingRequests.First(btemp);
	long ctemp = snac.GetRequestID();
	while( keepGoing ) {
		if( btemp.reqID == (unsigned)ctemp ) {
			pendingRequests.Delete(curPos);
			break;
		}
		keepGoing = pendingRequests.Next(btemp);
		++curPos;
	}
	
	// OK... now fire off a failure message
	if( keepGoing ) {
		BMessage* incomingInfo = new BMessage(BEAIM_UPDATE_INFO);
		incomingInfo->AddInt32( "wtype", (int32)USER_INFO_TYPE );
		incomingInfo->AddString( "userid", btemp.userid );
		if( user->IsUserBlocked(btemp.userid) )
			incomingInfo->AddString( "profile", "<b>Error:</b> No information is available, because you have blocked this user." );
		else
			incomingInfo->AddString( "profile", "<b>Error:</b> This is an invalid screen name, or this user is not signed on." );
		SendAppMessage( incomingInfo );
	}
}

//---------------------------------------------------------------------

void AIMNetThread::SendArbitraryData( BMessage* msg ) {

	DataContainer data;
	ssize_t size = 0;
	char* chardata;
	
	// get that data outta there!
	msg->FindData( "data", B_RAW_TYPE, (const void**)(&chardata), &size );
	for( short i = 0; i < size; ++i )
		data << (unsigned char)chardata[i];

	// Send it!
	SendPacket( data, 0x02 );
}

//---------------------------------------------------------------------

void AIMNetThread::SendIdleMessage( bool nowIdle ) {

	SNAC_Object snac;
	snac.Update( 0x01, 0x11, 0x00, 0x00, 0x00 );

	// are we idle now?
	if( nowIdle ) {
		snac.data << ToAIMWord(0x00);
		snac.data << (unsigned char)(2);
		snac.data << (unsigned char)('Z');	
	}
	
	// nope... guess not
	else {		
		snac.data << ToAIMWord(0x00);
		snac.data << ToAIMWord(0x00);
	}
	
	// send it
	SendSNACPacket( snac );
}
		
//---------------------------------------------------------------------

void AIMNetThread::SendIdlePulse() {
	
}
		
//---------------------------------------------------------------------

void AIMNetThread::SetAwayStatus( BMessage* msg ) {

	BString awayMsg;
	printf( "SetAwayStatus: %s\n", msg->FindBool("away") ? "true" : "false" );
	
	// cheap hack to make sure the profile isn't length 1
	BString profile = user->Profile();
	if( profile.Length() == 1 )
		profile << BString(" ");
	
	// Make a SNAC object for the away packet and tack on some required data
	SNAC_Object snac;
	snac.Update( 0x02, 0x04, 0x00, 0x00, 0x02 );
	snac.data << ToTLV( 0x01, 0x1F, "text/aolrtf; charset=\"us-ascii\"" );
	
	// If there is a profile, add the whole TLV...
	// otherwise, send a profile of length 1, consisting of a single zero
	if( profile.Length() ) {
		snac.data << ToTLV( 0x02, profile.Length(), profile.LockBuffer(0) );
		profile.UnlockBuffer();
	} else
		snac.data << ToAIMWord(0x02) << ToAIMWord(0x01) << (unsigned char)(0);
	snac.data << ToTLV( 0x03, 0x1F, "text/aolrtf; charset=\"us-ascii\"" );
	
	// if we are setting an away message, tack on the length and message
	if( msg->FindBool("away") ) {
		awayMsg = user->CurrentAwayMessage();
		if( awayMsg.Length() == 0 )
			awayMsg = " ";
		printf( "away message: %s\n", awayMsg.String() );
		snac.data << ToTLV( 0x04, awayMsg.Length(), awayMsg.String() );
	}
	
	// if we're coming back, then just send a zero length with no message
	else snac.data << ToAIMWord( 0x04 ) << ToAIMWord( 0x00 );
	
	// send it
	SendSNACPacket( snac );
}

//---------------------------------------------------------------------

void AIMNetThread::ReceiveWarning( SNAC_Object& snac ) {

	char warnMessage[512];
	unsigned short wLevel, i=0;
	BMessage* sndMessage;
	DataContainer screenName;
	bool anon = false;
	char snLength;

	wLevel = (unsigned short)( rint( (double)GetWord(snac.data,i) / 10.0 ) );
	i += 2;
	
	printf( "New warning level: %u%%\n", wLevel );
	printf( "... came from " );
	
	// 'twas an anonymous warning
	if( i >= snac.data.length() ) {
		anon = true;
		printf( "Anonymous\n" );
	}
	
	// grab the length, and then the screen name
	if( !anon ) {
		snLength = snac.data[i++];
		screenName = DataContainer( snac.data.getrange(i,snLength) );
		i += snLength;

		screenName << char(0);
		printf( "%s\n", screenName.c_ptr() );
	}
	
	// is our warning level going DOWN? if so, return
	if( wLevel < user->WarningLevel() ) {
		user->SetWarningLevel( wLevel );
		return;
	}

	// make the "warned" message
	user->SetWarningLevel( wLevel );
	sprintf( warnMessage, "You were just warned by %s! Your warning level has been increased to %u%%.", 
			 anon ? "[Anonymous]" : screenName.c_ptr(), wLevel );
	sndMessage = new BMessage(BEAIM_DISPLAY_MESSAGE);
	sndMessage->AddBool( "warning", true );
	sndMessage->AddInt32( "sound", WS_WARNED );
	sndMessage->AddString( "button", "Behave!" );
	sndMessage->AddString( "message", warnMessage );
	PostAppMessage( sndMessage );
	
	sndMessage = new BMessage(BEAIM_GOT_WARNED);
	sndMessage->AddInt32( "warninglevel", (int32)wLevel );
	sndMessage->AddBool("noappsend", true);
	PostAppMessage( sndMessage );
	
	/* after that, comes a bunch of TLV's and stuff, but we're not gonna
	// worry about those for now. This info is for future reference.
	
	00 00 00 05 - 4 bytes - TLV count

	TLV 1. ?????
	  tag 00 01, len 00 02
	  value 00 10
	
	TLV 2. ????? 
	  tag 00 04, len 00 02
	  value 00 00
	
	TLV 3. ?????
	  tag 00 0F, len 00 04
	  value 00 00 05 AD
	
	TLV 4. offended person's onsince time
	  tag 00 02, len 00 04
	  value - 4 bytes
	
	TLV 5. offended person's membersince time
	  tag 00 02, len 00 04
	  value - 4 bytes*/
}

//---------------------------------------------------------------------

void AIMNetThread::FailedMessage( SNAC_Object& snac ) {

	char errorMessage[512];
	unsigned short targetReqID;
	bool found = false, kg;
	userRequestID rid;

	targetReqID = snac.GetRequestID();
	kg = pendingNRRequests.First(rid);
	while( kg ) {
		if( targetReqID == rid.reqID ) {
			found = true;
			break;
		}
		kg = pendingNRRequests.Next(rid);
	}
	
	// if we didn't find it, stay quiet for now
	if( !found )
		return;
	
	// aha! Found it! Send an error message
	printf( ">>> Failed Message: %s\n", rid.userid );
	if( rid.type == RID_SEND_MESSAGE )
		if( user->IsUserBlocked(rid.userid) )
			sprintf( errorMessage, "Your message couldn't be sent to %s, because you have blocked that user.", rid.userid );
		else
			sprintf( errorMessage, "For some reason, your message couldn't be sent to %s. Check and make sure that this user is actually online.", rid.userid );
	else if( rid.type == RID_WARN_EM )
		sprintf( errorMessage, "Your attempt to warn %s failed.", rid.userid );
	else return;
	BMessage* sndMessage = new BMessage(BEAIM_DISPLAY_MESSAGE);
	sndMessage->AddBool( "warning", true );
	sndMessage->AddString( "message", errorMessage );
	PostAppMessage( sndMessage );
}

//---------------------------------------------------------------------

void AIMNetThread::SendWarning( BMessage* msg ) {

	SNAC_Object sendMsg;

	// debug stuff
	char message[100];
	sprintf( message, "Warning: %s  [mode=%s]\n", msg->FindString("userid"),
			 msg->FindBool("anonymous") ? "anonymous" : "normal" );
	printf( message );
	
	// Make the SNAC object for the warning
	sendMsg.Update( 0x04, 0x08, 0x00, 0x00, requestID );
	
	// make the warning anonymous (or not)
	if( msg->FindBool("anonymous") )
		sendMsg.data << ToAIMWord(0x01);
	else
		sendMsg.data << ToAIMWord(0x00);

	// finally, tack on the screen name
	sendMsg.data << char(strlen((char*)msg->FindString("userid")))		// sn length
				 << (char*)msg->FindString("userid");					// sn

	// log it, so we know what happened in case the warning fails
	LogMessageReqID( BString((char*)msg->FindString("userid")), RID_WARN_EM );
	
	// Now that we've gone to all the trouble of making that, send it
	SendSNACPacket( sendMsg );
}

//---------------------------------------------------------------------

void AIMNetThread::SetUserBlockiness( BMessage* msg, bool all ) {

	BString theUser;
	theUser = all ? BString() : msg->FindString( "userid" );
	SNAC_Object sendMsg;
	
	printf( "Setting user blockiness for %s: %s\n", all ? "[all]" : theUser.String(), (all || msg->FindBool("block")) ? "yup" : "nope" );
	
	// set the correct SNAC info
	if( all || msg->FindBool("block") )
		sendMsg.Update( 0x09, 0x07, 0x00, 0x00, 0x00 );
	else
		sendMsg.Update( 0x09, 0x08, 0x00, 0x00, 0x00 );
		
	// finally, tack on the screen name(s)
	if( !all ) {
		sendMsg.data << char(theUser.Length())							// sn length
					 << (char*)theUser.String();						// sn
	} else {
		BString tempName;
		bool kg = user->GetNextBlockedUser( tempName, true );
		while( kg ) {
			sendMsg.data << char(tempName.Length())						// sn length
						 << (char*)tempName.String();					// sn
			kg = user->GetNextBlockedUser( tempName, false );
		}	
	}
				 
	// Now that we've gone to all the trouble of making that, send it
	SendSNACPacket( sendMsg );
}

//---------------------------------------------------------------------
