#include <Application.h>
#include <Box.h>
#include <Button.h>
#include <stdlib.h>
#include <StringView.h>
#include <Box.h>
#include <ctype.h>
#include <Button.h>
#include "AboutBox.h"
#include "MiscStuff.h"
#include "SKMode.h"
#include "constants.h"
#include "BitmapView.h"
#include "Globals.h"
#include "Say.h"

const uint32 SK_MODE_BUTTON = 'SKmb';

//=========================================================================

AboutWindow::AboutWindow( BRect frame )
	 				: BWindow(frame, "About BeAIM", B_TITLED_WINDOW,
	 				  B_NOT_ZOOMABLE | B_NOT_RESIZABLE )
{
	// Set up the view
	BRect aRect( Bounds() );
	genView = new AboutView( aRect );
	AddChild( genView );
}

//-------------------------------------------------------------------------

AboutWindow::~AboutWindow() {

}

//-------------------------------------------------------------------------

bool AboutWindow::QuitRequested()
{
	BMessage* clsMessage = new BMessage(BEAIM_SINGLE_WINDOW_CLOSED);
	clsMessage->AddInt32( "wtype", SW_ABOUT_BOX );
	PostAppMessage( clsMessage );
	return(true);
}

//-------------------------------------------------------------------------

void AboutWindow::MessageReceived(BMessage* message)
{
	switch(message->what)
	{
		case BEAIM_SKMODE_CHANGED:
			genView->verSV->SetText( SKReplaceTerm(versionString).String() );
			genView->bySV->SetText( SKReplaceTerm("by Greg Nichols").String() );
			genView->majorSV->SetText( SKReplaceTerm("Major contributors:").String() );
			break;
	
		case SK_MODE_BUTTON:
			ToggleSKMode();
			break;
	
		default:
			BWindow::MessageReceived(message);
	}
}

//-------------------------------------------------------------------------

// the cancel function
void AboutWindow::DispatchMessage( BMessage* msg, BHandler* handler ) {

	char c;

	// if it's a cancel key, post a B_CANCEL message
	if( msg->what == B_KEY_DOWN ) {

		if( isalpha(c = msg->FindString("bytes")[0]) ) {
			uint32 mods = modifiers();
			if( genView->SKButton->IsHidden() && !(mods&B_COMMAND_KEY || mods&B_CONTROL_KEY || mods&B_MENU_KEY || mods&B_SHIFT_KEY) ) {
				if( SKLetterTyped(tolower(c)) )
					genView->SKButton->Show();
				return;
			}
		}

		if( msg->FindString("bytes")[0] == B_ESCAPE ) {
			PostMessage( new BMessage(B_QUIT_REQUESTED) );
			return;
		}
	}
	
	// our work here is done... dispatch normally
	BWindow::DispatchMessage( msg, handler );
}

//=========================================================================

AboutView::AboutView( BRect rect )
	   	   : BView(rect, "generic_input_view", B_FOLLOW_ALL_SIDES, B_WILL_DRAW)
{
	BRect sframe;
	BStringView* tempSV;
	LinkStringView* tempSV2;
	BBitmap* mainlogo = NULL;
	SetViewColor( 216, 216, 216 );
	
	// Set up the views
	GetBitmapFromResources( mainlogo, 42 );
	BRect bframe = Bounds();
	bframe.bottom = 77;
	BitmapView* bview = new BitmapView( bframe, mainlogo );
	AddChild( bview );
	
	// BeAIM version
	sframe = BRect( 0, 70, 198, 85 );
	verSV = new BStringView( sframe, "", SKReplaceTerm(versionString).String() );
	verSV->SetFont(be_plain_font);
	verSV->SetFontSize(9);
	verSV->SetAlignment(B_ALIGN_RIGHT);
	AddChild( verSV );

	// main credits (that is, me!)
	sframe = BRect( 7, 84, 198, 109 );
	bySV = new BStringView( sframe, "", SKReplaceTerm("by Greg Nichols").String() );
	bySV->SetFont(be_bold_font);
	bySV->SetFontSize(15);
	AddChild( bySV );	
	
	// other code contributors title
	sframe = BRect( 7, 110, 198, 130 );
	majorSV = new BStringView( sframe, "", SKReplaceTerm("Major contributors:").String() );
	majorSV->SetFont(be_bold_font);
	majorSV->SetFontSize(12);
	AddChild( majorSV );	
	
	// major code contributors text
	sframe = BRect( 13, 124, 198, 144 );
	tempSV = new BStringView( sframe, "", "Sean Heber, Justin Mierta" );
	tempSV->SetFont(be_plain_font);
	tempSV->SetFontSize(11);
	AddChild( tempSV );	
	
	// home page link
	sframe = BRect( 7, 155, 140, 175 );
	tempSV2 = new LinkStringView( sframe, "", "BeAIM Home Page" );
	tempSV2->SetURLMode( "http://www.fifthace.com/beaim/", true );
	tempSV2->SetFont(be_plain_font);
	tempSV2->SetFontSize(11);
	AddChild( tempSV2 );	
	
	// BeBits app page link
	sframe = BRect( 7, 175, 140, 195 );
	tempSV2 = new LinkStringView( sframe, "", "BeAIM page on BeBits" );
	tempSV2->SetURLMode( "http://www.bebits.com/app/1", true );
	tempSV2->SetFont(be_plain_font);
	tempSV2->SetFontSize(11);
	AddChild( tempSV2 );	
	
	// BeBits app page link
	sframe = BRect( 7, 195, 140, 215 );
	tempSV2 = new LinkStringView( sframe, "", "greg@fifthace.com" );
	tempSV2->SetURLMode( "mailto:greg@fifthace.com", false );
	tempSV2->SetFont(be_plain_font);
	tempSV2->SetFontSize(11);
	AddChild( tempSV2 );	
	
	// Set up "Get it at BeBits" logo
	GetBitmapFromResources( mainlogo, 8294 );
	bframe = BRect(0,0,49,62);
	bframe.OffsetTo( 145, 155 );
	bview = new BitmapView( bframe, mainlogo );
	bview->SetURLMode( "http://www.bebits.com/app/1", true );
	AddChild( bview ); 
	
	// the groovy line thang
	BBox* divider = new BBox(BRect(8,227,193,229),"divider", B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW, B_FANCY_BORDER);
	AddChild( divider );
	
	// the SK Mode button
	SKButton = new BButton(BRect(7,240,112,260), "skmode", GetSKButton().String(), new BMessage(SK_MODE_BUTTON));
	SKButton->Hide();
	AddChild( SKButton );
	SKEnable(true);
	
	// and finally, the OK button
	OKButton = new BButton(BRect(132,240,192,260), "loginbutton", "Close", new BMessage(B_QUIT_REQUESTED)); 
	OKButton->MakeDefault(true);
	AddChild( OKButton );
}

//=========================================================================

LinkStringView::LinkStringView( BRect frame, const char* name, const char* text )
			: BStringView( frame, name, text )
{
	urlMode = 0;
}

//-------------------------------------------------------------------------

void LinkStringView::Draw(BRect updateRect) {
	SetHighColor(0,0,255);
	BStringView::Draw(updateRect);
}

//-------------------------------------------------------------------------

void LinkStringView::MouseDown(BPoint point) {
	
	status_t result;

	if( urlMode ) {
	
		if( urlMode == 1 ) {
			char* link = url.LockBuffer(0);
			result = be_roster->Launch( "text/html", 1, &link );
			url.UnlockBuffer();
			if( (result != B_NO_ERROR) && (result != B_ALREADY_RUNNING) ) {
				Say( "There is no installed handler for 'text/html'." );
			}
		}
		
		if( urlMode == 2 ) {
			char* link = url.LockBuffer(0);
			result = be_roster->Launch( "text/x-email", 1, &link );
			url.UnlockBuffer();
			if( (result != B_NO_ERROR) && (result != B_ALREADY_RUNNING) ) {
				Say( "There is no installed handler for 'text/x-email'." );
			}
		}
	}
	
	BView::MouseDown(point);
}

//-------------------------------------------------------------------------

void LinkStringView::SetURLMode( BString u, bool web ) {
	urlMode = web ? 1 : 2;
	url = u;
}

//-------------------------------------------------------------------------


