// BeAIM - BuddyList.cpp
// The Buddy List module

#include <Application.h>
#include <View.h>
#include <TabView.h>
#include <MenuBar.h>
#include <MenuItem.h>	
#include <Alert.h>
#include <ScrollView.h>
#include <SupportKit.h>
#include <stdlib.h>
#include <stdio.h>
#include "BuddyList.h"
#include "MiscStuff.h"
#include "PeopleEdit.h"
#include "Say.h"
#include "Globals.h"
#include "SKMode.h"

//=========================================================================

const int LOGO_HEIGHT = 40;
const int STATUS_HEIGHT = 13;

//=========================================================================

BuddyList::BuddyList(BRect frame)
	     : BWindow(frame, "Buddy List", B_DOCUMENT_WINDOW, B_NOT_ZOOMABLE, B_ALL_WORKSPACES )
{
	BRect r; 
	BBitmap* smallLogo = NULL;
	PeopleTab *onlinetab, *setuptab;
	BMenuItem* menuItem;
	BMessage* tmpMessage;
	r = Bounds();
	
	// Add the menu bar stuff
	menubar = new BMenuBar(r, "menu_bar");
	beMenu = new BMenu("BeAIM");
	beMenu->AddItem(new BMenuItem("Preferences" B_UTF8_ELLIPSIS, new BMessage(BEAIM_OPEN_PREFS) ));
	tmpMessage = new BMessage(BEAIM_OPEN_SINGLE_WINDOW);
	tmpMessage->AddInt32( "wtype", SW_PROFILE_EDITOR );
	beMenu->AddItem(menuItem=new BMenuItem("Edit Profile" B_UTF8_ELLIPSIS, tmpMessage ));
	menuItem->SetTarget(APP_SIGNATURE);
	beMenu->AddSeparatorItem();
	beMenu->AddItem(menuItem=new BMenuItem("About BeAIM" B_UTF8_ELLIPSIS, new BMessage(B_ABOUT_REQUESTED) ));
	menuItem->SetTarget(APP_SIGNATURE);
	beMenu->AddSeparatorItem();
	beMenu->AddItem(menuItem=new BMenuItem("Logout", new BMessage(BEAIM_LOGOUT) ));
	menuItem->SetTarget(APP_SIGNATURE);
	beMenu->AddItem(new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED) ));
	//beMenu->AddItem(new BMenuItem("Setup Away messages" B_UTF8_ELLIPSIS, NULL ));

	peopleMenu = new BMenu("People");
	peopleMenu->AddItem(new BMenuItem("Contact User" B_UTF8_ELLIPSIS, new BMessage(BEAIM_SEND_NEW_MESSAGE) ));
	peopleMenu->AddItem(new BMenuItem("Get User Info" B_UTF8_ELLIPSIS, new BMessage(BEAIM_GET_PERSON_INFO) ));
	peopleMenu->AddItem(new BMenuItem("Search by email address" B_UTF8_ELLIPSIS, new BMessage(BEAIM_SEARCH_BY_EMAIL) ));
	peopleMenu->AddSeparatorItem();
	tmpMessage = new BMessage(BEAIM_OPEN_SINGLE_WINDOW);
	tmpMessage->AddInt32( "wtype", SW_BLOCKLIST_EDITOR );
	peopleMenu->AddItem(menuItem=new BMenuItem("Block List" B_UTF8_ELLIPSIS, tmpMessage) );
	menuItem->SetTarget(APP_SIGNATURE);
	tmpMessage = new BMessage(BEAIM_PERHAPS_IMPORT);
	peopleMenu->AddItem(menuItem=new BMenuItem("Import Buddy List" B_UTF8_ELLIPSIS, tmpMessage) );
	
	statMenu = new BMenu( "Away" );
	BMessage* awMsg = new BMessage(BEAIM_GOING_AWAY);
	awMsg->AddBool( "away", false );
	availItem = new BMenuItem("Available", awMsg, 'A');
	availItem->SetMarked(true);
	statMenu->AddItem( availItem );
	statMenu->AddSeparatorItem();
	
	// Load the away messages from the UserManager
	LoadAwayMessages();
	
	// Add the 'custom' item
	BMessage* custMessage = new BMessage(BEAIM_OPEN_SINGLE_WINDOW);
	custMessage->AddInt32( "wtype", SW_CUSTOM_AWAY_EDITOR );
	customItem = new BMenuItem( "Custom Away Message" B_UTF8_ELLIPSIS, custMessage );
	customItem->SetTarget(APP_SIGNATURE);
	statMenu->AddItem( customItem );
	
	// add the editor item
	awMsg = new BMessage(BEAIM_OPEN_SINGLE_WINDOW);
	awMsg->AddInt32( "wtype", SW_AWAY_EDITOR );
	statMenu->AddSeparatorItem();
	statMenu->AddItem(menuItem=new BMenuItem("Edit Away Messages" B_UTF8_ELLIPSIS, awMsg));
	menuItem->SetTarget(APP_SIGNATURE);
	menubar->AddItem(beMenu);
	menubar->AddItem(peopleMenu);
	menubar->AddItem(statMenu);
	AddChild(menubar);	

	// Add the logo thinger
	r.top = menubar->Bounds().bottom + 1.0;
	r.bottom = r.top + LOGO_HEIGHT;
	GetBitmapFromResources( smallLogo, 942 );
	logoView = new BitmapView( r, smallLogo );
	AddChild( logoView );
	
	// add the (currently empty) infoView
	r.top = r.bottom - 1;
	iView = new BuddyListInfoView( r );
	AddChild( iView );

	// Add the tabview control
	r = Bounds();
	r.top = menubar->Bounds().bottom +logoView->Bounds().bottom + 1.0;
	r.bottom -= (STATUS_HEIGHT+1);
	tabView = new BTabView(r, "tab_view"); 
	tabView->SetViewColor(216,216,216);
	AddChild( tabView );	
	tabView->SetTabWidth( B_WIDTH_FROM_WIDEST );  		
	
	// Add the tabs themselves
	r = tabView->Bounds();
	r.InsetBy(1,1); 
	r.bottom -= tabView->TabHeight(); 
	onlinetab = new PeopleTab(0, this);
	tabView->AddTab(onlineview = new BuddyListView(r), onlinetab);
	onlinetab->SetLabel("Online");
	onlinetab->SetControlView( onlineview );
	setuptab = new PeopleTab(1, this);
	tabView->AddTab(setupview = new BuddyListSetupView(r), setuptab);
	setuptab->SetLabel("List Setup");
	setuptab->SetControlView( setupview );
	tabView->Select( 0 );
	
	// add the kewl name view
	r = Bounds();
	r.top = r.bottom - STATUS_HEIGHT;
	r.right -= 14;
	myName = new NameStatusView( r );
	myName->SetName( user->User() );
	myName->SetWarningLevel( user->WarningLevel() );
	AddChild( myName );
	
	// set up 1-second pulses, to handle the buddy list change updates
	SetPulseRate( 1000000 );
	
	// add shortcuts to handle prev and next commands from the buddylist
	AddShortcut( ',', 0, new BMessage(BEAIM_PREV_CHAT_WINDOW) );
	AddShortcut( '.', 0, new BMessage(BEAIM_NEXT_CHAT_WINDOW) );
	
	// add the "secret" SendData shortcut
	AddShortcut( '\\', B_SHIFT_KEY, new BMessage(BEAIM_SEND_ARBITRARY_DATA) );
	
	// load the prefs
	iView->doIdleTime = false;
	LoadPrefs();
}

//-------------------------------------------------------------------------

BuddyList::~BuddyList()
{
}

//-------------------------------------------------------------------------

bool BuddyList::QuitRequested()
{
	BMessage* curMsg;

	// save the window position
	BRect winPos = Frame();
	user->WriteInt32( "buddylist.x1", int32(winPos.LeftTop().x) );
	user->WriteInt32( "buddylist.y1", int32(winPos.LeftTop().y) );
	user->WriteInt32( "buddylist.x2", int32(winPos.RightBottom().x) );
	user->WriteInt32( "buddylist.y2", int32(winPos.RightBottom().y) );

	curMsg = CurrentMessage();
	if( !curMsg->HasBool("dontkill") )
		be_app->PostMessage(B_QUIT_REQUESTED);
	return(true);
}

//-------------------------------------------------------------------------

void BuddyList::LoadAwayMessages() {

	BMenuItem* tempItem;
	BMessage* awMsg;
	awayStruct aw;
	char shortcut;
	int i = 0;
	bool kg;
	
	// first, clear out everything in the existing menu
	for( unsigned i = 0; i < awayMenuItems.Count(); ++i ) {
		tempItem = awayMenuItems[i];
		statMenu->RemoveItem(tempItem);
		delete tempItem;
	}
	awayMenuItems.Clear();
	
	// Get the first of the new away messages...
	kg = user->GetFirstAwayMessage(aw);
	
	// special case: are there no messages defined?
	if( !kg ) {
		BMenuItem* noMsgs = new BMenuItem( "No Away Messages Defined", NULL );
		noMsgs->SetEnabled(false);
		statMenu->AddItem( noMsgs, 2 );
		awayMenuItems.Add( noMsgs );
	}
	
	// OK, there are some. Add 'em.
	while( kg ) {
	
		shortcut = 0;
		if( i < 9 )
			shortcut = '1' + i;
	
		awMsg = new BMessage(BEAIM_GOING_AWAY);
		tempItem = new BMenuItem( aw.name.String(), awMsg, shortcut );
		awMsg->AddPointer( "menuitem", (void*)tempItem );
		statMenu->AddItem( tempItem, i+2 );
		awayMenuItems.Add( tempItem );
		if( user->AwayMode() == AM_STANDARD_MESSAGE ) {
			if( user->CurrentAwayMessageName() == aw.name.String() )
				tempItem->SetMarked(true);
		}
		kg = user->GetNextAwayMessage(aw);
		i++;
	}
}

//-------------------------------------------------------------------------

void BuddyList::GoAway( BMessage* msg ) {

	BMenuItem* markedItem;
	BMenuItem* menuItem;
	
	// get the needed info
	msg->FindPointer( "menuitem", (void**)&menuItem );
	bool away = !msg->HasBool("away");
	if( away )
		msg->FindPointer( "menuitem", (void**)&menuItem );

	// Find the currently marked item, and unmark it (if needed)
	markedItem = statMenu->FindMarked();
	availItem->SetMarked(!away);
	if( away && markedItem && markedItem == menuItem )
		return;
	if( markedItem )
		markedItem->SetMarked(false);
	if( away )
		menuItem->SetMarked(true);
	
	// Are we going away or coming back?
	if( away )
		user->SetAwayMode( AM_STANDARD_MESSAGE, BString(menuItem->Label()) );
	else
		user->SetAwayMode( AM_NOT_AWAY );
}

//-------------------------------------------------------------------------

void BuddyList::MessageReceived( BMessage* message ) {

	// Some variables needed for various messages
	BMessenger messenger(APP_SIGNATURE);
	BOutlineListView* source = NULL;
	BListItem* currentItem = NULL;
	PeopleItem* currentBuddyItem = NULL;	
	BString ScreenName;
	BMessage* sendMessage;
	int32 Selected, status;

	switch(message->what) {

		// The user has double-clicked on a buddy. Send a message to open up an IM window.
		case BEAIM_BUDDY_INVOKED:
			message->FindPointer( "source", (void**)&source );
			Selected = source->CurrentSelection();
			currentItem = source->ItemAt( Selected );
			if( (currentBuddyItem = dynamic_cast<PeopleItem*>(currentItem)) == 0 )
				return;
			ScreenName = currentBuddyItem->UserName();
			sendMessage = new BMessage( BEAIM_OPEN_USER_WINDOW );
			sendMessage->AddInt32( "wtype", (int32)USER_MESSAGE_TYPE );
			sendMessage->AddString( "userid", ScreenName.String() );
			messenger.SendMessage( sendMessage );
			break;

		// This whole thing is kind of hackish. I want to redo it later.
		case BEAIM_ONCOMING_BUDDY:
			if( unsigned(message->FindInt32("idletime")) > 10 )
				status = BS_IDLE;
			else status = BS_ACTIVE;
			if( bool(message->FindBool("away")) == true )
				status |= BS_AWAY;

			// Set the status... if this is a change it won't fail and we won't need to bother adding				
			list->SetPersonStatus( (char*)message->FindString("userid"), status,
									message->FindInt32("warninglevel") );
			break;
			
		case BEAIM_OFFGOING_BUDDY:
			list->SetPersonStatus( (char*)message->FindString("userid"), BS_OFFLINE );
			break;
			
		case BEAIM_EDIT_BUDDY:
			setupview->EditItem();
			break;

		case BEAIM_OPEN_PREFS: 
			sendMessage = new BMessage(BEAIM_OPEN_SINGLE_WINDOW);
			sendMessage->AddInt32( "wtype", SW_PREFERENCES );
			sendMessage->AddBool( "globalonly", false );
			PostAppMessage( sendMessage );
			break;

		case BEAIM_RELOAD_PREF_SETTINGS:
			LoadPrefs();
			break;
			
		case BEAIM_DELETE_PERSONORGROUP:
			setupview->RemoveItem();
			break;
			
		case BEAIM_TRY_ADD_PERSON:
			setupview->AddItem( ET_ADDPERSON );
			break;
			
		case BEAIM_TRY_ADD_GROUP:
			setupview->AddItem( ET_ADDGROUP );
			break;
			
		case BEAIM_SKMODE_CHANGED:
			statMenu->Superitem()->SetLabel( SKReplaceTerm("Away").String() );
			peopleMenu->Superitem()->SetLabel( SKReplaceTerm("People").String() );
			beMenu->Superitem()->SetLabel( SKReplaceTerm("BeAIM").String() );
			tabView->TabAt(0)->SetLabel( SKReplaceTerm("Online").String() );
			tabView->TabAt(1)->SetLabel( SKReplaceTerm("List Setup").String() );
			break;
			
		case BEAIM_ADD_PERSON:
			list->AddPerson( (char*)message->FindString( "newname"),
							 (char*)message->FindString("group"),
							 (bool)message->FindBool("commitnow") );
			break;
		
		case BEAIM_ADD_GROUP:
			list->AddGroup( (char*)message->FindString( "newname" ) );
			break;		
		
		case BEAIM_CHANGE_PERSON:
			list->ChangePerson( (char*)message->FindString("oldname"),
								(char*)message->FindString("newname"),
								(char*)message->FindString("group") );
			break;
		
		case BEAIM_CHANGE_GROUP:
			list->ChangeGroup( (char*)message->FindString("oldname"),
							   (char*)message->FindString("newname") );
			break;

		case BEAIM_SEND_ARBITRARY_DATA:
		case BEAIM_PREV_CHAT_WINDOW:
		case BEAIM_NEXT_CHAT_WINDOW:
			be_app->PostMessage( message );
			break;
			
		case BEAIM_SEND_NEW_MESSAGE:
			sendMessage = new BMessage(BEAIM_OPEN_USER_WINDOW);
			sendMessage->AddInt32( "wtype", (int32)USER_MESSAGE_TYPE );
			windows->OpenInputWindow( "Contact User", "Screen Name:", sendMessage, this, 
									  NULL, DISPLAY_NAME_MAX, true, "userid" );
			break;
			
		case BEAIM_SEARCH_BY_EMAIL:
			//sendMessage = new BMessage(BEAIM_OPEN_EMAIL_SEARCH);
			sendMessage = new BMessage(BEAIM_OPEN_SINGLE_WINDOW);
			sendMessage->AddInt32( "wtype", SW_EMAIL_RESULTS );
			windows->OpenInputWindow( "Search for User", "email address:", sendMessage, this, 
									  NULL, 100, true, "email" );
			break;
			
		case BEAIM_GET_PERSON_INFO:
			sendMessage = new BMessage(BEAIM_OPEN_USER_WINDOW);
			sendMessage->AddInt32( "wtype", (int32)USER_INFO_TYPE );
			sendMessage->AddPointer( "poswindow", this );
			windows->OpenInputWindow( "Get User Info", "Screen Name:", sendMessage, this, 
									  NULL, DISPLAY_NAME_MAX, true, "userid" );		
			break;
			
		case BEAIM_LOAD_AWAY_MENU:
			LoadAwayMessages();
			break;
			
		case BEAIM_GOING_AWAY:
			GoAway( message );
			break;
			
		case BEAIM_AWAY_STATUS_CHANGED:
			if( message->HasBool("custom") ) {
				BMenuItem* marked = statMenu->FindMarked();
				if( marked )
					marked->SetMarked(false);
				customItem->SetMarked(true);
			}
			myName->SetAway( message->FindBool("away") );
			break;
			
		case BEAIM_SWITCH_TO_BLIST_TAB:
			tabView->Select( 1 );
			break;
			
		case BEAIM_PERHAPS_IMPORT:
			tabView->Select( 1 );
			sendMessage = new BMessage(BEAIM_OPEN_SINGLE_WINDOW);
			sendMessage->AddInt32("wtype", SW_BUDDYLIST_IMPORTER);
			PostAppMessage( sendMessage );
			break;
			
		case BEAIM_GOT_WARNED:
			if( message->HasInt32("warninglevel") )
				myName->SetWarningLevel( (unsigned short)message->FindInt32("warninglevel") );
			break;
			
		default:
			BWindow::MessageReceived(message);
			break;
	}
}

//-------------------------------------------------------------------------

void BuddyList::LoadPrefs() {

	// load and implement the "all workspaces" setting
	bool allWorkspaces = user->ReadBool("BuddyListAllWorkspaces", true);
	if( allWorkspaces )
		SetWorkspaces( B_ALL_WORKSPACES );
	else
		SetWorkspaces( B_CURRENT_WORKSPACE );
		
	// load and implement the idle time setting
	bool doIdle = user->ReadBool("ReportIdleTime", true);
	iView->doIdleTime = doIdle;
	if( doIdle )
		PostAppMessage( new BMessage(BEAIM_NOW_ACTIVE) );
}

//=========================================================================

BuddyListInfoView::BuddyListInfoView(BRect rect)
	   	   : BView(rect, "BuddyList", B_FOLLOW_LEFT_RIGHT, B_WILL_DRAW | B_PULSE_NEEDED )
{
	SetFont(be_bold_font);
	SetFontSize(24);
	SetViewColor( 216, 216, 216 );
	
	// reset the idle counter
	idleCount = 0;
}

//-------------------------------------------------------------------------

// update the change list... pulse is used on this particular view because it is visible
//   whenever the buddylist is visible... pulse stops occasionally for the others
void BuddyListInfoView::Pulse() {

	// make any changes that need to be done in the list
	list->UpdateChangeList();
	
	// do the idle time stuff
	if( doIdleTime )
		DoIdleStuff();
}

//-------------------------------------------------------------------------

void BuddyListInfoView::DoIdleStuff() {

	bigtime_t idle;
	
	// get the idle time
	idle = idle_time();
	
	// bigger than 10 minutes? (600 seconds)
	if( idle > 600000000 ) {

		// did we just now get idle? If so, send an idle message to AIM
		if( !idleCount ) {
			BMessage* idleMsg = new BMessage(BEAIM_NOW_IDLE);
			PostAppMessage(idleMsg);
			printf( "Just idled!\n" );
		}

		// every 10 minutes, fire off an idle event to keep AIM happy (required?)
		if( ++idleCount >= 601 ) {

			// go ahead, fire one off
			BMessage* pulseMsg = new BMessage(BEAIM_SEND_IDLE_PULSE);
			PostAppMessage(pulseMsg);
			printf( "porko!\n" );
			
			// reset the idle counter to 1, not zero...
			//  zero is reserved for not being idle
			idleCount = 1;
		}
	}
	
	// did we just get "un-idle"?
	else {
	
		// if idleCount != 0, then we were idle before
		if( idleCount ) {

			// send out a non-idle message
			BMessage* activeMsg = new BMessage(BEAIM_NOW_ACTIVE);
			PostAppMessage(activeMsg);

			// set it to be non-idle
			idleCount = 0;
		}
	}		
}


//=========================================================================

BuddyListView::BuddyListView(BRect rect)
	   	   : BView(rect, "BuddyList", B_FOLLOW_ALL, B_WILL_DRAW | B_PULSE_NEEDED )
{
	SetViewColor( 216, 216, 216 );
	BFont baseFont; 
	baseFont.SetFamilyAndStyle( "Swis721 BT", "Roman" );
	baseFont.SetSize( 12.0 );
	
	// Add the outline control
	BRect r = Bounds();
	r.InsetBy( 1.0, 1.0 );
	online = new ClickableListView( r, "online_list", B_SINGLE_SELECTION_LIST, B_FOLLOW_ALL );
	online->SetFont( &baseFont );
	online->SetInvocationMessage( new BMessage(BEAIM_BUDDY_INVOKED) );	
	
	// set up the scroller
	scroller = new BScrollView( "scroller2", online, B_FOLLOW_ALL, 0, false, true, B_NO_BORDER );
	AddChild( scroller );
	
	// tell the list that the display list is there
	list->SetDisplayList( online );
}

//-------------------------------------------------------------------------

void BuddyListView::Draw( BRect )
{
	BRect r = Bounds();
	SetHighColor( 156, 154, 156 );		// dark grey
	StrokeRect( r );
}

//-------------------------------------------------------------------------

void BuddyListView::DoResizeStuff( float width, float height ) {

	ResizeTo( width, height );
	
	// Ugh... resizing the scroller stuff is a pain!
	online->ResizeTo( width - B_V_SCROLL_BAR_WIDTH - 2, height - 2 );
	scroller->ResizeTo( width - 2, height - 2 );
	scroller->ScrollBar(B_VERTICAL)->MoveTo( width - B_V_SCROLL_BAR_WIDTH - 1, -1 );
	scroller->ScrollBar(B_VERTICAL)->ResizeTo( B_V_SCROLL_BAR_WIDTH, height );
	scroller->ScrollBar(B_VERTICAL)->SetValue(0);
}

//=========================================================================

ClickableListView::ClickableListView( BRect frame, char* name, list_view_type type, uint32 flags )
				  					: BOutlineListView( frame, name, type, flags )
{
	userPopup = new BPopUpMenu( "userpopup", false );
	userPopup->AddItem( new BMenuItem( "Contact" B_UTF8_ELLIPSIS, NULL ) );
	userPopup->AddItem( new BMenuItem( "Get Info" B_UTF8_ELLIPSIS, NULL ) );
}

//-------------------------------------------------------------------------

ClickableListView::~ClickableListView() {
	delete userPopup;
}

//-------------------------------------------------------------------------

void ClickableListView::MouseDown( BPoint cursor ) {

	uint32 buttons;
	int32 clickIndex;
	PeopleItem* clickItem;
	BMessage* popupMsg;
	BMenuItem* selected;
	int32 option;

	// get the mouse position
	GetMouse( &cursor, &buttons );
		
	// it was a right mouse button click, popup the menu
	if( buttons & B_SECONDARY_MOUSE_BUTTON ) {
		
		// get the index of the item the user clicked on, and select it
		clickIndex = IndexOf(cursor);
		if( clickIndex < 0 )			// not an item?
			return;
		Select(clickIndex);

		// get a pointer to the item, and check to see if it's a person or not
		clickItem = dynamic_cast<PeopleItem*>( ItemAt(clickIndex) );
		if( !clickItem )
			return;

		// it's all good... display the popup menu
		ConvertToScreen( &cursor );
		selected = userPopup->Go( cursor );

		// if the user selected something, take action based on the choice
		if( selected ) {
			option = userPopup->IndexOf(selected);
			popupMsg = new BMessage(BEAIM_OPEN_USER_WINDOW);
			popupMsg->AddString( "userid", clickItem->UserName().String() );
			popupMsg->AddPointer( "poswindow", Window() );
			switch( option ) {
				case 0: popupMsg->AddInt32( "wtype", (int32)USER_MESSAGE_TYPE );
					break;
				case 1: popupMsg->AddInt32( "wtype", (int32)USER_INFO_TYPE );
					break;			
			}
			PostAppMessage( popupMsg );			
		}
	} else
		BOutlineListView::MouseDown( cursor );
}

//=========================================================================

BuddyListSetupView::BuddyListSetupView(BRect rect)
	   	   : BView(rect, "BuddyList", B_FOLLOW_ALL, B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE )
{
	SetFont(be_bold_font);
	SetViewColor( 216, 216, 216 );
	BFont baseFont;
	baseFont.SetFamilyAndStyle( "Swis721 BT", "Roman" );
	baseFont.SetSize( 12.0 );	

	// Make the outline control
	BRect r = Bounds();
	r.InsetBy( 1.0, 1.0 );
	r.bottom -= 26;
	r.right -= B_V_SCROLL_BAR_WIDTH + 2;
	buddies = new BOutlineListView( r, "buddies_setup_list", B_SINGLE_SELECTION_LIST, B_FOLLOW_ALL );
	buddies->SetFont( &baseFont );

	// Make the scrollview control and attach it
	scroller = new BScrollView( "scroller1", buddies, B_FOLLOW_ALL, 0, false, true, B_NO_BORDER );
	buddies->SetInvocationMessage( new BMessage(BEAIM_EDIT_BUDDY) );
	AddChild( scroller );	

	// Add some picture buttons
	BRect btnRect( r.right - 27, r.bottom + 4, r.right-1, r.bottom + 25 );
	BPicture* blank = new BPicture;
	BMessage* nMsg = new BMessage(BEAIM_OPEN_SINGLE_WINDOW);
	nMsg->AddInt32("wtype", SW_BUDDYLIST_IMPORTER );
	importBtn = new BPictureButton( btnRect, "import_list", blank, blank, nMsg,
									B_ONE_STATE_BUTTON, B_FOLLOW_RIGHT | B_FOLLOW_BOTTOM );				
	importBtn->SetTarget( APP_SIGNATURE );
	btnRect.left = 1;
	btnRect.right = btnRect.left + 26;
	addGroupBtn = new BPictureButton( btnRect, "add_group", blank, blank, new BMessage(BEAIM_TRY_ADD_GROUP),
									B_ONE_STATE_BUTTON, B_FOLLOW_LEFT | B_FOLLOW_BOTTOM );			
	addGroupBtn->SetTarget( this );	
	btnRect.left += 30;
	btnRect.right += 30;
	addBuddyBtn = new BPictureButton( btnRect, "add_person", blank, blank, new BMessage(BEAIM_TRY_ADD_PERSON),
									B_ONE_STATE_BUTTON, B_FOLLOW_LEFT | B_FOLLOW_BOTTOM );
	addBuddyBtn->SetTarget( this );	
	btnRect.left += 30;
	btnRect.right += 30;
	removeBtn = new BPictureButton( btnRect, "remove_person", blank, blank, new BMessage(BEAIM_DELETE_PERSONORGROUP),
									B_ONE_STATE_BUTTON, B_FOLLOW_LEFT | B_FOLLOW_BOTTOM );
	removeBtn->SetTarget( this );
	AddChild( addGroupBtn );
	AddChild( addBuddyBtn );
	AddChild( removeBtn );
	AddChild( importBtn );
		
	// tell the list that the setup list is there
	list->SetSetupList( buddies );
	
	buttonsDone = false;
	delete blank;
}

//-------------------------------------------------------------------------

// Set up the button glyphs
void BuddyListSetupView::AttachedToWindow() {

	if( !buttonsDone ) {
	
		BPicture* btnPict = NULL; 
		
		MakeButtonPicture( btnPict, this, 101 );
		addBuddyBtn->SetEnabledOn( btnPict );
		
		MakeButtonPicture( btnPict, this, 100 );
		addBuddyBtn->SetEnabledOff( btnPict );
		
		MakeButtonPicture( btnPict, this, 103 );
		addGroupBtn->SetEnabledOn( btnPict );
		
		MakeButtonPicture( btnPict, this, 102 );
		addGroupBtn->SetEnabledOff( btnPict );
		
		MakeButtonPicture( btnPict, this, 105 );
		removeBtn->SetEnabledOn( btnPict );
		
		MakeButtonPicture( btnPict, this, 104 );
		removeBtn->SetEnabledOff( btnPict );
		
		MakeButtonPicture( btnPict, this, 107 );
		importBtn->SetEnabledOn( btnPict );
		
		MakeButtonPicture( btnPict, this, 106 );
		importBtn->SetEnabledOff( btnPict );					
		
		buttonsDone = true;
		delete btnPict;
	}
}

//-------------------------------------------------------------------------

void BuddyListSetupView::Draw( BRect )
{
	BRect r = Bounds();
	r.bottom -= 26;	
	SetHighColor( 156, 154, 156 );		// dark grey
	StrokeRect( r );
}

//-------------------------------------------------------------------------

void BuddyListSetupView::RemoveItem()
{
	int32 Selected;
	BListItem* currentItem;
	PeopleSetupItem* currentSetupItem;
	BAlert* alert;
	PersonInfo* pi;
	char message[255];
	editType type;

	// Get the currently selected item
	Selected = buddies->CurrentSelection();
	currentItem = buddies->ItemAt( Selected );
	if( (currentSetupItem = dynamic_cast<PeopleSetupItem*>(currentItem)) == NULL )
		return;
		
	// Was this a person item?
	if( currentSetupItem->OutlineLevel() ) {
		pi = list->FindPerson( currentSetupItem->UserName() );
		if( !pi )
			return;
		sprintf( message, "Are you sure you want to delete\n'%s' from the list?", pi->name.String() );
		type = ET_ADDPERSON;
	}
	
	// No, it was a group.
	else {
		sprintf( message, "Are you sure you want to delete\nthe group '%s'?", currentSetupItem->UserName().String() );
		type = ET_ADDGROUP;
	}
	
	// ask for permission, then remove the sucker
	alert = new BAlert("title", message, "Cancel", "Delete", NULL, B_WIDTH_AS_USUAL, B_EVEN_SPACING, B_WARNING_ALERT);
	alert->SetShortcut( 0, B_ESCAPE );
	if( alert->Go() ) {
		if( type == ET_ADDPERSON )
			list->RemovePerson( currentSetupItem->UserName() );
		else
			list->RemoveGroup( currentSetupItem->UserName() );
	}
}

//-------------------------------------------------------------------------

void BuddyListSetupView::AddItem( editType type ) {

	BRect frame( 0, 0, 200, 85 );
	PersonAddEditWindow* dlg;

	// are there any groups?
	if( type == ET_ADDPERSON && !list->CountGroups() ) {
		Say( "You need to add a group first." );
		return;
	}

	// tweak the frame and open the dialog
	windows->MakeDialogFrame( frame, Window() );
 	dlg = new PersonAddEditWindow( frame, Window(), type, NULL );
	dlg->SetNow( false );
 	dlg->Show();
}

//-------------------------------------------------------------------------

void BuddyListSetupView::EditItem()
{
	int32 Selected;
	BRect itemFrame, frame;
	BListItem* currentItem;
	PeopleSetupItem* currentSetupItem;
	BPoint point;
	frame = BRect( 0, 0, 200, 85 );
	editType type;
	PersonAddEditWindow* dlg;
	PersonInfo* pi;

	// Get the currently selected item
	Selected = buddies->CurrentSelection();
	currentItem = buddies->ItemAt( Selected );
	if( (currentSetupItem = dynamic_cast<PeopleSetupItem*>(currentItem)) == NULL )
		return;
		
	// Was this a person item?
	if( currentSetupItem->OutlineLevel() ) {
		pi = list->FindPerson( currentSetupItem->UserName() );
		if( !pi )
			return;
		type = ET_PERSONEDIT;
	}

	// No, it was a group. Copy in the group name (hackish, but it works for now)
	else {
		pi = new PersonInfo;
		pi->name = currentSetupItem->UserName();
		type = ET_GROUPEDIT;
	}

	// Tweak the item frame
	itemFrame = buddies->ItemFrame( Selected );
	point = itemFrame.LeftTop();
	point.y -= scroller->ScrollBar(B_VERTICAL)->Value();
	ConvertToScreen( &point );
	windows->MakeDialogFrame( frame, point );

	// Open the dialog
 	dlg = new PersonAddEditWindow( frame, Window(), type, pi);
 	dlg->SetNow(false);
 	dlg->Show();
}

//-------------------------------------------------------------------------

void BuddyListSetupView::DoResizeStuff( float width, float height ) {

	ResizeTo( width, height );
	addGroupBtn->MoveTo( 0, height - 21 );
	addBuddyBtn->MoveTo( 30, height - 21 );
	removeBtn->MoveTo( 60, height - 21 );
	importBtn->MoveTo( width - 26, height - 21 );

	// Ugh... resizing the scroller stuff is a pain!
	buddies->ResizeTo( width - B_V_SCROLL_BAR_WIDTH - 2, height - 28 );
	scroller->ResizeTo( width - 2, height - 28 );
	scroller->ScrollBar(B_VERTICAL)->MoveTo( width - B_V_SCROLL_BAR_WIDTH - 1, -1 );	
	scroller->ScrollBar(B_VERTICAL)->ResizeTo( B_V_SCROLL_BAR_WIDTH, height - 26 );
	scroller->ScrollBar(B_VERTICAL)->SetValue(0);
}

//=========================================================================

// Notes: when you set a tab up with a view that has B_FOLLOW_ALL (or whatever), it only resizes
//   itself when it is the active tab. So this is a major hack that tells the view to resize itself
//   correctly just before it is shown. Sometimes the BeAPI, nice as it is, seems really dumb!

PeopleTab::PeopleTab( int32 num, BuddyList* mn ) {
	view = NULL;
	number = num;
	main = mn;
}

//-------------------------------------------------------------------------

void PeopleTab::SetControlView( BView* vw ) {
	view = vw;
}

//-------------------------------------------------------------------------

void PeopleTab::Select( BView* vw ) {

	BRect rect = main->Bounds();
	rect.bottom -= (46 + LOGO_HEIGHT + STATUS_HEIGHT);
	rect.right -= 5;

	if( view ) {
		if( number == 0 ) {
			BuddyListView* blist = dynamic_cast<BuddyListView*>( view );		
			if( blist )
				blist->DoResizeStuff( rect.Width(), rect.Height()-1 );
		}
		if( number == 1 ) {
			BuddyListSetupView* slist = dynamic_cast<BuddyListSetupView*>( view );		
			if( slist )
				slist->DoResizeStuff( rect.Width(), rect.Height()-2 );
		}
	}
	
	BTab::Select( vw );
}

//-------------------------------------------------------------------------

void PeopleTab::Deselect() {

	// Done adding/removing stuff; commit the changes
	if( number == 1 )
		list->Commit();
	
	BTab::Deselect();
}


//=========================================================================
