#include <Application.h>
#include <Message.h>
#include <Roster.h>
#include <Window.h>
#include <MenuBar.h>
#include <Menu.h>
#include <Entry.h>
#include <Alert.h>
#include <Path.h>
#include <MenuItem.h>
#include <Box.h>
#include <FilePanel.h>
#include <ScrollView.h>
#include <TextView.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <SupportKit.h>
#include <ctype.h>

#include "Say.h"
#include "Globals.h"
#include "constants.h"
#include "MiscStuff.h"
#include "ChatWindow.h"
#include "PeopleEdit.h"
#include "HTMLStuff.h"
#include "Globals.h"
#include "Linkify.h"
#include "SKMode.h"

const int STATUS_HEIGHT = 13;

ChatWindow::ChatWindow( BString userName, BRect frame, float div )
			: BWindow(frame, userName.String(), B_DOCUMENT_WINDOW, 0, B_ALL_WORKSPACES )
{
	otherName = userName;
	popupOnReceive = false;
	showFontColorSizes = false;
	hasNewMessage = false;
	prefixNewMessages = false;
	enterIsNewline = false;
	tabIsTab = false;
	didAutoRespond = false;
	firstWasMe = false;
	canWarn = false;
	wLevel = 0;
	divider = div;
	
	// colors
	kThis.red = 255;
	kThis.green = kThis.blue = 0;
	kThat.blue = 255;
	kThat.green = kThat.red = 0;

	// preparatory stuff
	_InitWindow();
	srand(unsigned(time(NULL)));
	LoadPrefs();
	
	// Get the screen name for this client
	myName = user->User();

	// Tell the main app that the window has been opened
	BMessage* sendMessage = new BMessage( BEAIM_IM_WINDOW_OPENED );
	sendMessage->AddInt32( "wtype", (int32)USER_MESSAGE_TYPE );
	sendMessage->AddString( "userid", otherName.String() );
	sendMessage->AddPointer( "new_window", this );
	PostAppMessage( sendMessage );
}

//-----------------------------------------------------

// Initialize the window.
void ChatWindow::_InitWindow(void) {

	BRect r;
	BMenu *menu;
	float HeightCalc;
	BMenuItem* tempItem;
	
	// Add the menu bar
	r = Bounds();
	menubar = new BMenuBar(r, "menu_bar");
	AddChild(menubar);
	
	// add the "last message" view
	BRect lastRect = Bounds();
	lastRect.top = menubar->Bounds().bottom + 1.0;
	lastRect.bottom = lastRect.top + 13;
	lastView = new PointerStringView( lastRect, "lastmsg", "(no messages)", B_FOLLOW_TOP | B_FOLLOW_LEFT_RIGHT,
								B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE );
	lastView->SetViewColor( 125, 125, 125 );
	lastView->SetLowColor( 125, 125, 125 );
	//lastView->SetFont( be_bold_font );
	lastView->SetFontSize(10);
	lastView->SetHighColor( 255, 255, 255 );
	lastView->SetAlignment( B_ALIGN_RIGHT );
	AddChild( lastView );
	
	// Set up the Divider
	topUIMargin = menubar->Bounds().bottom + lastRect.Height() + 1.0;
	midUIMargin = 9;
	botUIMargin = 51;//37;
	totalUIMargin = topUIMargin + midUIMargin + botUIMargin;

	// Set up the view rectangles
	BRect textframe = Bounds();
	HeightCalc = textframe.bottom - textframe.top - totalUIMargin;
	textframe.right -= B_V_SCROLL_BAR_WIDTH;
	BRect editframe = textframe;		
	textframe.top = topUIMargin;
	textframe.bottom = HeightCalc*divider + topUIMargin;	
	editframe.top = textframe.bottom + midUIMargin;
	editframe.bottom -= (botUIMargin+1);

	// Add the text (conversation) BHTMLView
	BRect textrect = textframe;
	textrect.OffsetTo(B_ORIGIN);
	textrect.InsetBy( 2.0, 2.0 );
	textview = new FancyTextView( textframe, "text_view", textrect, B_FOLLOW_ALL,
								  B_PULSE_NEEDED | B_WILL_DRAW | B_NAVIGABLE_JUMP );
	AddChild(textscrollview = new BScrollView("text_scroll_view", textview,
				B_FOLLOW_LEFT_RIGHT, 0, false, true, B_NO_BORDER));
	textview->SetDoesUndo(true);
	textview->SetViewColor( 247, 247, 247 );
	
	// Add the message editing BHTMLView
	BRect editrect = editframe;
	editrect.OffsetTo(B_ORIGIN);
	editrect.InsetBy( 2.0, 2.0 );
	editview = new HTMLView( true, editframe, "edit_view", editrect,
				B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_PULSE_NEEDED | B_NAVIGABLE );
	AddChild(editscrollview = new BScrollView("edit_scroll_view", editview,
				B_FOLLOW_LEFT_RIGHT, 0, false, true, B_NO_BORDER));
	editview->SetDoesUndo(true);
	editview->SetMaxBytes( 1024 );
	editview->ResetFontToBase();
	editview->MakeFocus(true);	

	// Add the splitter bar
	rgb_color rayed;
	BRect divframe = Bounds();
	divframe.top = textframe.bottom + 1;
	divframe.bottom = editframe.top - 1;
	divframe.right += 1;
	rayed.red = rayed.green = rayed.blue = 200;
	splitter = new MakSplitterView( divframe, textscrollview, editscrollview, MAK_V_SPLITTER, 
									B_FOLLOW_LEFT_RIGHT, rayed );
	AddChild( splitter );
	
	// Add the bottom view
	BRect btRect = Bounds();
	btRect.top = btRect.bottom - botUIMargin;
	btRect.bottom -= STATUS_HEIGHT;
	btView = new BottomView( btRect );
	AddChild( btView );
	
	// add the name view
	r = Bounds();
	r.top = r.bottom - STATUS_HEIGHT;
	r.right -= 14;
	otherNameView = new NameStatusView( r );
	otherNameView->SetName( otherName );
	otherNameView->SetWarningLevel( user->WarningLevel() );
	if( user->AwayMode() != AM_NOT_AWAY )
		otherNameView->SetAway( true );
	AddChild( otherNameView );
	
	// Add the Window menu to the menubar
	menu = new BMenu( "Window" );
	BMenu* workspaceView = new BMenu( "Show this window" );
	workspaceView->SetRadioMode( true );
	BMenuItem* miThisWorkspace = new BMenuItem("Only in This Workspace", new BMessage(CHATWINDOW_SHOW_IN_THIS_WORKSPACE));
	workspaceView->AddItem( miThisWorkspace );
	workspaceView->AddItem( miAllWorkspaces = new BMenuItem("In All Workspaces", new BMessage(CHATWINDOW_SHOW_IN_ALL_WORKSPACES)) );
	menu->AddItem(workspaceView);	
	menu->AddItem(miPopupOnReceive = new BMenuItem("Popup on Message Receive", new BMessage(CHATWINDOW_POPUP_ON_RECEIVE), 'P'));
	menu->AddItem(miShowFontColorSizes = new BMenuItem("Show Font Colors and Sizes", new BMessage(CHATWINDOW_SHOW_FONT_COLORS_SIZES)));
	menu->AddItem(miEnableLinks = new BMenuItem("Enable web/email links", new BMessage(CHATWINDOW_ENABLE_LINKS)));
	menu->AddSeparatorItem();
	menu->AddItem(new BMenuItem("Previous Chat Window", new BMessage(BEAIM_PREV_CHAT_WINDOW), ','));
	menu->AddItem(new BMenuItem("Next Chat Window", new BMessage(BEAIM_NEXT_CHAT_WINDOW), '.'));
	menu->AddItem(tempItem=new BMenuItem("Jump to Buddy List", new BMessage(BEAIM_JUMP_TO_BUDDYLIST), '/'));
	tempItem->SetTarget(APP_SIGNATURE);
	menu->AddSeparatorItem();
	menu->AddItem(new BMenuItem("Close", new BMessage(MENU_FILE_CLOSE), 'W'));
	menubar->AddItem(menu);

	// Add the Edit menu to the menu bar
	menu = new BMenu("Edit");
	menu->AddItem(miUndo=new BMenuItem("Undo", new BMessage(B_UNDO), 'Z'));
	miUndo->SetTarget(editview);
	menu->AddSeparatorItem();
	menu->AddItem(miCut=new BMenuItem("Cut", new BMessage(B_CUT), 'X'));
	miCut->SetTarget(editview);
	menu->AddItem(miCopy=new BMenuItem("Copy", new BMessage(B_COPY), 'C'));
	miCopy->SetTarget(editview);
	menu->AddItem(miPaste=new BMenuItem("Paste", new BMessage(B_PASTE), 'V'));
	miPaste->SetTarget(editview);
	menu->AddSeparatorItem();
	menu->AddItem(miSelectAll=new BMenuItem("Select All", new BMessage(B_SELECT_ALL), 'A'));
	miSelectAll->SetTarget(editview);
	menubar->AddItem(menu);	

	// Add the Font menu to the menubar
	menu = new BMenu("Style");
	menu->SetEnabled( false );
	menu->AddItem(miNormal=new BMenuItem("Normal", new BMessage(CHATWINDOW_FONT_NORMAL), 'N'));
	miNormal->SetTarget(editview);
	menu->AddItem(miBold=new BMenuItem("Bold", new BMessage(CHATWINDOW_FONT_BOLD), 'B'));
	miBold->SetTarget(editview);		
	menu->AddItem(miItalic=new BMenuItem("Italic", new BMessage(CHATWINDOW_FONT_ITALIC), 'I'));
	miItalic->SetTarget(editview);
	menu->AddSeparatorItem();
	menu->AddItem(miS1=new BMenuItem("Size 1", new BMessage(CHATWINDOW_FONT_SIZE_1), '1'));
	miS1->SetTarget(editview);
	menu->AddItem(miS2=new BMenuItem("Size 2", new BMessage(CHATWINDOW_FONT_SIZE_2), '2'));
	miS2->SetTarget(editview);
	menu->AddItem(miS3=new BMenuItem("Size 3", new BMessage(CHATWINDOW_FONT_SIZE_3), '3'));
	miS3->SetTarget(editview);
	menu->AddItem(miS4=new BMenuItem("Size 4", new BMessage(CHATWINDOW_FONT_SIZE_4), '4'));
	miS4->SetTarget(editview);
	menu->AddItem(miS5=new BMenuItem("Size 5", new BMessage(CHATWINDOW_FONT_SIZE_5), '5'));
	miS5->SetTarget(editview);
	menu->AddItem(miS6=new BMenuItem("Size 6", new BMessage(CHATWINDOW_FONT_SIZE_6), '6'));
	miS6->SetTarget(editview);
	menu->AddItem(miS7=new BMenuItem("Size 7", new BMessage(CHATWINDOW_FONT_SIZE_7), '7'));
	miS7->SetTarget(editview);			
	//menubar->AddItem(menu);		

	// Add the <ScreenName> menu to the menubar
	BMenuItem* buddyItem;
	menu = new BMenu( otherName.String() );
	menu->AddItem(buddyItem = new BMenuItem("Add to Buddy List" B_UTF8_ELLIPSIS,
							  new BMessage(BEAIM_TRY_ADD_PERSON) ));
	menu->AddItem(buddyItem = new BMenuItem("Get Info" B_UTF8_ELLIPSIS,
							  new BMessage(BEAIM_GET_PERSON_INFO) ));
	menu->AddSeparatorItem();
	menu->AddItem(warnItem = new BMenuItem("Warn"B_UTF8_ELLIPSIS, new BMessage(BEAIM_WARN_SOMEONE) ));

	if( user->IsUserBlocked(otherName) )
		menu->AddItem(blockItem = new BMenuItem("Unblock" B_UTF8_ELLIPSIS, new BMessage(BEAIM_SET_USER_BLOCKINESS) ));
	else
		menu->AddItem(blockItem = new BMenuItem("Block" B_UTF8_ELLIPSIS, new BMessage(BEAIM_SET_USER_BLOCKINESS) ));
		
	menubar->AddItem(menu);
	
	// load the default prefs settings
	if( user->ReadBool( "ChatWindowAllWorkspaces", false ) ) {
		SetWorkspaces( B_ALL_WORKSPACES );
		miAllWorkspaces->SetMarked( true );
	} else {
		SetWorkspaces( B_CURRENT_WORKSPACE );
		miThisWorkspace->SetMarked( true );
	}
	if( user->ReadBool( "PopupOnMessage", false ) ) {
		popupOnReceive = true;
		miPopupOnReceive->SetMarked( true );
	}
	if( user->ReadBool( "ShowFontColorSizes", true ) ) {
		showFontColorSizes = true;
		textview->SetShowFontColorSizes( true );
		miShowFontColorSizes->SetMarked( true );
	} else
		textview->SetShowFontColorSizes( false );
	if( user->ReadBool( "ShowLinks", true ) ) {
		linksEnabled = true;
		textview->SetShowLinks( true );
		miEnableLinks->SetMarked( true );
	} else
		textview->SetShowLinks( false );
	prefixNewMessages = user->ReadBool( "PrefixNewMessages", false );
	
	// Set the BTextView attributes
	textview->MakeEditable( false );
	textview->SetStylable( true );
	editview->MakeEditable( true );
	editview->SetStylable( false );
	
	// no warnings yet!
	SetCanWarn( false );
}

//-----------------------------------------------------

// Adjust the size of the BTextView's text rectangles
// when the window is resized.
void ChatWindow::FrameResized(float width, float height) {

	BScrollBar* vert;

	// resize the windows themselves
	float newHeight = (height - totalUIMargin) * divider;
	textscrollview->ResizeTo( width, newHeight );
	editscrollview->ResizeTo( width, height - totalUIMargin - newHeight );
	editscrollview->MoveTo( 0, topUIMargin + midUIMargin + newHeight );
	splitter->MoveTo( 0, topUIMargin + newHeight + 1 );

	// muck with the scrollview's scrollbars
	vert = textscrollview->ScrollBar(B_VERTICAL);
	if( vert ) {
		vert->SetValue( 0 );
	}
		
	// set the new text rects
	BRect textrect = textview->Bounds();
	textrect.InsetBy( 2.0, 2.0 );
	textview->SetTextRect( textrect );
	textrect = editview->Bounds();
	textrect.InsetBy( 2.0, 2.0 );
	editview->SetTextRect( textrect );
}

//-----------------------------------------------------

void ChatWindow::WarnEm() {

	int32 buttonResult;
	BMessage* msg;

	// ask for permission, then remove the sucker
	BAlert* alert = new BAlert("title", "Would you like to send this user a normal warning or an anonymous warning? (You will be identified as the sender with a normal warning, but anonymous warnings are less severe.)",
							   "Cancel", "Anonymous", "Normal", B_WIDTH_AS_USUAL, B_OFFSET_SPACING, B_IDEA_ALERT);
	alert->SetShortcut( 0, B_ESCAPE );
	if( (buttonResult = alert->Go()) ) {
		
		msg = new BMessage(BEAIM_WARN_SOMEONE);
		msg->AddString( "userid", otherName.String() );
	
		if( buttonResult == 1 )
			msg->AddBool( "anonymous", true );
		else
			msg->AddBool( "anonymous", false );
			
		PostAppMessage( msg );
		
		// we just warned 'em; can't do it twice in a row
		SetCanWarn( false );
	}
}

//-----------------------------------------------------

void ChatWindow::MessageReceived( BMessage *message ) {

	// Some variables needed for various messages
	BMessage* sendMessage;

	switch(message->what) {

		case BEAIM_ATTEMPT_SEND_IM:
			SendMessage();
			break;
			
		case BEAIM_INCOMING_IM:
			ProcessIncomingIM( message );
			break;
			
		case BEAIM_RELOAD_PREF_SETTINGS:
			LoadPrefs();
			break;			
			
		case CHATWINDOW_SHOW_IN_THIS_WORKSPACE:
			SetWorkspaces( B_CURRENT_WORKSPACE );
			break;
			
		case CHATWINDOW_SHOW_IN_ALL_WORKSPACES:
			SetWorkspaces( B_ALL_WORKSPACES );
			break;
			
		case CHATWINDOW_POPUP_ON_RECEIVE:
			miPopupOnReceive->SetMarked( !miPopupOnReceive->IsMarked() );
			popupOnReceive = miPopupOnReceive->IsMarked();
			break;
			
		case CHATWINDOW_SHOW_FONT_COLORS_SIZES:
			miShowFontColorSizes->SetMarked( !miShowFontColorSizes->IsMarked() );
			showFontColorSizes = miShowFontColorSizes->IsMarked();
			textview->SetShowFontColorSizes( showFontColorSizes );
			break;
			
		case CHATWINDOW_ENABLE_LINKS:
			miEnableLinks->SetMarked( !miEnableLinks->IsMarked() );
			linksEnabled = miEnableLinks->IsMarked();
			textview->SetShowLinks( linksEnabled );
			break;
		
		case B_CANCEL:
		case MENU_FILE_CLOSE:
			QuitRequested();
			Close();
			break;

		case BEAIM_GET_PERSON_INFO:
			sendMessage = new BMessage(BEAIM_OPEN_USER_WINDOW);
			sendMessage->AddString( "userid", otherName.String() );
			sendMessage->AddInt32( "wtype", (int32)USER_INFO_TYPE );
			sendMessage->AddPointer( "poswindow", this );
			PostAppMessage( sendMessage );
			break;
			
		case BEAIM_SKMODE_CHANGED:
			btView->sendButton->SetLabel( SKReplaceTerm("Send").String() );
			break;

		case BEAIM_TRY_ADD_PERSON:
			AddPerson(message);
			break;

		case BEAIM_NEXT_CHAT_WINDOW:
		case BEAIM_PREV_CHAT_WINDOW:
			message->AddString( "userid", otherName.String() );
			message->AddInt32( "wtype", (int32)USER_MESSAGE_TYPE );

		case BEAIM_ADD_PERSON:
			be_app->PostMessage( message );
			break;
			
		case BEAIM_AWAY_STATUS_CHANGED:
			if( user->AwayMode() != AM_NOT_AWAY ) {
				otherNameView->SetAway(true);
				didAutoRespond = false;
				firstWasMe = false;
			} else
				otherNameView->SetAway(false);
			break;

		case CHATWINDOW_FONT_SIZE_1:
		case CHATWINDOW_FONT_SIZE_2:
		case CHATWINDOW_FONT_SIZE_3:
		case CHATWINDOW_FONT_SIZE_4:
		case CHATWINDOW_FONT_SIZE_5:
		case CHATWINDOW_FONT_SIZE_6:
		case CHATWINDOW_FONT_SIZE_7:
		case CHATWINDOW_FONT_NORMAL:
		case CHATWINDOW_FONT_ITALIC:
		case CHATWINDOW_FONT_BOLD:
			SetStyles( message );			
			break;
			
		case MAK_START_SPLITTING:
			textview->SetBeingDragged(true);
			editview->SetBeingDragged(true);
			break;
			
		case MAK_DONE_SPLITTING:
			textview->SetBeingDragged(false);
			editview->SetBeingDragged(false);
			divider = textscrollview->Frame().Height() / (Bounds().Height() - totalUIMargin);\
			break;

		case BEAIM_WARN_SOMEONE:
			WarnEm();
			break;
			
		case BEAIM_SET_USER_BLOCKINESS:
			DoBlockStuff(message);
			break;

		case BEAIM_ONCOMING_BUDDY:
			if( message->HasInt32("warninglevel") )
				otherNameView->SetWarningLevel( (unsigned short)message->FindInt32("warninglevel") );
			break;

		default:
			BWindow::MessageReceived(message);
			break;
	}
}

//-----------------------------------------------------

// the cancel function
void ChatWindow::DispatchMessage( BMessage* msg, BHandler* handler ) {

	// the enter key does a bunch of stuff, depending on prefs...
	if( msg->what == B_KEY_DOWN && msg->HasString("bytes") ) {
		if( msg->FindString("bytes")[0] == B_ENTER && editview->IsFocus() ) {

			uint32 mods = modifiers();

			// shift-enter... depends on pref
			if( mods & B_SHIFT_KEY ) {
				if( !enterIsNewline ) {
					editview->Insert("\n");
					editview->ScrollToSelection();
					return;
				}
				
				// we can remove the shift key from the mask, so we don't navigate backwards
				msg->ReplaceInt32( "modifiers", mods & ~B_SHIFT_KEY );
			}

			// plain old enter... depends on pref
			else {
				if( enterIsNewline ) {
					editview->Insert("\n");
					editview->ScrollToSelection();
					return;
				}
			}
		}
		
		// handle the almighty tab key
		else if( msg->FindString("bytes")[0] == B_TAB && editview->IsFocus() ) {

			uint32 mods = modifiers();

			if( !tabIsTab ) {
				if( !(mods&B_COMMAND_KEY || mods&B_CONTROL_KEY || mods&B_MENU_KEY || mods&B_SHIFT_KEY) )
					msg->ReplaceInt32( "modifiers", mods | B_COMMAND_KEY );
			}
			
			else {
				if( mods & B_SHIFT_KEY )
					msg->ReplaceInt32( "modifiers", (mods | B_COMMAND_KEY) & ~B_SHIFT_KEY );
			}
		}
		
		else if( msg->FindString("bytes")[0] == B_ESCAPE ) {
			uint32 mods = modifiers();		
			if( mods == 0 || mods == 32 && !menubar->IsFocus()) {
				PostMessage( new BMessage(B_QUIT_REQUESTED) );
				return;
			}
		}
	}

	// our work here is done... dispatch normally
	BWindow::DispatchMessage( msg, handler );
}

//-----------------------------------------------------

bool ChatWindow::QuitRequested()
{
	BMessage* curMsg = CurrentMessage();
	if( !curMsg->HasBool("dontreport") ) {
	
		// Tell the main app that the window has been closed
		BMessage* sendMessage = new BMessage( BEAIM_IM_WINDOW_CLOSED );
		sendMessage->AddInt32( "wtype", (int32)USER_MESSAGE_TYPE );
		sendMessage->AddRect( "frame", Frame() );
		sendMessage->AddFloat( "divider", divider );
		sendMessage->AddString( "userid", otherName.String() );
		be_app->PostMessage( sendMessage );	
	}
	return(true);
}

//-----------------------------------------------------

float ChatWindow::Divider() {
	return divider;
}

//-----------------------------------------------------

void ChatWindow::SetCanWarn( bool cw ) {

	canWarn = cw;
	warnItem->SetEnabled( canWarn );	
}

//-----------------------------------------------------

void ChatWindow::MenusBeginning() {

	// Adjust the Edit menu... if the read-only TextView is selected, Cut, Paste and Undo
	// should be disabled. Also, the targets on Copy/SelectAll need be correct.
	
	if( textview->IsFocus() ) {
		miSelectAll->SetTarget( textview );
		miCopy->SetTarget( textview );
		miCut->SetEnabled( false );
		miPaste->SetEnabled( false );
		miUndo->SetEnabled( false );				
	}
	
	if( editview->IsFocus() ) {
		miSelectAll->SetTarget( editview );
		miCopy->SetTarget( editview );
		miCut->SetEnabled( true );
		miPaste->SetEnabled( true );
		miUndo->SetEnabled( true );	
	}
}

//-----------------------------------------------------

void ChatWindow::DoBlockStuff( BMessage* msg ) {

	if( msg->HasBool("alreadydone") ) {
		if( msg->FindBool("block") )
			blockItem->SetLabel( "Unblock" B_UTF8_ELLIPSIS );
		else
			blockItem->SetLabel( "Block" B_UTF8_ELLIPSIS );
		return;
	}
	
	bool blocked = user->IsUserBlocked(otherName);
	BString message;
	
	if( blocked ) {
		message = "If you unblock ";
		message += otherName;
		message += BString(", this person will be able to tell when you are online, and will be able to communicate normally with you.\n\nUnblock ");
		message += otherName;
		message += BString("?");
	} else {
		message = "If you block ";
		message += otherName;
		message += BString(", this person will not be able to tell when you are online, and won't be able to communicate with you.\n\nBlock ");
		message += otherName;
		message += BString("?");
	}
	
	BAlert* alert = new BAlert("title", message.String(), blocked ? "Don't Unblock" : "Don't Block", 
							   blocked ? "Unblock" : "Block", NULL, B_WIDTH_AS_USUAL, B_EVEN_SPACING, B_IDEA_ALERT);
	alert->SetShortcut( 0, B_ESCAPE );
	if( alert->Go() ) {
		if( blocked ) {
			blockItem->SetLabel( "Block" B_UTF8_ELLIPSIS );
			user->UnblockUser( otherName );
			SetCanWarn(false);
		} else {
			blockItem->SetLabel( "Unblock" B_UTF8_ELLIPSIS );
			user->BlockUser( otherName );
			SetCanWarn(false);
		}
	}
}

//-----------------------------------------------------

void ChatWindow::ProcessIncomingIM( BMessage* incomingIM ) {

	HTMLParser parse;
	styleList styles;
	bool empty = textview->IsEmpty();
	bool wasAutoResponse = incomingIM->FindBool("autorespond");
	BString message = BString(incomingIM->FindString("message"));
	unsigned short wL;
	
	wL = (unsigned short)(incomingIM->FindInt32("warninglevel"));
	otherNameView->SetWarningLevel( wL );
	
	// automatically link the web and mail addresses that weren't linked
	SKFixString( message );
	Linkify( message );
	
	// clear stuff out
	textview->ClearInsertStuff();
	
	// if this was an auto-responded message, do the var replacements
	if( wasAutoResponse )
		user->ReplaceTagVars(message);

	// insert the other person's name (blue font)
	textview->SetFontAttribute( B_BOLD_FACE, true );
	textview->SetFontColor( kThat );
	if( wasAutoResponse )
		textview->InsertSomeText( "Auto response from " );
	textview->InsertSomeText( (char*)incomingIM->FindString("userid") );
	textview->InsertSomeText( ": " );
	textview->ResetFontToBase();
	
	// now parse and insert the HTML
	parse.Parse( const_cast<char*>(message.String()) );
	styles = parse.Styles();
	textview->AddStyledText( parse.ParsedString(), styles );
	styles.Clear();

	// Finalize the insertion, and update the timestamp
	textview->AddStatement();
	UpdateLastMessage();

	// Play the sound
	if( empty )
		sounds->PlaySound( WS_NEWMSG );
	else
		sounds->PlaySound( WS_MSGRECEIVE );
		
	// do the autoresponse, if necessary
	if( user->AwayMode() != AM_NOT_AWAY )
		DoAutoRespond();

	// popup the window if the user has specified that option
	if( popupOnReceive )
		Popup();
	
	// otherwise, update the window title if we aren't active
	else if( prefixNewMessages && !IsActive() ) {
		char newTitle[DISPLAY_NAME_MAX+10];
		sprintf( newTitle, "> %s", otherName.String() );
		SetTitle( newTitle );
		hasNewMessage = true;
	}
	
	// a message just came in; you can warn them now
	SetCanWarn( true );
}

//-----------------------------------------------------

void ChatWindow::AddMyStatement( BString statement ) {

	HTMLParser parse;
	styleList styles;

	// will this be the first message in the window?
	if( textview->IsEmpty() )
		firstWasMe = true;
		
	// automatically link the web and mail addresses that weren't linked
	Linkify( statement );
	SKFixString( statement );

	// insert our name (red font)
	textview->ClearInsertStuff();
	textview->SetFontAttribute( B_BOLD_FACE, true );
	textview->SetFontColor( kThis );
	textview->InsertSomeText( const_cast<char*>(user->User().String()) );
	textview->InsertSomeText( ": " );
	textview->ResetFontToBase();
	
	// now parse and insert the HTML
	parse.Parse( const_cast<char*>(statement.String()) );
	styles = parse.Styles();
	textview->AddStyledText( parse.ParsedString(), styles );
	styles.Clear();

	// Finalize the insertion, and update the timestamp
	textview->AddStatement();
	UpdateLastMessage();
}

//-----------------------------------------------------

void ChatWindow::Popup() {
	
	// Check to see if this window is already displayed in all workspaces
	// if not, move it to the current workspace		
	if( Workspaces() != B_ALL_WORKSPACES )
		SetWorkspaces( B_CURRENT_WORKSPACE );

	// if the window is hidden, pop it up
	if( IsHidden() || !IsFront() ) {
		Show();
		Activate();
	}
}

//-----------------------------------------------------

void ChatWindow::LoadPrefs() {

	BString colTemp, defTemp;
	rgb_color textCol;

	prefixNewMessages = user->ReadBool( "PrefixNewMessages", false );
	enterIsNewline = user->ReadBool( "EnterInsertsNewline", false );
	tabIsTab = user->ReadBool( "TabIsTab", false );
	
	float textMag = float(user->ReadInt32("TextMagnification",100)) / 100;
	textview->SetTextMagnification( textMag );
	
	user->ReadString( "ChatBGColor", colTemp, defTemp = "#FFFFFF" );
	textview->SetViewColor( isValidColor(colTemp) ? StringToColor(colTemp) : StringToColor(defTemp) );
	editview->SetViewColor( isValidColor(colTemp) ? StringToColor(colTemp) : StringToColor(defTemp) );
	user->ReadString( "ChatFontColor", colTemp, defTemp = "#000000" );
	textCol = isValidColor(colTemp) ? StringToColor(colTemp) : StringToColor(defTemp);
	user->ReadString( "ChatFromYouColor", colTemp, defTemp = "#FF0000" );
	kThis = isValidColor(colTemp) ? StringToColor(colTemp) : StringToColor(defTemp);
	user->ReadString( "ChatToYouColor", colTemp, defTemp = "#0000FF" );
	kThat = isValidColor(colTemp) ? StringToColor(colTemp) : StringToColor(defTemp);
	if( !IsHidden() && !IsMinimized() ) {
		textview->Invalidate();
		editview->Invalidate();	
	}
	
	chatFont.SetSize( 12.0 );
	textview->SetBaseFontAndColor( chatFont, textCol );
	chatFont.SetSize( 12.0 * textMag );
	editview->SetBaseFontAndColor( chatFont, textCol );
}

//-----------------------------------------------------

void ChatWindow::SetStyles( BMessage* message ) {
/*	
	int32 start, finish, attrib;
	text_run_array* curStyles = NULL;
	BFont font, font2;
	uint16 curMask;
	
	// Get the current selection, and the current styles for that selection
	editview->GetSelection( &start, &finish );
	curStyles = editview->RunArray( start, finish );
	font = curStyles->runs[0].font;
	curMask = font.Face();
		
	bool on;
		
	switch( message->what ) {
		
		// set the font back to the normal font	
		case CHATWINDOW_FONT_NORMAL:
			font = editview->GetBaseFont();
			editview->SetFontAndColor( start, finish, &font );
			break;
		
		// make it italic
		case CHATWINDOW_FONT_ITALIC:
			if( message->what == CHATWINDOW_FONT_ITALIC )
				attrib = B_ITALIC_FACE;

			editview->GetFontAndColor( start, &font2 );

			on = !(curMask & attrib);
			
			on = true;

				for( int i = 0; i < curStyles->count; ++i ) {
					curMask = curStyles->runs[i].font.Face();
					if( on ) curMask = ((curMask & B_REGULAR_FACE) ? 0 : curMask) | attrib;
					else curMask = (curMask == attrib) ? B_REGULAR_FACE : curMask & (~attrib);
					curStyles->runs[i].font.SetFace( curMask );
				}
				editview->SetRunArray( start, finish, curStyles );
				break;
		
		// make it bold
		case CHATWINDOW_FONT_BOLD:
			break;
		
		// font size change
		default:
			font.SetSize( ConvertFontSize(message->what - CHATWINDOW_FONT_BASE) );
			editview->SetFontAndColor( start, finish, &font );
			break;
	}
		
	delete curStyles;
	*/
}

//-----------------------------------------------------

void ChatWindow::AddPerson( BMessage* msg ) {

	BRect frame( 0, 0, 200, 85 );
	PersonAddEditWindow* dlg;

	// are there any groups?
	if( !list->CountGroups() ) {
		Say( "You need to add a group first." );
		return;
	}

	// tweak the frame and open the dialog
	windows->MakeDialogFrame( frame, this );
 	dlg = new PersonAddEditWindow( frame, this, ET_ADDPERSON, NULL );
 	dlg->SetInitialName( const_cast<char*>(otherName.String()) );
 	dlg->SetNow( true );
 	dlg->Show();
}

//-----------------------------------------------------

void ChatWindow::UpdateLastMessage() {
	
	char theTime[100];
	
	int rnd = (int)(random() % 1979);
	if( rnd == 42 )
		lastView->SetText( "Run! The squirrels are coming!" );
	else {
		time_t now;
		time(&now);		
		strftime( theTime, 100, "%b %e, %Y (%I:%M %p)", localtime(&now) );
		lastView->SetText( theTime );
	}
}

//-----------------------------------------------------

void ChatWindow::SendMessage() {

	BString text;
	BMessage* sendMessage;

	if( editview->IsEmpty() )	// don't send if nothing is there
		return;
	text = BString( editview->GetFormattedMessage() );
	Linkify( text );
	sendMessage = new BMessage(BEAIM_SEND_MESSAGE);
	sendMessage->AddString( "send_to", otherName.String() );			
	sendMessage->AddString( "message", text.String() );
	sendMessage->AddBool( "autorespond", false );
	AddMyStatement( text );
	editview->SetText( "" );
	editview->ResetFontToBase();
	PostAppMessage( sendMessage );
}

//-----------------------------------------------------

void ChatWindow::DoAutoRespond() {

	HTMLParser parse;
	styleList styles;
	BMessage* sendMessage;
	BString text;
	
	// if we already *did* an autorespond, don't do it again
	if( didAutoRespond || firstWasMe )
		return;

	// get the autorespond message, translate the vars, and wrap it with some nice blue text
	text = user->CurrentAwayMessage();
	text.Prepend( "<html><font color=\"#0000ff\">" );
	text.ReplaceAll( "%n", otherName.String() );
	user->ReplaceTagVars( text );
	text.Append( "</font></html>" );

	// send off the auto-respond message
	sendMessage = new BMessage(BEAIM_SEND_MESSAGE);
	sendMessage->AddString( "send_to", otherName.String() );
	sendMessage->AddString( "message", text.String() );
	sendMessage->AddBool( "autorespond", true );
	PostAppMessage( sendMessage );
	
	// now... we get to post it!
	textview->ClearInsertStuff();

	// insert our name (red font)
	textview->SetFontAttribute( B_BOLD_FACE, true );
	textview->SetFontColor( kThis );
	textview->InsertSomeText( "Auto response from " );
	textview->InsertSomeText( const_cast<char*>(user->User().String()) );
	textview->InsertSomeText( ": " );
	textview->ResetFontToBase();
	
	// now parse and insert the HTML
	parse.Parse( const_cast<char*>(text.String()) );
	styles = parse.Styles();
	textview->AddStyledText( parse.ParsedString(), styles );
	styles.Clear();

	// Finalize the insertion, and update the timestamp
	textview->AddStatement();
	UpdateLastMessage();
	didAutoRespond = true;
}

//-----------------------------------------------------

void ChatWindow::WindowActivated( bool activated ) {

	// if we are getting re-activated and there is a new message,
	//  then take out the '>' from the titlebar
	if( activated && hasNewMessage ) {
		SetTitle( otherName.String() );
		hasNewMessage = false;	
	}
}

//=====================================================

BottomView::BottomView( BRect frame )
		   : BView( frame, "bottom_view", B_FOLLOW_BOTTOM | B_FOLLOW_LEFT_RIGHT,
		   								  B_WILL_DRAW | B_NAVIGABLE_JUMP )
{
	SetViewColor( 216, 216, 216 );
	
	BRect btnFrame = Bounds();
	btnFrame.right -= 7;
	btnFrame.left = btnFrame.right - 60;
	btnFrame.top = 7;
	btnFrame.bottom = 27;
	sendButton = new BButton( btnFrame, "send_button", "Send", new BMessage(BEAIM_ATTEMPT_SEND_IM),
										B_FOLLOW_RIGHT | B_FOLLOW_TOP, B_NAVIGABLE | B_WILL_DRAW );
	sendButton->MakeDefault( true );
	
	AddChild( sendButton );
}

//-----------------------------------------------------

void BottomView::Draw( BRect updateRect ) {

	BRect frame = Bounds();

	SetHighColor( 156, 154, 156 );		// dark grey
	BPoint start = frame.LeftTop();
	BPoint end = frame.RightTop();
	StrokeLine( start, end );
}

//=====================================================

PointerStringView::PointerStringView( BRect frame, const char *name, const char *text, 
					   uint32 resizingMode = B_FOLLOW_LEFT | B_FOLLOW_TOP, 
					   uint32 flags = B_WILL_DRAW )
				 : BStringView( frame, name, text, resizingMode, flags )
{
}

//-----------------------------------------------------

void PointerStringView::MouseMoved( BPoint where, uint32 code, const BMessage *msg ) {

	be_app->SetCursor(B_HAND_CURSOR);
	BStringView::MouseMoved(where, code, msg);
}

//=====================================================