#include <Application.h>
#include <Box.h>
#include <Button.h>
#include <stdlib.h>
#include <StringView.h>
#include "ImporterWindow.h"
#include "MiscStuff.h"
#include "Globals.h"
#include "constants.h"
#include "SKMode.h"
#include "Say.h"

const uint32 BEAIM_IMPORT_1 = 'bAi1';
const uint32 BEAIM_IMPORT_2 = 'bAi2';
const uint32 BEAIM_IMPORT_3 = 'bAi3';
const uint32 BEAIM_IMPORT_4 = 'bAi4';
const uint32 BEAIM_INFO_1 = 'bAn1';
const uint32 BEAIM_INFO_2 = 'bAn2';
const uint32 BEAIM_INFO_3 = 'bAn3';
const uint32 BEAIM_INFO_4 = 'bAn4';
const uint32 BEAIM_GOT_FILE_1 = 'bAf1';
const uint32 BEAIM_GOT_FILE_2 = 'bAf2';
const uint32 BEAIM_GOT_FILE_3 = 'bAf3';
const uint32 BEAIM_GOT_FILE_4 = 'bAf4';
const uint32 BEAIM_CLOSED_CONF = 'clC?';

//=========================================================================

ImporterWindow::ImporterWindow( BRect frame )
	 				: BWindow(frame, "Import Buddylist", B_TITLED_WINDOW,
	 				  B_NOT_ZOOMABLE | B_NOT_RESIZABLE )
{
	// Set up the view
	BRect aRect( Bounds() );
	genView = new ImporterView( aRect );
	AddChild( genView );

	// pointers
	panel1 = NULL;
	panel2 = NULL;
	panel3 = NULL;
	confWin = NULL;	
	
	// window feel stuff
	SetFeel( B_MODAL_SUBSET_WINDOW_FEEL );
	AddToSubset( windows->GetBuddyList() );
}

//-------------------------------------------------------------------------

void ImporterWindow::Show() {

	// first off, call the base class function
	BWindow::Show();

	// we're doing this here instead of the constructor, to prevent the half-second or so
	// "pause" before the window shows up; give the user some immediate visual response, even
	// if the window doesn't actually *work* right away. The pause is probably from the file
	// panels getting lists of stuff.
	if( !panel1 || !panel2 || !panel3 ) {
	
		// panel pointers
		panel1 = new BFilePanel( B_OPEN_PANEL, (BMessenger*)this, NULL, 0, false, new BMessage(BEAIM_GOT_FILE_1) );
		panel1->SetTarget(this);
		panel2 = new BFilePanel( B_OPEN_PANEL, (BMessenger*)this, NULL, 0, false, new BMessage(BEAIM_GOT_FILE_2) );
		panel2->SetTarget(this);
		panel3 = new BFilePanel( B_OPEN_PANEL, (BMessenger*)this, NULL, 0, false, new BMessage(BEAIM_GOT_FILE_3) );
		panel3->SetTarget(this);
		panel4 = new BFilePanel( B_OPEN_PANEL, (BMessenger*)this, NULL, 0, false, new BMessage(BEAIM_GOT_FILE_4) );
		panel4->SetTarget(this);

		// beaim settings dir for #'s 2 and 3
		entry_ref homeRef;
		if( GetSettingsRef(homeRef) ) {
			panel2->SetPanelDirectory(&homeRef);
			panel3->SetPanelDirectory(&homeRef);
		}
	}
}

//-------------------------------------------------------------------------

bool ImporterWindow::GetSettingsRef( entry_ref& ref ) {

	BEntry entry;
	BDirectory dir;
	if( dir.SetTo("/boot/home/config/settings/BeAIM") != B_OK )
		return false;
	if( dir.GetEntry( &entry ) != B_OK )
		return false;
	if( entry.GetRef( &ref ) != B_OK )
		return false;
	return true;
}

//-------------------------------------------------------------------------

ImporterWindow::~ImporterWindow() {

	delete panel1;
	delete panel2;
	delete panel3;
}

//-------------------------------------------------------------------------

bool ImporterWindow::QuitRequested()
{
	BMessage* clsMessage = new BMessage(BEAIM_SINGLE_WINDOW_CLOSED);
	clsMessage->AddInt32( "wtype", SW_BUDDYLIST_IMPORTER );
	PostAppMessage( clsMessage );
	return(true);
}

//-------------------------------------------------------------------------

void ImporterWindow::MessageReceived(BMessage* message)
{
	switch(message->what)
	{
		case BEAIM_CLOSED_CONF:
			confWin = NULL;
			break;
	
		case BEAIM_GOT_FILE_1:
			Import1(message);
			break;
			
		case BEAIM_GOT_FILE_2:
			Import2(message);
			break;
			
		case BEAIM_GOT_FILE_3:
			Import3(message);
			break;
			
		case BEAIM_GOT_FILE_4:
			Import4(message);
			break;
	
		case BEAIM_IMPORT_1:
			panel1->Show();
			break;
			
		case BEAIM_IMPORT_2:
			panel2->Show();
			break;
			
		case BEAIM_IMPORT_3:
			panel3->Show();
			break;
			
		case BEAIM_IMPORT_4:
			panel4->Show();
			break;
			
		case BEAIM_INFO_1:
			Info1();
			break;
			
		case BEAIM_INFO_2:
			Info2();
			break;
			
		case BEAIM_INFO_3:
			Info3();
			break;
			
		case BEAIM_INFO_4:
			Info4();
			break;
	
		default:
			BWindow::MessageReceived(message);
	}
}

//-------------------------------------------------------------------------

// the cancel function
void ImporterWindow::DispatchMessage( BMessage* msg, BHandler* handler ) {

	// if it's a cancel key, post a B_CANCEL message
	if( msg->what == B_KEY_DOWN )
		if( msg->FindString("bytes")[0] == B_ESCAPE ) {
			PostMessage( new BMessage(B_QUIT_REQUESTED) );
			return;
		}
	
	// our work here is done... dispatch normally
	BWindow::DispatchMessage( msg, handler );
}

//-------------------------------------------------------------------------

void ImporterWindow::Import1( BMessage* msg ) {

	entry_ref ref;
	status_t err;

	// grab the ref in the file
	if ((err = msg->FindRef("refs", &ref)) != B_OK)
		return;

	SetupConfView( ref, 1 );
}

//-------------------------------------------------------------------------

void ImporterWindow::Import2( BMessage* msg ) {

	entry_ref ref;
	status_t err;

	// grab the ref in the file
	if ((err = msg->FindRef("refs", &ref)) != B_OK)
		return;

	SetupConfView( ref, 2 );
}

//-------------------------------------------------------------------------

void ImporterWindow::Import3( BMessage* msg ) {

	entry_ref ref;
	status_t err;

	// grab the ref in the file
	if ((err = msg->FindRef("refs", &ref)) != B_OK)
		return;

	SetupConfView( ref, 3 );
}

//-------------------------------------------------------------------------

void ImporterWindow::Import4( BMessage* msg ) {

	entry_ref ref;
	status_t err;

	// grab the ref in the file
	if ((err = msg->FindRef("refs", &ref)) != B_OK)
		return;

	SetupConfView( ref, 4 );
}

//-------------------------------------------------------------------------

void ImporterWindow::Info1() {

	BString msg;
	msg = "This will let you import a buddylist from a Windows AIM client. It will definitely work with buddylists from AIM version 3.0 or higher; earlier versions of AIM might also work.\n\n";
	msg.Append( "AIM for Windows stores buddylists in a file called \"buddy.lst\". This file is usually located in your windows directory, in a subdirectory called \"aim95\". " );
	msg.Append( "So, assuming you've mounted your C:\\ drive (and it's called \"c\"), you would look for the buddy.lst file here:\n\n" );
	msg.Append( "/c/Windows/aim95/scrnname/buddy.lst\n\n" );
	msg.Append( B_UTF8_ELLIPSIS "where scrnname is your screen name (in all lowercase, and with no spaces)." );
	SKFixString( msg );
	Say( msg );
}

//-------------------------------------------------------------------------

void ImporterWindow::Info2() {

	BString msg;
	msg = "This will let you import a buddylist from an older version of BeAIM (Beta 1 and higher, if memory serves). ";
	msg.Append( " These buddylists are stored in the usual BeAIM settings directory, and have a \"user-\" prefix. So, look for:\n\n" );
	msg.Append( "/boot/home/config/settings/BeAIM/user-scrnname\n\n" );
	msg.Append( B_UTF8_ELLIPSIS "where scrnname is your screen name (in all lowercase, and with no spaces)." );
	SKFixString( msg );
	Say( msg );
}

//-------------------------------------------------------------------------

void ImporterWindow::Info3() {

	BString msg;
	msg = "This will let you import a buddylist from a different user of BeAIM. You might want to do this, for example, if you have more than one screen name and would like to grab your buddylist from the screen name that you aren't currently using.\n\n";
	msg.Append( " These buddylists are stored in the usual BeAIM settings directory, and have a \"xu-\" prefix. So, look for:\n\n" );
	msg.Append( "/boot/home/config/settings/BeAIM/user-scrnname\n\n" );
	msg.Append( B_UTF8_ELLIPSIS "where scrnname is the screen name in question (in all lowercase, and with no spaces)." );
	SKFixString( msg );
	Say( msg );
}

//-------------------------------------------------------------------------

void ImporterWindow::Info4() {

	BString msg;
	msg = "This will let you import a buddylist from a user of GAIM, an AIM clone that runs on Linux and a few other UNIX-like operating systems.\n\n";
	msg.Append( "If you use GAIM, you probably know what you're doing. Which is good since this nifty little message isn't going to be a whole lot of help." );
	SKFixString( msg );
	Say( msg );
}

//-------------------------------------------------------------------------

void ImporterWindow::SetupConfView( entry_ref ref, int type ) {

	BPoint lt = Frame().LeftTop();
	BRect frame( 0, 0, 282, 180 );

	frame.OffsetTo( lt.x + 20, lt.y + 40 );
	
	if( confWin )
		confWin->Activate();
	else {
		confWin = new ImporterConfirmWindow( frame, this, ref, type );
		confWin->AddToSubset( this );
		confWin->AddToSubset( windows->GetBuddyList() );
		confWin->Show();	
	}
	genView->closeButton->MakeFocus(true);
}

//=========================================================================

ImporterView::ImporterView( BRect rect )
	   	   : BView(rect, "generic_input_view", B_FOLLOW_ALL_SIDES, B_WILL_DRAW)
{
	BRect sframe;
	BStringView* tempSV;
	BButton* tempButton;
	int top = 25;

	SetViewColor( 216, 216, 216 );
	
	sframe = BRect( 10, 7, 220, 22 );
	tempSV = new BStringView( sframe, "", "Import a buddylist:" );
	tempSV->SetFont(be_bold_font);
	tempSV->SetFontSize(11);
	AddChild( tempSV );
	
	sframe = BRect( 20, top+8, 220, top+23 );
	tempSV = new BStringView( sframe, "", "from a Windows AIM client"B_UTF8_ELLIPSIS );
	tempSV->SetFont(be_plain_font);
	tempSV->SetFontSize(11);
	AddChild( tempSV );
	
	sframe = BRect( 195, top, 260, top+24 );
	tempButton = new BButton( sframe, "", "Import", new BMessage(BEAIM_IMPORT_1) );
	AddChild( tempButton );
	
	sframe = BRect( 267, top, 332, top+24 );
	tempButton = new BButton( sframe, "", "More Info", new BMessage(BEAIM_INFO_1) );
	AddChild( tempButton );
	
	top += 30;
	
	sframe = BRect( 20, top+8, 220, top+23 );
	tempSV = new BStringView( sframe, "", "from a beta version of BeAIM"B_UTF8_ELLIPSIS );
	tempSV->SetFont(be_plain_font);
	tempSV->SetFontSize(11);
	AddChild( tempSV );
	
	sframe = BRect( 195, top, 260, top+24 );
	tempButton = new BButton( sframe, "", "Import", new BMessage(BEAIM_IMPORT_2) );
	AddChild( tempButton );	
	
	sframe = BRect( 267, top, 332, top+24 );
	tempButton = new BButton( sframe, "", "More Info", new BMessage(BEAIM_INFO_2) );
	AddChild( tempButton );
	
	top += 30;
	
	sframe = BRect( 20, top+8, 220, top+23 );
	tempSV = new BStringView( sframe, "", "from another BeAIM 1.0 user"B_UTF8_ELLIPSIS );
	tempSV->SetFont(be_plain_font);
	tempSV->SetFontSize(11);
	AddChild( tempSV );
	
	sframe = BRect( 195, top, 260, top+24 );
	tempButton = new BButton( sframe, "", "Import", new BMessage(BEAIM_IMPORT_3) );
	AddChild( tempButton );	
	
	sframe = BRect( 267, top, 332, top+24 );
	tempButton = new BButton( sframe, "", "More Info", new BMessage(BEAIM_INFO_3) );
	AddChild( tempButton );
	
	top += 30;
	
	sframe = BRect( 20, top+8, 220, top+23 );
	tempSV = new BStringView( sframe, "", "from a GAIM client"B_UTF8_ELLIPSIS );
	tempSV->SetFont(be_plain_font);
	tempSV->SetFontSize(11);
	AddChild( tempSV );
	
	sframe = BRect( 195, top, 260, top+24 );
	tempButton = new BButton( sframe, "", "Import", new BMessage(BEAIM_IMPORT_4) );
	AddChild( tempButton );	
	
	sframe = BRect( 267, top, 332, top+24 );
	tempButton = new BButton( sframe, "", "More Info", new BMessage(BEAIM_INFO_4) );
	AddChild( tempButton );
	
	top += 38;
	
	// Make the groovy little line
	BBox* divider = new BBox(BRect(11,top,Bounds().right-11,top+2),"divider", B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW, B_FANCY_BORDER);	
	AddChild( divider );
	
	top += 12;
	
	// close button
	sframe = BRect( 267, top, 332, top+24 );
	closeButton = new BButton( sframe, "", "Close", new BMessage(B_QUIT_REQUESTED) );
	closeButton->MakeDefault(true);
	AddChild( closeButton );	
}

//=========================================================================

ImporterConfirmWindow::ImporterConfirmWindow( BRect frame, BWindow* own, entry_ref r, int t )
	 				: BWindow(frame, "Import Buddylist", B_TITLED_WINDOW,
	 				  B_NOT_ZOOMABLE | B_NOT_RESIZABLE )
{
	SetFeel( B_MODAL_SUBSET_WINDOW_FEEL );

	// Set up the view
	BRect aRect( Bounds() );
	genView = new ImporterConfirmView( aRect );
	AddChild( genView );
	owner = own;
	ref = r;
	thelist = NULL;
	type = t;
	
	// fill the list
	FillList();
}

//-------------------------------------------------------------------------

ImporterConfirmWindow::~ImporterConfirmWindow() {

}

//-------------------------------------------------------------------------

bool ImporterConfirmWindow::QuitRequested()
{
	NukeList(thelist);
	thelist = NULL;
	owner->PostMessage( new BMessage(BEAIM_CLOSED_CONF) );
	return(true);
}

//-------------------------------------------------------------------------

void ImporterConfirmWindow::MessageReceived(BMessage* message)
{
	switch(message->what)
	{
		case B_OK:
			Save();
			PostMessage( new BMessage(B_QUIT_REQUESTED) );
			break;
	
		default:
			BWindow::MessageReceived(message);
	}
}

//-------------------------------------------------------------------------

// does the actual importing
void ImporterConfirmWindow::Save()
{
	BString igroup;
	person* temp;
	while (thelist!=NULL)
	{
		igroup = BString(thelist->name);
		list->AddGroup( igroup = BString(thelist->name) );
		temp = thelist->scrnname;
		while (temp!=NULL)
		{
			list->AddPerson( BString(temp->name), igroup );
			temp = temp->next;
		}
		thelist = thelist->next;
	}
}

//-------------------------------------------------------------------------

void ImporterConfirmWindow::FillList() {

	BString toAdd;
	bool first = true;
	group* thebackup;

	// get the actual list	
	if( thelist ) {
		NukeList(thelist);
		thelist = NULL;
	}
	ImportAIMList( ref, thelist, type );
	thebackup = thelist;

	person* temp;
	while (thelist!=NULL)
	{
		if( !first )
			toAdd.Append( "\n\n" );
		else
			first = false;
	
		toAdd.Append( "[" );
		toAdd.Append( thelist->name );
		toAdd.Append( "]" );

		temp = thelist->scrnname;
		if( temp == NULL ) {
			toAdd.Append( "\n      (none)" );
		}
		while (temp!=NULL)
		{
			toAdd.Append( "\n    " );
			toAdd.Append( temp->name );
			temp = temp->next;
		}
		thelist = thelist->next;
	}
	genView->textview->SetText( toAdd.String() );
	thelist = thebackup;
}

//-------------------------------------------------------------------------

// the cancel function
void ImporterConfirmWindow::DispatchMessage( BMessage* msg, BHandler* handler ) {

	// if it's a cancel key, post a B_CANCEL message
	if( msg->what == B_KEY_DOWN )
		if( msg->FindString("bytes")[0] == B_ESCAPE ) {
			PostMessage( new BMessage(B_QUIT_REQUESTED) );
			return;
		}
	
	// our work here is done... dispatch normally
	BWindow::DispatchMessage( msg, handler );
}

//=========================================================================

ImporterConfirmView::ImporterConfirmView( BRect rect )
	   	   : BView(rect, "generic_input_view", B_FOLLOW_ALL_SIDES, B_WILL_DRAW)
{
	BRect sframe;
	BStringView* tempSV;
	SetViewColor( 216, 216, 216 );
	
	sframe = BRect( 7, 7, 300, 22 );
	tempSV = new BStringView( sframe, "", "Here are the screen names that will be imported"B_UTF8_ELLIPSIS );
	tempSV->SetFontSize(11);
	AddChild( tempSV );
	sframe = BRect( 7, 15, 300, 35 );
	tempSV = new BStringView( sframe, "", "Please make sure everything is OK before continuing." );
	tempSV->SetFontSize(11);
	AddChild( tempSV );
	
	// Set up the view rectangles
	BRect textframe = BRect( 7, 40, 275, 136 );
	textframe.right -= B_V_SCROLL_BAR_WIDTH;

	// make the new textview
	BRect textrect = textframe;
	textrect.OffsetTo(B_ORIGIN);
	textrect.InsetBy( 2.0, 2.0 );	
	textview = new BTextView( textframe, "text_view", textrect, B_FOLLOW_NONE, B_WILL_DRAW | B_NAVIGABLE | B_NAVIGABLE_JUMP );
	
									 
	// make the scrollview
	AddChild(scroll = new BScrollView("text_scroll_view", textview,
			B_FOLLOW_NONE, 0, false, true, B_PLAIN_BORDER));
	
	// set attributes
	textview->MakeEditable( false );
	BFont chatFont;
	rgb_color black;
	black.red = black.green = black.blue = 0;
	chatFont.SetSize( 11.0 );
	textview->SetFontAndColor( &chatFont, B_FONT_ALL, &black );
	
	// close button
	int top = 145;
	sframe = BRect( 135, top, 200, top+24 );
	BButton* tempButton = new BButton( sframe, "", "Do Import", new BMessage(B_OK) );
	tempButton->MakeDefault(true);
	AddChild( tempButton );
	sframe = BRect( 210, top, 275, top+24 );
	tempButton = new BButton( sframe, "", "Close", new BMessage(B_QUIT_REQUESTED) );
	AddChild( tempButton );
}
