#include <Application.h>
#include <Box.h>
#include <stdio.h>
#include "Say.h"
#include "InfoWindow.h"
#include "PeopleEdit.h"
#include "MiscStuff.h"
#include "Globals.h"
#include "HTMLStuff.h"
#include "constants.h"
#include "Linkify.h"
#include "SKMode.h"

const int awayOffset = 75;

//-------------------------------------------------------------------------

InfoWindow::InfoWindow( BRect frame, BString uName )
				: BWindow(frame, "", B_TITLED_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE )
{
	char windowTitle[DISPLAY_NAME_MAX+25];

	// do some name stuff
	userName = uName;
	sprintf( windowTitle, "User Info: %s", userName.String() );
	SetTitle( windowTitle );

	// Make the message to request the info
	BMessage* infMessage = new BMessage( BEAIM_GET_PERSON_INFO );
	infMessage->AddString( "userid", userName.String() );

	// invoke the instantiation routine to create a new view 
	BRect rect = Bounds();
	rect.bottom -= 15;
	iView = new InfoView( rect, "InfoView", infMessage ); 

	// add the view to the window 
	AddChild(iView);
	
	// make the stats view
	BRect statFrame = Bounds();
	statFrame.bottom = Bounds().bottom;
	statFrame.top = statFrame.bottom - 14;
	statView = new GStatusView( "Waiting for response" B_UTF8_ELLIPSIS, statFrame );
	statView->SetSpinner(true);
	AddChild( statView );
	
	// Tell the main app that the window has been opened
	BMessage* sendMessage = new BMessage( BEAIM_IM_WINDOW_OPENED );
	sendMessage->AddInt32( "wtype", (int32)USER_INFO_TYPE );
	sendMessage->AddString( "userid", userName.String() );
	sendMessage->AddPointer( "new_window", this );
	PostAppMessage( sendMessage );
	
	gotInfo = false;
	gotAway = false;
	needsAway = false;
	askedAway = false;
}

//-------------------------------------------------------------------------

bool InfoWindow::QuitRequested()
{
	// Tell the main app that the window has been closed
	BMessage* sendMessage = new BMessage( BEAIM_IM_WINDOW_CLOSED );
	sendMessage->AddInt32( "wtype", (int32)USER_INFO_TYPE );
	sendMessage->AddString( "userid", userName.String() );
	PostAppMessage( sendMessage );
	
	//PostAppMessage( new BMessage(B_QUIT_REQUESTED) );
	
	return true;
}

//-------------------------------------------------------------------------

void InfoWindow::AskAway() {

	BMessage* awMsg = new BMessage(BEAIM_GET_AWAY_INFO);
	awMsg->AddString( "userid", userName.String() );
	PostAppMessage( awMsg );
	askedAway = true;
}

//-------------------------------------------------------------------------

void InfoWindow::MessageReceived( BMessage* msg ) {

	switch( msg->what ) {
	
		case BEAIM_UPDATE_INFO:
			iView->UpdateDisplay( msg );
			gotInfo = true;
			needsAway = needsAway || (msg->HasBool("away") && msg->FindBool("away"));
			if( needsAway && !askedAway )
				AskAway();
			if( needsAway && !gotAway )
				statView->SetMessage("Waiting for away message" B_UTF8_ELLIPSIS );
			else {
				statView->SetMessage("Successfully received user info.");
				statView->SetSpinner(false);
			}
			break;
			
		case BEAIM_UPDATE_AWAY_INFO:
			if( gotInfo ) {
				statView->SetMessage("Successfully received user info.");
				statView->SetSpinner(false);
			} else
				statView->SetMessage("Waiting for regular info" B_UTF8_ELLIPSIS);
			EnableAwayMode();
			iView->UpdateAwayDisplay( msg );
			gotAway = true;
			break;

		case B_CANCEL:
		case BEAIM_CLOSE_WINDOW:
			QuitRequested();
			Close();
			break;
			
		case BEAIM_TRY_ADD_PERSON:
			AddPerson();
			break;
			
		case BEAIM_ADD_PERSON:
			be_app->PostMessage( msg );
			break;
			
		default:
			BWindow::MessageReceived( msg );
			break;
	}
}

//-------------------------------------------------------------------------

// the cancel function
void InfoWindow::DispatchMessage( BMessage* msg, BHandler* handler ) {

	// if it's a cancel key, post a B_CANCEL message
	if( msg->what == B_KEY_DOWN )
		if( msg->FindString("bytes")[0] == B_ESCAPE ) {
			PostMessage( new BMessage(B_CANCEL) );
			return;
		}
	
	// our work here is done... dispatch normally
	BWindow::DispatchMessage( msg, handler );
}

//-------------------------------------------------------------------------

void InfoWindow::AddPerson() {

	BRect frame( 0, 0, 200, 85 );
	PersonAddEditWindow* dlg;

	// are there any groups?
	if( !list->CountGroups() ) {
		Say( "You need to add a group first." );
		return;
	}

	// tweak the frame and open the dialog
	windows->MakeDialogFrame( frame, this );
 	dlg = new PersonAddEditWindow( frame, this, ET_ADDPERSON, NULL );
 	dlg->SetInitialName( const_cast<char*>(userName.String()) );
 	dlg->SetNow( true );
 	dlg->Show();
}

//-------------------------------------------------------------------------

void InfoWindow::EnableAwayMode() {

	// resize for the away stuff if necessary
	iView->ResizeBy( 0, awayOffset );
	ResizeBy( 0, awayOffset );
	
	// move some interface elements
	iView->profHolder->MoveBy( 0, awayOffset );
	iView->profileLabel->MoveBy( 0, awayOffset );
	iView->addToListButton->MoveBy( 0, awayOffset );
	iView->closeButton->MoveBy( 0, awayOffset );
	
	// make some others visible
	iView->awayLabel->Show();
	iView->awayHolder->Show();
}

//=========================================================================

InfoView::InfoView( BRect rect, const char *name, BMessage* rqm )
	   	   : BView(rect, name, B_FOLLOW_ALL, B_WILL_DRAW)
{
	SetViewColor( 216, 216, 216 );
	reqMsg = rqm;
}

//-------------------------------------------------------------------------

void InfoView::AttachedToWindow() {

	bool isAway = false;
	int awayOff = awayOffset;
	if( !isAway )
		awayOff = 0;

	BRect labelFrame1, labelFrame2;
	BFont chatFont;

	// Make the box frame
	BRect boxFrame = Bounds();
	boxFrame.InsetBy(10,5);
	boxFrame.bottom = 125;
	
	// set up the label frames
	labelFrame1 = boxFrame;
	labelFrame1.InsetBy( 12, 15 );
	labelFrame1.bottom = labelFrame1.top + 20;
	labelFrame2 = labelFrame1;
	labelFrame2.right += 15;
	labelFrame2.left = Bounds().Width() / 2 - 35;
	
	// make lots of labels
	label1 = new BStringView( labelFrame1, "label1", "User Class:" );
	label1->SetFont(be_bold_font);
	label1->SetFontSize(12);
	labelFrame1.top += 18;
	labelFrame1.bottom += 18;
	AddChild( label1 );
	
	label2 = new BStringView( labelFrame2, "label2", "N/A" );
	label2->SetFont(be_plain_font);
	label2->SetFontSize(12);
	labelFrame2.top += 18;
	labelFrame2.bottom += 18;
	AddChild( label2 );
	
	label3 = new BStringView( labelFrame1, "label3", "Member Since:" );
	label3->SetFont(be_bold_font);
	label3->SetFontSize(12);
	labelFrame1.top += 18;
	labelFrame1.bottom += 18;
	AddChild( label3 );

	label4 = new BStringView( labelFrame2, "label4", "N/A" );
	label4->SetFont(be_plain_font);
	label4->SetFontSize(12);
	labelFrame2.top += 18;
	labelFrame2.bottom += 18;
	AddChild( label4 );
	
	label5 = new BStringView( labelFrame1, "label5", "Online Time:" );
	label5->SetFont(be_bold_font);
	label5->SetFontSize(12);
	labelFrame1.top += 18;
	labelFrame1.bottom += 18;
	AddChild( label5 );

	label6 = new BStringView( labelFrame2, "label6", "N/A" );
	label6->SetFont(be_plain_font);
	label6->SetFontSize(12);
	labelFrame2.top += 18;
	labelFrame2.bottom += 18;
	AddChild( label6 );	
	
	label7 = new BStringView( labelFrame1, "label7", "Warning Level:" );
	label7->SetFont(be_bold_font);
	label7->SetFontSize(12);
	labelFrame1.top += 18;
	labelFrame1.bottom += 18;
	AddChild( label7 );

	label8 = new BStringView( labelFrame2, "label8", "N/A" );
	label8->SetFont(be_plain_font);
	label8->SetFontSize(12);
	labelFrame2.top += 18;
	labelFrame2.bottom += 18;
	AddChild( label8 );

	label9 = new BStringView( labelFrame1, "label9", "Status:" );
	label9->SetFont(be_bold_font);
	label9->SetFontSize(12);
	labelFrame1.top += 18;
	labelFrame1.bottom += 18;
	AddChild( label9 );

	label10 = new BStringView( labelFrame2, "label10", "N/A" );
	label10->SetFont(be_plain_font);
	label10->SetFontSize(12);
	labelFrame2.top += 18;
	labelFrame2.bottom += 18;
	AddChild( label10 );	
	
	// resize for the away stuff if necessary
	if( isAway ) {
		ResizeBy( 0, awayOff );
		Window()->ResizeBy( 0, awayOff );
	}

	// make the away label
	BRect awayLabelRect = boxFrame;
	awayLabelRect.bottom = awayLabelRect.top + 20;
	awayLabelRect.OffsetBy( 0, boxFrame.Height() + 3 );
	awayLabel = new BStringView( awayLabelRect, "awaylabel", "Away Message:" );
	awayLabel->SetFont(be_bold_font);
	awayLabel->SetFontSize(12);		
	AddChild( awayLabel );
	if( !isAway )
		awayLabel->Hide();
	
	// make the textview
	BRect trect, txrect;
	trect = boxFrame;
	trect.OffsetBy( 0, boxFrame.Height() + 25 );
	trect.right -= B_V_SCROLL_BAR_WIDTH;
	trect.bottom = trect.top + 48;
	txrect = trect;
	txrect.OffsetTo( 0, 0 );
	txrect.InsetBy( 2, 2 );
	awaymessage = new FancyTextView( trect, "text_view", txrect, B_FOLLOW_NONE );
	awayHolder = new BScrollView( "profholder", awaymessage, B_FOLLOW_NONE, 0, false, true ); 
	if( !isAway )
		awayHolder->Hide();

	// Whew... make the profile label
	BRect profLabelRect = boxFrame;
	profLabelRect.bottom = profLabelRect.top + 20;
	profLabelRect.OffsetBy( 0, boxFrame.Height() + 3 + awayOff );
	profileLabel = new BStringView( profLabelRect, "proflabel", "Profile:" );
	profileLabel->SetFont(be_bold_font);
	profileLabel->SetFontSize(12);	
	AddChild( profileLabel );	
	
	// make the box
	BBox* infoBox = new BBox( boxFrame );
	infoBox->SetLabel( "Info:" );
	
	// make the textview
	trect = boxFrame;
	trect.OffsetBy( 0, boxFrame.Height() + 25 + awayOff );
	trect.right -= B_V_SCROLL_BAR_WIDTH;
	trect.bottom = trect.top + 80;
	txrect = trect;
	txrect.OffsetTo( 0, 0 );
	txrect.InsetBy( 2, 2 );
	profile = new FancyTextView( trect, "text_view", txrect, B_FOLLOW_NONE );
	profHolder = new BScrollView( "profholder", profile, B_FOLLOW_NONE, 0, false, true ); 
	
	// Set the textview's base font and attributes
	rgb_color black;
	black.red = black.green = black.blue = 0;
	chatFont.SetSize( 12.0 );
	profile->SetBaseFontAndColor( chatFont, black );
	profile->MakeEditable( false );
	profile->SetStylable( true );
	profile->SetAutoScrollEnabled( false );
	awaymessage->SetBaseFontAndColor( chatFont, black );
	awaymessage->MakeEditable( false );
	awaymessage->SetStylable( true );
	awaymessage->SetAutoScrollEnabled( false );
	
	// make some buttons
	closeButton = new BButton( BRect(Bounds().right-68, trect.bottom + 10,
							   Bounds().right-8, trect.bottom + 34), "close", "Close", new BMessage(BEAIM_CLOSE_WINDOW) );
	closeButton->MakeDefault( true );
	addToListButton = new BButton( BRect(7,trect.bottom+10,120,trect.bottom+34), "add", "Add to Buddy List", new BMessage(BEAIM_TRY_ADD_PERSON) );
	
	// add a vanilla "waiting" message so the user isn't left hanging
	//profile->Insert( "Waiting for response..." );
	
	// add the views
	AddChild( infoBox );
	AddChild( profHolder );
	AddChild( awayHolder );
	AddChild( addToListButton );
	AddChild( closeButton );
	
	profile->MakeEditable( false );
	profile->SetStylable( true );
	
	// finally, send the message
	PostAppMessage( reqMsg );
}

//-------------------------------------------------------------------------

void InfoView::UpdateAwayDisplay( BMessage* message ) {

	// away message
	if( message->HasString("away_message" ) ) {

		HTMLParser parse;
		styleList styles;	
		BString origMsg = BString((char*)message->FindString("away_message"));
		
		// automatically link the web and mail addresses that weren't linked
		SKFixString( origMsg );
		Linkify( origMsg );
		
		// handle the away message variables
		user->ReplaceTagVars( origMsg );

		// Now parse the bad boy and display the results
		awaymessage->SetText("");
		awaymessage->ClearInsertStuff();
		awaymessage->ResetFontToBase();
		parse.Parse( const_cast<char*>(origMsg.String()) );
		styles = parse.Styles();
		awaymessage->SetViewColor( styles.bgColor.R(), styles.bgColor.G(), styles.bgColor.B() );
		awaymessage->Invalidate();
		awaymessage->AddStyledText( parse.ParsedString(), styles );
		styles.Clear();
		awaymessage->AddStatement();
	} 
}

//-------------------------------------------------------------------------

void InfoView::UpdateDisplay( BMessage* message ) {
	
	time_t *membersince, *onsince;
	char theTime[100], stemp[100];
	BString origProfile;
	unsigned idletime;
	ssize_t size;
	bool away = false;
	
	// away?
	if( message->HasBool("away") && message->FindBool("away") )
		away = true;
	
	// username
	if( message->HasString("userid") ) {
		char windowTitle[DISPLAY_NAME_MAX+35];
		sprintf( windowTitle, "User Info: %s", message->FindString("userid") );
		if( away )
			strcat( windowTitle, "  (away)" );
		Window()->SetTitle( windowTitle );
	}

	// user class
	if( message->HasString("userclass") ) {
		label2->SetText( message->FindString( "userclass" ) );
		printf( "IW: userclass done.\n" );
		fflush(stdout);
	}

	// member since
	if( message->HasData("membersince", B_TIME_TYPE) ) {
		message->FindData( "membersince", B_TIME_TYPE, (const void**)(&membersince), &size );
		strftime( theTime, 100, "%B %e, %Y", localtime(membersince) );
		label4->SetText( theTime );
	}
	
	// onsince	
	if( message->HasData("onsince", B_TIME_TYPE) ) {
	
		message->FindData( "onsince", B_TIME_TYPE, (const void**)(&onsince), &size );
		MakeElapsedTimeString( *onsince, theTime );
		label6->SetText( theTime );
	}
	
	// warning level
	if( message->HasInt32("warninglevel") ) {
		sprintf( stemp, "%u%%", (unsigned)(message->FindInt32("warninglevel")) );
		label8->SetText( stemp );
	}
	
	// idle time
	if( message->HasInt32( "idletime" ) ) {
		idletime = (unsigned)message->FindInt32("idletime");
		if( idletime ) {
			label9->SetText( "Idle Time:" );
			MakeElapsedTimeString( (double)(idletime*60), stemp );
			label10->SetText( stemp );
		} else {
			label10->SetText( "Active" );
		}
	}
		
	// profile
	if( message->HasString("profile" ) )
		origProfile = BString((char*)message->FindString("profile"));

	if( origProfile.Length() ) {
		HTMLParser parse;
		styleList styles;
		
		// automatically link the web and mail addresses that weren't linked
		SKFixString( origProfile );
		Linkify( origProfile );

		profile->SetText("");
		profile->ClearInsertStuff();
		profile->ResetFontToBase();
		parse.Parse( const_cast<char*>(origProfile.String()) );
		styles = parse.Styles();
		profile->SetViewColor( styles.bgColor.R(), styles.bgColor.G(), styles.bgColor.B() );
		profile->Invalidate();
		profile->AddStyledText( parse.ParsedString(), styles );
		styles.Clear();
		profile->AddStatement();
	} 
	
	// put up a generic "no profile" message
	else
		profile->SetText("There is no profile for this user.");
}

//-------------------------------------------------------------------------

void InfoView::MakeElapsedTimeString( time_t start, char theTime[100] ) {

	time_t now;
	time(&now);
	start += user->ServerTimeOffset();
	double elapsed = difftime( now, start );
	MakeElapsedTimeString( elapsed, theTime );
}

//-------------------------------------------------------------------------

void InfoView::MakeElapsedTimeString( double elapsed, char theTime[100] ) {

	char stemp[100];
	int temp, temp2;

	theTime[0] = '\0';
	
	// calculate days, hours, and minutes
	temp = temp2 = int(elapsed / 86400);
	if( temp ) {
		sprintf( theTime, "%d day%s, ", temp, temp == 1 ? "" : "s" );
		elapsed -= (temp*86400);
	}
	temp = int(elapsed / 3600);
	if( temp ) {
		sprintf( stemp, "%d hour%s", temp, temp==1?"":"s" );
		strcat( theTime, stemp );
		elapsed -= (temp*3600);
		if( int(elapsed / 60) ) {
			if( temp2 )
				strcat( theTime, "," );
			strcat( theTime, " and " );
		}
	}
	temp2 = temp;
	temp = int(elapsed / 60);
	if( temp ) {
		sprintf( stemp, "%d minute%s", temp, temp == 1 ? "" : "s" );
		strcat( theTime, stemp );
		if( !temp2 && temp < 10 ) {
			strcat( theTime, " and " );
		}
	}
	if( !temp2 && !temp ) {
		temp = int(elapsed);
		sprintf( stemp, "%d second%s", temp, temp == 1 ? "" : "s" );
		strcat( theTime, stemp );
	} else if( !temp2 && temp < 10 ) {
		temp = int( elapsed - (temp*60) );
		sprintf( stemp, "%d second%s", temp, temp == 1 ? "" : "s" );
		strcat( theTime, stemp );
	}
}

//=========================================================================


