#include <Bitmap.h>
#include <Alert.h>
#include <Resources.h>
#include <SupportKit.h>
#include <Application.h>
#include <ctype.h>
#include <stdlib.h>
#include "constants.h"
#include "MiscStuff.h"
#include "BitmapView.h"
#include "Globals.h"
#include "Say.h"

//-------------------------------------------------------------------------

void GetBitmapFromResources( BBitmap*& themap, int32 resid, char *resfile ) 
{
	BResources res; 
   	status_t err; 
   	
   	size_t  reslength;
   	const char * resbuf;
   	BMessage archive;
   	
   	if( !resfile )
   		resfile = AppFileName;

	// 	Open resource file
	BFile file( resfile, B_READ_ONLY );  

	// if the file loads OK, that is
   	if ( (file.InitCheck() || (err = res.SetTo(&file)) ) == B_NO_ERROR )  {

		// Find and load resource
   		resbuf = (char*)res.FindResource( B_RAW_TYPE, resid, &reslength );
   							
   		if( resbuf ) {
   		
			// Inflate and unarchive BBitmap
	   		archive.Unflatten( resbuf );	
   			themap = new BBitmap( &archive );	

		} else {
	   		(new BAlert("", "Error while reading resource from file.", "OK"))->Go();
		}
		    		
   	} else {	// Error
   		(new BAlert("", "Error opening resource file.", "OK"))->Go();
   		exit(1);
   	}
}

//-------------------------------------------------------------------------

void MakeButtonPicture( BPicture*& pic, BView* view, int32 resid, char *resfile ) 
{
   	if( !resfile )
   		resfile = AppFileName;

	BBitmap* bitmap = NULL;
	GetBitmapFromResources( bitmap, resid, resfile );
	
	//tempview for creating the picture
	BView *tempView = new BView( BRect(0,0,50,50), "temp", B_FOLLOW_NONE, B_WILL_DRAW );
	view->AddChild(tempView);	
	
	// draw the bitmap into the BPicture
	tempView->BeginPicture(new BPicture); 
	tempView->DrawBitmap(bitmap);
	pic = tempView->EndPicture();

	//get rid of tempview
	view->RemoveChild(tempView);
	delete tempView;	
	delete bitmap;
}

//-------------------------------------------------------------------------

int ConvertFontSize( int fontSize ) {

	switch( fontSize ) {
		case 1: return 8; 	break;
		case 2: return 10; 	break;	
		case 3: return 12; 	break;
		case 4: return 14; 	break;
		case 5: return 18; 	break;
		case 6: return 24; 	break;
		case 7: return 38; 	break;					
		default: return 12; break;
	}	
}

//-------------------------------------------------------------------------

void PostAppMessage( BMessage* msg ) {
	be_app->PostMessage( msg );
	delete msg;
}

//-------------------------------------------------------------------------

void GetAppPath() {

	app_info winfo;
	be_app->GetAppInfo( &winfo );
}

//-------------------------------------------------------------------------

bool CompareScreenNames( char* a, char* b ) {

	while( *a && *a == ' ' ) ++a;
	while( *b && *b == ' ' ) ++b;
	if( !(*a) && !(*b) )
		return true;		
	if( !(*a) || !(*b) )
		return false;

	while( *a && *b ) {
		while( *a && *a == ' ' ) ++a;
		while( *b && *b == ' ' ) ++b;
		if( !(*a) && !(*b) )
			return true;
		if( !(*a) || !(*b) )
			return false;
		if( tolower(*a) != tolower(*b) )
			return false;
		++a;
		++b;
	}
	if( !(*a) && !(*b) )
		return true;		
	if( !(*a) || !(*b) )
		return false;
	return true;
}

//-------------------------------------------------------------------------

void MakeFilenameFromUsername( char* fullpath, char* username ) {

	char* uname2 = new char[strlen(username)+1];
	strcpy( uname2, username );
	
	char* a = uname2;
	char* b = a;
	int endoff = 0;
	
	while( *a ) {
		if( *a != ' ' ) {
			if( b != a ) {
				endoff += int(a-b);
				b = a;
			}
			++b;
		}
		*(a-endoff) = tolower(*a);	
		++a;
	}
	if( b != a )
		endoff += int(a-b);
	*(a-endoff) = '\0';
	
	sprintf( fullpath, "/boot/home/config/settings/BeAIM/user-%s", uname2 );
	delete uname2;
}

//-------------------------------------------------------------------------

void SquishScreenName( char* username ) {

	char* a = username;
	char* b = a;
	int endoff = 0;
	
	while( *a ) {
		if( *a != ' ' ) {
			if( b != a ) {
				endoff += int(a-b);
				b = a;
			}
			++b;
		}
		*(a-endoff) = tolower(*a);	
		++a;
	}
	if( b != a )
		endoff += int(a-b);
	*(a-endoff) = '\0';
}

//-------------------------------------------------------------------------

unsigned char HexcharToInt( char hex )
{
	// is upper-case A-F
	if ( (hex >= 0x41) && (hex <= 0x46) )
		return (unsigned char)((hex - 0x41) + 0xa);

	// is lower-case a-f
	else if ( (hex >= 0x61) && (hex <= 0x66) )
		return (unsigned char)((hex - 0x61) + 0xa);		

	// is numeric digit
	else if ( (hex >= 0x30) && (hex <= 0x39) )
		return (unsigned char)(hex - 0x30);				
	else
		return (unsigned char)(0);
}

//-------------------------------------------------------------------------

BString ColorToString( rgb_color color ) {

	BString ret = "#";
	char temp[15];
	sprintf( temp, "%02X", color.red );
	ret += BString(temp);
	sprintf( temp, "%02X", color.green );
	ret += BString(temp);
	sprintf( temp, "%02X", color.blue );
	ret += BString(temp);
	return ret;
}

//-------------------------------------------------------------------------

bool isValidColor( BString str ) {
	
	char c;
	if( str.Length() != 7 )
		return false;
	str = str.ToLower();
	if( str[0] != '#' )
		return false;
	for( int i = 1; i < 7; ++i ) {
		c = str[i];
		if( !((c >= '0' && c <= '9') || (c >= 'a' && c <= 'f')) )
			return false;
	}
}

//-------------------------------------------------------------------------

rgb_color StringToColor( BString str ) {

	rgb_color ret;
	ret.red = 16*HexcharToInt(str[1]) + HexcharToInt(str[2]);
	ret.green = 16*HexcharToInt(str[3]) + HexcharToInt(str[4]);
	ret.blue = 16*HexcharToInt(str[5]) + HexcharToInt(str[6]);
	return ret;
}

//-------------------------------------------------------------------------
