#include <Window.h>
#include <TabView.h>
#include <Box.h>
#include <stdio.h>
#include "PrefsWindow.h"
#include "constants.h"
#include "Globals.h"
#include "ColorPicker.h"
#include "Say.h"

//-------------------------------------------------------------------------------------------------------


GenPrefView::GenPrefView(BRect rect, bool go)
	   	   : BView(rect, "GenPrefsTab", B_FOLLOW_LEFT_RIGHT, B_WILL_DRAW )
{
	SetViewColor( 216, 216, 216 );
	globalOnly = go;
	
	// Show Buddy list in all workspaces
	BRect CheckRect = BRect(4,4,20,20);
	BuddyListAllWorkspaces = new BCheckBox( CheckRect, "", "Show the Buddy List in all workspaces",  NULL );
	BuddyListAllWorkspaces->ResizeToPreferred();
	BuddyListAllWorkspaces->SetEnabled( !globalOnly );
	AddChild( BuddyListAllWorkspaces );	
	
	// Play sound effects or not
	CheckRect.top += 20;
	PlaySounds = new BCheckBox( CheckRect, "", "Play sound effects",  NULL );
	PlaySounds->ResizeToPreferred();
	PlaySounds->SetEnabled( !globalOnly );
	AddChild( PlaySounds );

	// enable the BeAIM Deskbar Icon
	CheckRect.top += 20;
	UseDeskbar = new BCheckBox( CheckRect, "", "Enable BeAIM deskbar icon",  NULL );
	UseDeskbar->ResizeToPreferred();
	UseDeskbar->SetEnabled( !globalOnly );
	AddChild( UseDeskbar );

	// enable the BeAIM Deskbar Icon
	CheckRect.top += 20;	
	DoIdleTime = new BCheckBox( CheckRect, "", "Report your idle time to the AIM server",  NULL );
	DoIdleTime->ResizeToPreferred();
	DoIdleTime->SetEnabled( !globalOnly );
	AddChild( DoIdleTime );
	
	// Enter inserts line break instead of sending message
	CheckRect.top += 20;
	EnterInsertsNewline = new BCheckBox( CheckRect, "", "Enter inserts line break instead of pressing the default button",  NULL );
	EnterInsertsNewline->ResizeToPreferred();
	EnterInsertsNewline->SetEnabled( !globalOnly );
	AddChild( EnterInsertsNewline );
	
	// Tab inserts tab instead of changing focus
	CheckRect.top += 20;
	TabIsTab = new BCheckBox( CheckRect, "", "Tab inserts tab instead of changing focus",  NULL );
	TabIsTab->ResizeToPreferred();
	TabIsTab->SetEnabled( !globalOnly );
	AddChild( TabIsTab );

	// Now initialize them
	BuddyListAllWorkspaces->SetValue( user->ReadBool("BuddyListAllWorkspaces", true) );
	PlaySounds->SetValue( user->ReadBool("PlaySounds", true) );
	UseDeskbar->SetValue( user->ReadBool("UseDeskbarIcon", true) );
	DoIdleTime->SetValue( user->ReadBool("ReportIdleTime", true) );
	EnterInsertsNewline->SetValue( user->ReadBool( "EnterInsertsNewline", false ) );
	TabIsTab->SetValue( user->ReadBool( "TabIsTab", false ) );
}


void GenPrefView::Save() {

	// save the prefs
	if( !globalOnly ) {
		user->WriteBool("BuddyListAllWorkspaces", BuddyListAllWorkspaces->Value() );
		user->WriteBool("PlaySounds", PlaySounds->Value() );
		user->WriteBool("UseDeskbarIcon", UseDeskbar->Value() );
		user->WriteBool("ReportIdleTime", DoIdleTime->Value() );
		user->WriteBool("EnterInsertsNewline", EnterInsertsNewline->Value() );
		user->WriteBool("TabIsTab", TabIsTab->Value() );
	}
}

//-------------------------------------------------------------------------------------------------------


ConnectionPrefView::ConnectionPrefView(BRect rect, bool go)
	   	   : BView(rect, "ConPrefsTab", B_FOLLOW_LEFT_RIGHT, B_WILL_DRAW )
{
	SetViewColor( 216, 216, 216 );
	BBox* serverBox;
	char portString[50];
	BString aimHost;
	int32 aimPort;
	globalOnly = go;

	// Add the server group box
	serverBox = new BBox( BRect(8,4,250,82) );
	serverBox->SetLabel("AIM Server");
	AddChild( serverBox );

	// set up the server text boxes
	conHost = new BTextControl( BRect(15,20,225,40), "hostname", "Host:", "", NULL );
	conHost->SetViewColor( 216, 216, 216 );
	conHost->SetDivider( 50 );
	conPort = new BTextControl( BRect(15,45,225,65), "portname", "Port:", "", NULL );
	conPort->SetViewColor( 216, 216, 216 );
	conPort->SetDivider( 50 );
	
	// add them
	serverBox->AddChild( conHost );
	serverBox->AddChild( conPort );
	
	// now initialize them
	user->ReadString( "AIMHost", aimHost, "login.oscar.aol.com", true, false );
	conHost->SetText( aimHost.LockBuffer(0) );
	aimHost.UnlockBuffer();
	aimPort = user->ReadInt32( "AIMPort", 5190, true );
	sprintf( portString, "%lu", aimPort );
	conPort->SetText( portString );
}


void ConnectionPrefView::Save() {

	int32 port = 0;

	user->WriteString( "AIMHost", BString(conHost->Text()), true );
	sscanf( (char*)conPort->Text(), "%lu", &port );
	user->WriteInt32( "AIMPort", port, true );
}


//-------------------------------------------------------------------------------------------------------


ChatWindowPrefView::ChatWindowPrefView(BRect rect, bool go)
	   	   : BView(rect, "ConPrefsTab", B_FOLLOW_LEFT_RIGHT, B_WILL_DRAW )
{
	EnStringView* tempSV;
	BString colTemp, defTemp;
	BRect sframe;
	SetViewColor( 216, 216, 216 );
	globalOnly = go;

	// Show chat windows in all workspaces
	BRect CheckRect = BRect(4,4,20,20);
	ChatWindowAllWorkspaces = new BCheckBox( CheckRect, "", "Show chat windows in all workspaces by default",  NULL );
	ChatWindowAllWorkspaces->ResizeToPreferred();
	ChatWindowAllWorkspaces->SetEnabled( !globalOnly );
	AddChild( ChatWindowAllWorkspaces );

	// Popup the chat window when a message is received
	CheckRect.top += 20;
	ChatWindowPopup = new BCheckBox( CheckRect, "", "Popup chat windows when messages are received by default",  NULL );
	ChatWindowPopup->ResizeToPreferred();
	ChatWindowPopup->SetEnabled( !globalOnly );
	AddChild( ChatWindowPopup );

	// Show font color and size attributes in the chat windows
	CheckRect.top += 20;
	ChatFontColorSizes = new BCheckBox( CheckRect, "", "Show font color and size attributes by default",  NULL );
	ChatFontColorSizes->ResizeToPreferred();
	ChatFontColorSizes->SetEnabled( !globalOnly );
	AddChild( ChatFontColorSizes );

	// Show URL/mailto links by default
	CheckRect.top += 20;
	ChatShowLinks = new BCheckBox( CheckRect, "", "Enable web/email links",  NULL );
	ChatShowLinks->ResizeToPreferred();
	ChatShowLinks->SetEnabled( !globalOnly );
	AddChild( ChatShowLinks );
	
	// Prefix incoming messages with '>' in the title bar
	CheckRect.top += 20;
	ChatPrefixNewMessages = new BCheckBox( CheckRect, "", "Prefix incoming messages with '>' in the title bar when window is inactive",  NULL );
	ChatPrefixNewMessages->ResizeToPreferred();
	ChatPrefixNewMessages->SetEnabled( !globalOnly );
	AddChild( ChatPrefixNewMessages );
	
	// Color, baby, yeah!	
	bgColor = new ColorView( BRect( 6, 108, 36, 123 ), 1 );
	bgColor->SetEnabled( !globalOnly );
	sframe = BRect( 43, 110, 214, 122 );
	tempSV = new EnStringView( sframe, "", "Default background color" );
	tempSV->SetEnabled( !globalOnly );
	AddChild( bgColor );
	AddChild( tempSV );
	
	// Color, baby, yeah!	
	fontColor = new ColorView( BRect( 216, 108, 246, 123 ), 2);
	fontColor->SetEnabled( !globalOnly );
	sframe = BRect( 253, 110, 400, 122 );
	tempSV = new EnStringView( sframe, "", "Default font color" );
	tempSV->SetEnabled( !globalOnly );
	AddChild( fontColor );
	AddChild( tempSV );
	
	// Color, baby, yeah!	
	fromYouColor = new ColorView( BRect( 6, 128, 36, 143 ), 3);
	fromYouColor->SetEnabled( !globalOnly );
	sframe = BRect( 43, 130, 214, 142 );
	tempSV = new EnStringView( sframe, "", "Username: messages from you" );
	tempSV->SetEnabled( !globalOnly );
	AddChild( fromYouColor );
	AddChild( tempSV );
	
	// Color, baby, yeah!	
	toYouColor = new ColorView( BRect( 216, 128, 246, 143 ), 4);
	toYouColor->SetEnabled( !globalOnly );
	sframe = BRect( 253, 130, 400, 142 );
	tempSV = new EnStringView( sframe, "", "Username: messages to you" );
	tempSV->SetEnabled( !globalOnly );
	AddChild( toYouColor );
	AddChild( tempSV );
	
	// slider, baby, yeah!
	BRect slidRect = BRect( 6, 153, 400, 168 );
	magSlider = new GSlider( slidRect, 25, 250 );
	magSlider->SetEnabled( !globalOnly );
	AddChild( magSlider );

	// now initialize them
	ChatWindowAllWorkspaces->SetValue( user->ReadBool("ChatWindowAllWorkspaces", false) );
	ChatWindowPopup->SetValue( user->ReadBool("PopupOnMessage", false) );
	ChatFontColorSizes->SetValue( user->ReadBool("ShowFontColorSizes", true) );
	ChatShowLinks->SetValue( user->ReadBool("ShowLinks", true) );
	ChatPrefixNewMessages->SetValue( user->ReadBool( "PrefixNewMessages", false ) );
	magSlider->SetValue( user->ReadInt32("TextMagnification", 100) );

	user->ReadString( "ChatBGColor", colTemp, defTemp = "#FFFFFF" );
	bgColor->SetColor( isValidColor(colTemp) ? StringToColor(colTemp) : StringToColor(defTemp) );
	user->ReadString( "ChatFontColor", colTemp, defTemp = "#000000" );
	fontColor->SetColor( isValidColor(colTemp) ? StringToColor(colTemp) : StringToColor(defTemp) );
	user->ReadString( "ChatFromYouColor", colTemp, defTemp = "#FF0000" );
	fromYouColor->SetColor( isValidColor(colTemp) ? StringToColor(colTemp) : StringToColor(defTemp) );
	user->ReadString( "ChatToYouColor", colTemp, defTemp = "#0000FF" );
	toYouColor->SetColor( isValidColor(colTemp) ? StringToColor(colTemp) : StringToColor(defTemp) );
}


void ChatWindowPrefView::OpenColorPicker( BMessage* msg ) {

	int cid;
	BRect frame(0,0,312,110);
	BPoint lt = Window()->Frame().LeftTop();
	rgb_color* startColor;
	BString title;
	ssize_t size;
	
	cid = msg->FindInt32("cid");
	msg->FindData( "color", B_RGB_COLOR_TYPE, (const void**)&startColor, &size );
	
	switch( cid ) {
		case 1: title = "Default Background Color";
			break;
		case 2: title = "Default Font Color";
			break;
		case 3: title = "Username: Messages From You";
			break;
		case 4: title = "Username: Messages To You";
			break;
	}

	frame.OffsetTo( lt.x + 20, lt.y + 40 );
	ColorPickerWindow* stuff = new ColorPickerWindow( frame, title.String(), cid, Window() );
	stuff->AddToSubset( Window() );
	stuff->SetTheColor( *startColor );
	stuff->Show();
}


void ChatWindowPrefView::SetNewColor( BMessage* msg ) {

	int cid;
	rgb_color* startColor;
	ssize_t size;
	
	cid = msg->FindInt32("cid");
	msg->FindData( "color", B_RGB_COLOR_TYPE, (const void**)&startColor, &size );
	
	switch( cid ) {
	
		case 1:
			bgColor->SetColor( *startColor );
			break;
			
		case 2:
			fontColor->SetColor( *startColor );
			break;
			
		case 3:
			fromYouColor->SetColor( *startColor );
			break;
			
		case 4:
			toYouColor->SetColor( *startColor );
			break;
	}
}


void ChatWindowPrefView::Save() {

	// save the prefs
	if( !globalOnly ) {
		user->WriteBool("ChatWindowAllWorkspaces", ChatWindowAllWorkspaces->Value() );
		user->WriteBool("PopupOnMessage", ChatWindowPopup->Value() );
		user->WriteBool("ShowFontColorSizes", ChatFontColorSizes->Value() );
		user->WriteBool("ShowLinks", ChatShowLinks->Value() );
		user->WriteBool("PrefixNewMessages", ChatPrefixNewMessages->Value() );
		user->WriteInt32("TextMagnification", magSlider->Value() );
		user->WriteString("ChatBGColor", ColorToString(bgColor->GetColor()) );
		user->WriteString("ChatFontColor", ColorToString(fontColor->GetColor()) );
		user->WriteString("ChatFromYouColor", ColorToString(fromYouColor->GetColor()) );
		user->WriteString("ChatToYouColor", ColorToString(toYouColor->GetColor()) );
	}
}


//-------------------------------------------------------------------------------------------------------


AwayPrefView::AwayPrefView(BRect rect, bool go)
			: BView(rect, "AwayPrefsTab", B_FOLLOW_LEFT_RIGHT, B_WILL_DRAW )
{
	SetViewColor( 216, 216, 216 );
	globalOnly = go;

	// Show chat windows in all workspaces
/*	BRect CheckRect = BRect(4,4,20,20);
	ChatWindowAllWorkspaces = new BCheckBox( CheckRect, "", "Show chat windows in all workspaces by default",  NULL );
	ChatWindowAllWorkspaces->ResizeToPreferred();
	ChatWindowAllWorkspaces->SetEnabled( !globalOnly );
	AddChild( ChatWindowAllWorkspaces );

	// Popup the chat window when a message is received
	CheckRect.top += 20;
	ChatWindowPopup = new BCheckBox( CheckRect, "", "Popup chat windows when messages are received by default",  NULL );
	ChatWindowPopup->ResizeToPreferred();
	ChatWindowPopup->SetEnabled( !globalOnly );
	AddChild( ChatWindowPopup );*/

/*
	// now initialize them
	ChatWindowAllWorkspaces->SetValue( user->ReadBool("ChatWindowAllWorkspaces", false) );
	ChatWindowPopup->SetValue( user->ReadBool("PopupOnMessage", false) );
	ChatFontColorSizes->SetValue( user->ReadBool("ShowFontColorSizes", true) );
	ChatShowLinks->SetValue( user->ReadBool("ShowLinks", true) );
	ChatPrefixNewMessages->SetValue( user->ReadBool( "PrefixNewMessages", false ) );
	*/
}


void AwayPrefView::Save() {

	// save the prefs
	if( !globalOnly ) {
	/*	user->WriteBool("ChatWindowAllWorkspaces", ChatWindowAllWorkspaces->Value() );
		user->WriteBool("PopupOnMessage", ChatWindowPopup->Value() );
		user->WriteBool("ShowFontColorSizes", ChatFontColorSizes->Value() );
		user->WriteBool("ShowLinks", ChatShowLinks->Value() );
		user->WriteBool("PrefixNewMessages", ChatPrefixNewMessages->Value() );*/
	}
}


//-------------------------------------------------------------------------------------------------------

PrefsWindow::PrefsWindow(BRect frame, bool globalOnly)
			: BWindow(frame, "BeAIM Preferences", B_TITLED_WINDOW, B_NOT_ZOOMABLE | B_NOT_RESIZABLE )
{
	BRect r = Bounds();
	BTab *generaltab, *connectiontab, *cwtab, *awaytab;
	BTabView *tabView; 

	// Add the basic background
	BView* bgView = new BView( Bounds(), "PrefView", B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE );
	bgView->SetViewColor( 216, 216, 216 );
	AddChild( bgView );

	// Add the tabview control
	r.InsetBy( 4, 4 );
	r.bottom -= 31;
	tabView = new BTabView(r, "tab_view"); 
	tabView->SetViewColor( 216, 216, 216 );
	bgView->AddChild( tabView );
	BRect bottomrect = Bounds();
	bottomrect.InsetBy(4,4);
	bottomrect.top = r.bottom + 4;	
	
	// Add the tabs themselves
	r = tabView->Bounds();
	r.InsetBy(5,5); 
	r.bottom -= tabView->TabHeight(); 

	// General tab
	generaltab = new BTab(); 
	tabView->AddTab( genPrefs = new GenPrefView(r,globalOnly), generaltab ); 
	generaltab->SetLabel("General"); 
	
	// Chat Window prefs tab
	cwtab = new BTab(); 
	tabView->AddTab( chatPrefs = new ChatWindowPrefView(r,globalOnly), cwtab );
	cwtab->SetLabel("Chat windows");
	
	// away prefs tab
	awaytab = new BTab(); 
	awayPrefs = new AwayPrefView(r,globalOnly);
	//tabView->AddTab( awayPrefs, awaytab );
	awaytab->SetLabel("Away stuff");
	
	// Connection prefs tab
	connectiontab = new BTab(); 
	tabView->AddTab( conPrefs = new ConnectionPrefView(r,globalOnly), connectiontab );
	connectiontab->SetLabel("Connection");
	
	//Add buttons
	BRect buttonrect = bottomrect;
	buttonrect.top++;
	buttonrect.left = buttonrect.right - 60;
	BButton *btnCancel = new BButton(buttonrect, "Cancel", "Cancel", new BMessage(PREFS_CANCEL), B_FOLLOW_BOTTOM | B_FOLLOW_RIGHT);
	buttonrect.right = buttonrect.left - 8;
	buttonrect.left = buttonrect.right - 60;
	BButton *btnSave = new BButton(buttonrect, "Save", "Save", new BMessage(PREFS_SAVE), B_FOLLOW_BOTTOM | B_FOLLOW_RIGHT);	
	btnSave->MakeDefault( true );
	bgView->AddChild(btnSave);
	bgView->AddChild(btnCancel);	

	// Show the view
	tabView->Select( globalOnly ? 3 : 0 );
}


void PrefsWindow::MessageReceived( BMessage* message ) {

	switch( message->what ) {
			
		case BEAIM_COLORVIEW_CLICKED:
			chatPrefs->OpenColorPicker(message);
			break;
			
		case BEAIM_NEW_COLOR_PICKED:
			chatPrefs->SetNewColor(message);
			break;

		case PREFS_SAVE:

			// have all the views save themselves
			genPrefs->Save();
			conPrefs->Save();
			chatPrefs->Save();
			
			// send a message to all windows telling them to reload settings
			windows->BroadcastMessage( new BMessage(BEAIM_RELOAD_PREF_SETTINGS) );

			// ... fall thru to Close();

		case B_CANCEL:
		case PREFS_CANCEL:
			PostMessage( new BMessage(B_QUIT_REQUESTED) );
			break;
			
		default:
			BWindow::MessageReceived( message );
	}
}



bool PrefsWindow::QuitRequested()
{
	BMessage* clsMessage = new BMessage(BEAIM_SINGLE_WINDOW_CLOSED);
	clsMessage->AddInt32( "wtype", SW_PREFERENCES );
	PostAppMessage( clsMessage );
	
	return(true);
}


// the cancel function
void PrefsWindow::DispatchMessage( BMessage* msg, BHandler* handler ) {

	// if it's a cancel key, post a B_CANCEL message
	if( msg->what == B_KEY_DOWN )
		if( msg->FindString("bytes")[0] == B_ESCAPE ) {
			PostMessage( new BMessage(B_CANCEL) );
			return;
		}
	
	// our work here is done... dispatch normally
	BWindow::DispatchMessage( msg, handler );
}

//-------------------------------------------------------------------------------------------------------

GSlider::GSlider( BRect frame, int32 min, int32 max )
	   : BSlider( frame, "blah", "Text Magnification:", NULL, min, max )
{
	SetValue( 100 );
	SetLimitLabels( "25%", "250%" );
	SetHashMarks( B_HASH_MARKS_BOTTOM );
	SetHashMarkCount( 5 );
}


char* GSlider::UpdateText( void ) const {

	sprintf( const_cast<char*>(upText), "current: %ld%%", Value() );
	return (char*)upText;
}
