#include <stdio.h>
#include <string.h>
#include <time.h>
#include "UserManager.h"
#include "MiscStuff.h"
#include "Globals.h"
#include "Say.h"
#include "SKMode.h"
#include "xmlparse.h"

//-----------------------------------------------------

UserManager::UserManager() {

	status_t error;
	prefsDirectory = new BDirectory();

	// Try and open the BeAIM prefs folder, and create a new one if it doesn't exist
	error = prefsDirectory->SetTo( "/boot/home/config/settings/BeAIM" );
	if( error == B_ENTRY_NOT_FOUND )
		prefsDirectory->CreateDirectory( "/boot/home/config/settings/BeAIM", prefsDirectory );

	loggedIn = false;
	readMode = 0;
	awayMode = AM_NOT_AWAY;
	serverTimeOffset = 0;
	warningLevel = 0;
	ReadGlobalFile();
}

//-----------------------------------------------------

UserManager::~UserManager() {

	WriteGlobalFile();
}

//-----------------------------------------------------

void UserManager::SetUser( BString user ) {

	// same actual screen name? We're probably just getting a format update.
	if( CompareScreenNames( (char*)user.String(), (char*)currentUser.String() ) ) {
		currentUser = user;
		PostAppMessage( new BMessage(BEAIM_RELOAD_PREF_SETTINGS) );
		return;
	}
	
	// otherwise, we're actually setting a new user. Freaky.
	// if there was someone there before, save their settings and stuff
	if( currentUser.Length() ) {
		WriteUserFile();
		Clear();
	}
	
	// now load the new one.
	currentUser = user;
	ReadUserFile();
	
	// now, everybody reload everything...
	PostAppMessage( new BMessage(BEAIM_RELOAD_PREF_SETTINGS) );
}

//-----------------------------------------------------

BString UserManager::User() {
	return currentUser;
}

//-----------------------------------------------------

BString UserManager::Profile() {
	return profile;
}

//-----------------------------------------------------

void UserManager::Clear() {

	profile = "";
	awayMessages.Clear();
	prefBool.Clear();
	prefInt.Clear();
	prefString.Clear();
	blockList.Clear();
	windows->ClearWindowPositions();
	list->Clear();
}

//-----------------------------------------------------

void UserManager::SetProfile( BString pr ) {
	profile = pr;
	EntitizeCDataString(profile);
	
	// send the message
	BMessage* gMsg = new BMessage( BEAIM_GOING_AWAY );
	if( awayMode != AM_NOT_AWAY )
		gMsg->AddBool( "away", true );
	PostAppMessage( gMsg );
}

//-----------------------------------------------------

void UserManager::SetAwayMessage( BString name, BString oldname, BString message ) {

	int pos;

	awayStruct aw;
	aw.name = name;
	aw.message = message;

	// if there's an oldname, look for it
	if( oldname.Length() ) {
		pos = FindAwayMessage(oldname);
		if( pos != -1 ) {
			
			awayMessages[(unsigned)pos] = aw;
			
			// if this away message in is use, we have to modify it!
			if( awayMode == AM_STANDARD_MESSAGE && oldname == awayMessageName )
				SetAwayMode( AM_STANDARD_MESSAGE, name );
		} else
			awayMessages.Add(aw);
	} else
		awayMessages.Add(aw);
}

//-----------------------------------------------------

bool UserManager::GetAwayMessage( BString name, BString& message ) {

	awayStruct aw;
	
	// search for the away message w/ that name
	bool kg = awayMessages.First(aw);
	while( kg && aw.name != name )
		kg = awayMessages.Next(aw);
		
	// bail if we can't find it
	if( !kg )
		return false;

	// return the string
	message = aw.message;
	return true;
}

//-----------------------------------------------------

bool UserManager::GetFirstAwayMessage( awayStruct& aw ) {
	return awayMessages.First(aw);
}

//-----------------------------------------------------

bool UserManager::GetNextAwayMessage( awayStruct& aw ) {
	return awayMessages.Next(aw);
}

//-----------------------------------------------------

int UserManager::FindAwayMessage( BString name ) {

	for( int i = 0; i < (int)awayMessages.Count(); ++i )
		if( awayMessages[i].name == name )
			return i;
	return -1;
}

//-----------------------------------------------------

void UserManager::RemoveAwayMessage( BString name ) {

	int pos = FindAwayMessage(name);
	if( pos != -1 )
		awayMessages.Delete(pos);
}

//-----------------------------------------------------

int UserManager::AwayMode() {
	return awayMode;
}

//-----------------------------------------------------

void UserManager::SetAwayMode( int am, BString messageInfo ) {

	BString finalMsg;
	bool away = true;
	
	// reset the away message if we are coming back
	if( am == AM_NOT_AWAY ) {
		currentAwayMessage = "";
		away = false;
	}

	// no need to do anything if we're coming back and we're already back
	if( am == AM_NOT_AWAY && awayMode == AM_NOT_AWAY )
		return;

	// grab what will become the (new?) away message
	if( am == AM_CUSTOM_MESSAGE ) {
		EntitizeCDataString( messageInfo );
		finalMsg = customAwayMessage = messageInfo;
	}
	else if( am == AM_STANDARD_MESSAGE )
		GetAwayMessage( messageInfo, finalMsg );

	// if it's a standard message, has anything changed?
	if( am == AM_STANDARD_MESSAGE && am == awayMode &&
		awayMessageName == messageInfo && currentAwayMessage == finalMsg )
		return;

	// save the name of the current message
	if( am == AM_STANDARD_MESSAGE )
		awayMessageName = messageInfo;

	// save the mode and actual message
	currentAwayMessage = finalMsg;
	awayMode = am;

	// Finally... go away!
	BMessage* awMsg = new BMessage(BEAIM_GOING_AWAY);
	awMsg->AddBool( "away", away );
	PostAppMessage( awMsg );
	
	// broadcast that the away status changed
	BMessage* awchangedMsg = new BMessage(BEAIM_AWAY_STATUS_CHANGED);
	awchangedMsg->AddBool( "away", away );
	if( awayMode == AM_CUSTOM_MESSAGE )
		awchangedMsg->AddBool( "custom", true );
	windows->BroadcastMessage( awchangedMsg );
}

//-----------------------------------------------------

BString UserManager::CurrentAwayMessage() {
	BString msg = currentAwayMessage;
	SKFixString( msg );
	return msg;
}

//-----------------------------------------------------

BString UserManager::CurrentAwayMessageName() {
	return awayMessageName;
}

//-----------------------------------------------------

BString UserManager::CustomAwayMessage() {
	return customAwayMessage;
}

//-----------------------------------------------------

void UserManager::ReplaceTagVars( BString& origMsg ) {

	time_t now;
	char theTime[50];
	char theDate[50];

	// make the time/date strings
	time(&now);
	strftime( theTime, 50, "%I:%M:%S %p", localtime(&now) );
	strftime( theDate, 50, "%b %e, %Y", localtime(&now) );

	// handle the "variables" in AIM away messages...
	origMsg.ReplaceAll( "%n", currentUser.String() );
	origMsg.ReplaceAll( "%t", theTime );
	origMsg.ReplaceAll( "%d", theDate );
}

//-----------------------------------------------------

void UserManager::WriteAwayMessages( FILE* ptr ) {

	BString temp, temp2;
	awayStruct aw;
	bool kg;

	if( !awayMessages.Count() )
		return;
		
	// write out all the away messages
	fprintf( ptr, "	<awaymessages>\n" );
	kg = awayMessages.First(aw);
	while( kg ) {
	
		temp = aw.name;
		temp2 = aw.message;
		EntitizeNormalString(temp);
		EntitizeCDataString(temp2);
	
		fprintf( ptr, "		<message name=\"%s\"><![CDATA[%s]]></message>\n", 
				 temp.String(), temp2.String() );
		kg = awayMessages.Next(aw);
	}
	fprintf( ptr, "	</awaymessages>\n" );
	
	fprintf( ptr, "	<customawaymessage><![CDATA[%s]]></customawaymessage>\n",
			 customAwayMessage.String() );
}

//-----------------------------------------------------

void UserManager::WriteBuddyList( FILE* ptr ) {

	BString temp;
	bool kg, kg2;
	
	// write out the buddy list
	fprintf( ptr, "	<buddylist>\n" );
	
	kg = list->GetFirstGroup(temp);
	while( kg ) {
	
		// write out a group
		if( temp.Length() ) {
			fprintf( ptr, "		<group name=\"%s\">\n", temp.LockBuffer(0) );
			temp.UnlockBuffer();
			
			// go through each buddy in the group
			kg2 = list->GetFirstBuddyForCurrentGroup(temp);
			while(kg2) {
				
				if( temp.Length() ) {
					fprintf( ptr, "			<buddy name=\"%s\">\n", temp.LockBuffer(0) );
					fprintf( ptr, "			</buddy>\n" );
					temp.UnlockBuffer();
				}
				kg2 = list->GetNextBuddyForCurrentGroup(temp);
			}
			
			// write the end tag and get the next group (if any)
			fprintf( ptr, "		</group>\n" );
		}
		kg = list->GetNextGroup(temp);
	}
	
	// done!
	fprintf( ptr, "	</buddylist>\n" );
}

//-----------------------------------------------------

void UserManager::WritePrefs( FILE* ptr ) {

	fprintf( ptr, "	<config>\n" );

	// print out the bools
	for( unsigned i = 0; i < prefBool.Count(); ++i ) {
		fprintf( ptr, "		<option type=\"bool\" name=\"%s\" value=\"%s\"/>\n",
					prefBool[i].name.LockBuffer(0),
					prefBool[i].val ? "true" : "false" );
		prefBool[i].name.UnlockBuffer();
	}
	
	// print out the ints
	for( unsigned i = 0; i < prefInt.Count(); ++i ) {
		fprintf( ptr, "		<option type=\"int32\" name=\"%s\" value=\"%ld\"/>\n",
					prefInt[i].name.LockBuffer(0),
					prefInt[i].val );
		prefInt[i].name.UnlockBuffer();
	}
	
	// print out the strings
	for( unsigned i = 0; i < prefString.Count(); ++i ) {
		fprintf( ptr, "		<option type=\"string\" name=\"%s\" value=\"%s\"/>\n",
					prefString[i].name.LockBuffer(0),
					prefString[i].val.Length() ? prefString[i].val.LockBuffer(0) : "" );
		prefString[i].name.UnlockBuffer();
		if( prefString[i].val.Length() )
			prefString[i].val.UnlockBuffer();
	}

	fprintf( ptr, "	</config>\n" );
}

//-----------------------------------------------------

void UserManager::WriteUserFile() {

	if( !currentUser.Length() )
		return;

	BString fName = GetUserFileName();
	FILE* ptr = fopen( fName.LockBuffer(0), "w" );
	fName.UnlockBuffer();
	
	// print out some comments and stuff
	fprintf( ptr, "<!-- BeAIM XML user file for %s -->\n\n", currentUser.String() );
	
	// print out the opening tag
	fprintf( ptr, "<beaim>\n" );

	// write out various bits o' data
	WritePrefs( ptr );
	WriteBuddyList( ptr );
	WriteBlockedUsers( ptr );
	WriteWindowPositions( ptr );

	// print out the user's profile (if any)
	if( profile.Length() ) {
		fprintf( ptr, "	<profile><![CDATA[%s]]></profile>\n", profile.LockBuffer(0) );
		profile.UnlockBuffer();
	}

	// write out the away messages
	WriteAwayMessages( ptr );
	
	// now the closing tag
	fprintf( ptr, "</beaim>" );
	
	fclose(ptr);
}

//-----------------------------------------------------

BString UserManager::GetUserFileName() {

	// remove all spaces and change the letters to lowercase
	BString fName = currentUser;
	fName.ToLower();
	fName.RemoveSet( " " );
	fName = fName.Prepend( "/boot/home/config/settings/BeAIM/xu-" );
	return fName;
}

//-----------------------------------------------------

void UserManager::EntitizeNormalString( BString& str ) {

	str = str.ReplaceAll( "&", "&amp;" );
	str = str.ReplaceAll( "<", "&lt;" );
	str = str.ReplaceAll( ">", "&gt;" );
	str = str.ReplaceAll( "'", "&apos;" );
	str = str.ReplaceAll( "\"", "&quot;" );
}

//-----------------------------------------------------

void UserManager::UnEntitizeNormalString( BString& str ) {

	str = str.ReplaceAll( "&amp;", "&" );
	str = str.ReplaceAll( "&lt;", "<" );
	str = str.ReplaceAll( "&gt;", ">" );
	str = str.ReplaceAll( "&apos;", "'" );
	str = str.ReplaceAll( "&quot;", "\"" );
}

//-----------------------------------------------------

void UserManager::EntitizeCDataString( BString& str ) {
	str = str.ReplaceAll( "]]>", "]]&gt;" );
}

//-----------------------------------------------------

void UserManager::ReadUserFile() {

	XML_Parser parser;
	char data[512];
	size_t len;
	bool done;
	
	// open the file
	Clear();
	readGlobal = false;
	BString fName = GetUserFileName();
	FILE* ptr = fopen( fName.LockBuffer(0), "r" );
	fName.UnlockBuffer();
	if( !ptr )
		return;
		
	printf( "reading XML: %s\n", fName.String() );
	
	// create the XML parser and set handlers
	parser = XML_ParserCreate(NULL);
	XML_SetUserData( parser, this );
	XML_SetElementHandler( parser, startElement, endElement );
	XML_SetCharacterDataHandler( parser, characterData );
	
	do {
		len = fread( data, 1, 512, ptr );
		done = bool(len < 512);
	
		if (!XML_Parse(parser, data, len, done)) {
			fprintf(stderr,
				"%s at line %d\n",
				XML_ErrorString(XML_GetErrorCode(parser)),
				XML_GetCurrentLineNumber(parser) );
			return;
		}	
	} while( !done );

	// clean up
	list->ClearCommitList();
	XML_ParserFree(parser);
	fclose(ptr);
}

//-----------------------------------------------------

void UserManager::startElement( void *userData, const char *name, const char **atts ) {
	
	UserManager* theClass = (UserManager*)userData;

	// is it a userdata tag?
	if( strcasecmp(name,"userdata") == 0 ) {
		theClass->plTemp.name = atts[1];
		theClass->plTemp.savePassword = false;
		theClass->plTemp.autoLogin = false;
	}

	// is it the savepassword tag?
	else if( strcasecmp(name,"savepassword") == 0 )
		theClass->plTemp.savePassword = true;
		
	// is it the password itself?
	else if( strcasecmp(name,"password") == 0 )
		theClass->plTemp.password = BString(atts[1]);
		
	// is it the autologin tag?
	else if( strcasecmp(name,"autologin") == 0 )
		theClass->plTemp.autoLogin = true;
		
	// is it the "last" tag?
	else if( strcasecmp(name,"last") == 0 )
		theClass->lastLogin = theClass->plTemp.name;

	// is it a profile?
	else if( strcasecmp(name,"profile") == 0 )
		theClass->readMode = 1;
		
	// is it a buddylist?
	else if( strcasecmp(name,"buddylist") == 0 )
		theClass->readMode = 2;
		
	// is it a group?
	else if( strcasecmp(name,"group") == 0 ) {
		theClass->rGroup = atts[1];
		theClass->readMode = 3;
		list->AddGroup( BString(atts[1]) );
	}
		
	// is it a buddy?
	else if( strcasecmp(name,"buddy") == 0 ) {
		theClass->rBuddy = atts[1];
		list->AddPerson( BString(atts[1]), theClass->rGroup.LockBuffer(0) );
		theClass->rGroup.UnlockBuffer();
		theClass->readMode = 4;
	}
	
	// is it the list of away messages?
	else if( strcasecmp(name,"awaymessages") == 0 )
		theClass->readMode = 5;
		
	// is it an away message?
	else if( strcasecmp(name,"message") == 0 ) {
		theClass->rAwayName = atts[1];
		theClass->UnEntitizeNormalString( theClass->rAwayName );
		theClass->rAwayMessage = "";
		theClass->readMode = 6;
	}
	
	// is it a custom away message?
	else if( strcasecmp(name,"customawaymessage") == 0 ) {
		theClass->customAwayMessage = "";
		theClass->readMode = 19;
	}
	
	// is it an option (pref setting)?
	else if( strcasecmp(name,"option") == 0 ) {
		theClass->ReadOption( name, atts );
	}
	
	// is it a blocked user?
	else if( strcasecmp(name,"blockeduser") == 0 ) {
		theClass->BlockUser( BString(atts[1]), false );
	}

	// is it a saved window position?
	else if( strcasecmp(name,"userwinpos") == 0 ) {
		float left, top, right, bottom, div;
		theClass->wpRect.user = BString(atts[1]);
		sscanf( atts[3], "%f", &left );
		sscanf( atts[5], "%f", &top );	
		sscanf( atts[7], "%f", &right );	
		sscanf( atts[9], "%f", &bottom );
		sscanf( atts[11], "%f", &div );
		theClass->wpRect.frame.Set( left, top, right, bottom );
		theClass->wpRect.divider = div;
		windows->SetWindowPos( theClass->wpRect );
	}
}

//-----------------------------------------------------

void UserManager::endElement( void *userData, const char *name ) {

	UserManager* theClass = (UserManager*)userData;
	
	// done reading in user data
	if( strcasecmp(name, "userdata") == 0 ) {
		theClass->preLogins.Add( theClass->plTemp );
	}

	else if( theClass->readMode == 1 )
		theClass->readMode = 0;
		
	else if( theClass->readMode == 6 ) {
		awayStruct aw;
		aw.name = theClass->rAwayName;
		aw.message = theClass->rAwayMessage;
		theClass->awayMessages.Add( aw );
		theClass->readMode = 0;	
	}
	
	else if( theClass->readMode == 19 ) {
		theClass->readMode = 0;
	}
}

//-----------------------------------------------------

void UserManager::characterData( void *userData, const char *data, int len ) {

	UserManager* theClass = (UserManager*)userData;
	BString temp = "";
	
	if( theClass->readMode == 1 ) {
		theClass->profile = theClass->profile.Append( data, len );
	}
	
	else if( theClass->readMode == 6 ) {
		theClass->rAwayMessage = theClass->rAwayMessage.Append( data, len );
	}
	
	else if( theClass->readMode == 19 ) {
		theClass->customAwayMessage = theClass->customAwayMessage.Append( data, len );
	}
}

//-----------------------------------------------------

void UserManager::ReadOption( const char *name, const char **atts ) {

	// there must be a type and value
	if( !atts[0] || !atts[1] || !atts[2] || !atts[3] || !atts[4] || !atts[5] )
		return;
	if(  strcasecmp(atts[0],"type") != 0 || 
		 strcasecmp(atts[2],"name") != 0 ||
		 strcasecmp(atts[4],"value") != 0 )
		return;

	// it's a boolean option
	if( strcasecmp(atts[1], "bool") == 0 ) {
		GenList<prefBoolStruct>* list = readGlobal ? &gPrefBool : &prefBool;
		prefBoolStruct addStruct;
		for( unsigned i = 0; i < list->Count(); ++i )
			if( (*list)[i].name == BString(atts[3]) )
				return;
		addStruct.name = atts[3];
		if( strcasecmp(atts[5], "true" ) == 0 )
			addStruct.val = true;
		else if( strcasecmp(atts[5], "false" ) == 0 )
			addStruct.val = false;
		else
			return;
		list->Add( addStruct );
	}
	
	// it's an int32 option
	else if( strcasecmp(atts[1], "int32") == 0 ) {
		GenList<prefInt32Struct>* list = readGlobal ? &gPrefInt : &prefInt;
		prefInt32Struct addStruct;
		for( unsigned i = 0; i < list->Count(); ++i )
			if( (*list)[i].name == BString(atts[3]) )
				return;
		addStruct.name = atts[3];
		sscanf( atts[5], "%ld", &addStruct.val );
		list->Add( addStruct );
	}
	
	// it's a string option
	else if( strcasecmp(atts[1], "string") == 0 ) {
		GenList<prefStringStruct>* list = readGlobal ? &gPrefString : &prefString;
		prefStringStruct addStruct;
		for( unsigned i = 0; i < list->Count(); ++i )
			if( (*list)[i].name == BString(atts[3]) )
				return;
		addStruct.name = atts[3];
		addStruct.val = BString(atts[5]);
		list->Add( addStruct );
	}
}

//-----------------------------------------------------

void UserManager::WriteBool( BString name, bool value, bool global ) {

	GenList<prefBoolStruct>* list = global ? &gPrefBool : &prefBool;
	prefBoolStruct addStruct;
	
	for( unsigned i = 0; i < list->Count(); ++i ) {
		if( (*list)[i].name == name ) {
			(*list)[i].val = value;
			return;
		}
	}
	
	addStruct.name = name;
	addStruct.val = value;
	list->Add( addStruct );
}

//-----------------------------------------------------

bool UserManager::ReadBool( BString name, bool def, bool global ) {

	bool ret = def;
	GenList<prefBoolStruct>* list = global ? &gPrefBool : &prefBool;
	
	for( unsigned i = 0; i < list->Count(); ++i ) {
		if( (*list)[i].name == name )
			ret = (*list)[i].val;
	}
	return ret;
}

//-----------------------------------------------------

void UserManager::WriteInt32( BString name, int32 value, bool global ) {

	GenList<prefInt32Struct>* list = global ? &gPrefInt : &prefInt;
	prefInt32Struct addStruct;
	
	for( unsigned i = 0; i < list->Count(); ++i ) {
		if( (*list)[i].name == name ) {
			(*list)[i].val = value;
			return;
		}
	}
	
	addStruct.name = name;
	addStruct.val = value;
	list->Add( addStruct );
}

//-----------------------------------------------------

int32 UserManager::ReadInt32( BString name, int32 def, bool global ) {

	int32 ret = def;
	GenList<prefInt32Struct>* list = global ? &gPrefInt : &prefInt;
	
	for( unsigned i = 0; i < list->Count(); ++i ) {
		if( (*list)[i].name == name )
			ret = (*list)[i].val;
	}
	return ret;
}

//-----------------------------------------------------

void UserManager::WriteString( BString name, BString value, bool global ) {

	GenList<prefStringStruct>* list = global ? &gPrefString : &prefString;
	prefStringStruct addStruct;
	
	for( unsigned i = 0; i < list->Count(); ++i ) {
		if( (*list)[i].name == name ) {
			(*list)[i].val = value;
			return;
		}
	}
	
	addStruct.name = name;
	addStruct.val = value;
	list->Add( addStruct );
}
		
//-----------------------------------------------------
		
void UserManager::ReadString( BString name, BString& buf, BString def, bool global, bool canBeEmpty ) {

	buf = def;
	GenList<prefStringStruct>* list = global ? &gPrefString : &prefString;
	
	for( unsigned i = 0; i < list->Count(); ++i ) {
		if( (*list)[i].name == name )
			buf = (*list)[i].val;
	}
	
	if( !buf.Length() && !canBeEmpty )
		buf = def;
}

//-----------------------------------------------------

BString UserManager::LastLogin() {
	return lastLogin;
}

//-----------------------------------------------------

void UserManager::SetLastLogin( BString ll ) {
	lastLogin = ll;
}

//-----------------------------------------------------

void UserManager::WritePreLoginData( FILE* ptr ) {

	preLogStruct pl;
	bool kg;

	fprintf( ptr, "	<prelogin>\n" );
	
	kg = preLogins.First(pl);
	while( kg ) {
		fprintf( ptr, "		<userdata name=\"%s\">\n", pl.name.String() );
		if( pl.savePassword ) {
			fprintf( ptr, "			<savepassword/>\n" );
			fprintf( ptr, "			<password value=\"%s\"/>\n", pl.password.String() );
		}
		if( pl.autoLogin )
			fprintf( ptr, "			<autologin/>\n" );
		if( CompareScreenNames(const_cast<char*>(lastLogin.String()),
							   const_cast<char*>(pl.name.String())) )
			fprintf( ptr, "			<last/>\n" );
		fprintf( ptr, "		</userdata>\n" );
		
		kg = preLogins.Next(pl);
	}
	fprintf( ptr, "	</prelogin>\n" );
}

//-----------------------------------------------------

void UserManager::WriteGlobalFile() {

	FILE* ptr = fopen( "/boot/home/config/settings/BeAIM/global", "w" );
	if( !ptr )
		return; 
	
	// print out some comments and stuff
	fprintf( ptr, "<!-- BeAIM global XML preferences file -->\n\n" );
	fprintf( ptr, "<beaim>\n" );

	// write out the prelogin data
	WritePreLoginData( ptr );	

	// write the config stuff
	fprintf( ptr, "	<config>\n" );

	// print out the bools
	for( unsigned i = 0; i < gPrefBool.Count(); ++i ) {
		fprintf( ptr, "		<option type=\"bool\" name=\"%s\" value=\"%s\"/>\n",
					gPrefBool[i].name.LockBuffer(0),
					gPrefBool[i].val ? "true" : "false" );
		gPrefBool[i].name.UnlockBuffer();
	}
	
	// print out the ints
	for( unsigned i = 0; i < gPrefInt.Count(); ++i ) {
		fprintf( ptr, "		<option type=\"int32\" name=\"%s\" value=\"%ld\"/>\n",
					gPrefInt[i].name.LockBuffer(0),
					gPrefInt[i].val );
		gPrefInt[i].name.UnlockBuffer();
	}
	
	// print out the strings
	for( unsigned i = 0; i < gPrefString.Count(); ++i ) {
		fprintf( ptr, "		<option type=\"string\" name=\"%s\" value=\"%s\"/>\n",
					gPrefString[i].name.LockBuffer(0),
					gPrefString[i].val.Length() ? gPrefString[i].val.LockBuffer(0) : "" );
		gPrefString[i].name.UnlockBuffer();
		if( gPrefString[i].val.Length() )
			gPrefString[i].val.UnlockBuffer();
	}

	fprintf( ptr, "	</config>\n" );
	fprintf( ptr, "</beaim>" );

	fclose(ptr);	
}

//-----------------------------------------------------

void UserManager::ReadGlobalFile() {

	XML_Parser parser;
	char data[1024];
	size_t len;
	bool done;
	
	// open the file
	Clear();
	readGlobal = true;
	FILE* ptr = fopen( "/boot/home/config/settings/BeAIM/global", "r" );
	if( !ptr )
		return;
		
	printf( "Reading XML: Global prefs\n" );
	
	// create the XML parser and set handlers
	parser = XML_ParserCreate(NULL);
	XML_SetUserData( parser, this );
	XML_SetElementHandler( parser, startElement, endElement );
	XML_SetCharacterDataHandler( parser, characterData );
	
	do {
		len = fread( data, 1, 1024, ptr );
		done = bool(len < 1024);
	
		if (!XML_Parse(parser, data, len, done)) {
			fprintf(stderr,
				"%s at line %d\n",
				XML_ErrorString(XML_GetErrorCode(parser)),
				XML_GetCurrentLineNumber(parser) );
			return;
		}
	
	} while( !done );

	// clean up
	XML_ParserFree(parser);
	fclose(ptr);
}

//-----------------------------------------------------

bool UserManager::GetNextUser( BString& user, bool first ) {

	bool retval;
	preLogStruct pl;
	
	retval = first ? preLogins.First(pl) : preLogins.Next(pl);
	if( retval )
		user = pl.name;
	
	return retval;
}

//-----------------------------------------------------

void UserManager::GetUserLoginParameters( BString user, BString& pw, bool& al, bool& sp ) {

	preLogStruct pl;
	bool kg;

	// first, set some defaults
	pw = "";
	sp = false;
	al = false;

	// now find and read the real values
	kg = preLogins.First(pl);
	while( kg ) {
		if( CompareScreenNames(const_cast<char*>(user.String()),
							   const_cast<char*>(pl.name.String())) )
		{
			pw = pl.password;
			sp = pl.savePassword;
			al = pl.autoLogin;
			return;
		}
		kg = preLogins.Next(pl);
	}
}

//-----------------------------------------------------

void UserManager::SaveLoginParameters( BString user, BString pw, bool al, bool sp, bool addIfNeeded ) {

	preLogStruct pl, pl2;

	// first, setup the preLogStruct
	pl.name = user;
	pl.password = sp ? pw : "";
	pl.savePassword = sp;
	pl.autoLogin = al;

	// now look for the appropriate index to save it in
	for( unsigned index = 0; index < preLogins.Count(); ++index ) {
		pl2 = preLogins[index];
		if( CompareScreenNames(const_cast<char*>(user.String()),
							   const_cast<char*>(pl2.name.String())) )
		{
			preLogins[index] = pl;
			return;
		}
	}
	
	// didn't find it... add it if needed
	if( addIfNeeded )
		preLogins.Add( pl );
}

//-----------------------------------------------------

void UserManager::SetLoggedIn( bool li ) {

	loggedIn = li;
	if( loggedIn && !list->CountPeople() )
		PostAppMessage( new BMessage(BEAIM_PERHAPS_IMPORT) );
		
	if( !loggedIn ) {
		Clear();
		currentUser = "";
	}
}

//-----------------------------------------------------

void UserManager::SetServerTimeOffset( time_t them ) {

	printf( "SetServerTimeOffset\n" );

	char theTime[100];
	time_t now;
	
	time(&now);
	
	strftime( theTime, 100, "them: %b %e, %Y (%I:%M %p)", localtime(&them) );
	printf( "%s\n", theTime );
	strftime( theTime, 100, "us: %b %e, %Y (%I:%M %p)", localtime(&now) );
	printf( "%s\n", theTime );
	
	serverTimeOffset = now - them;
	printf( "offset - %ld\n", serverTimeOffset );
}
	
//-----------------------------------------------------
		
time_t UserManager::ServerTimeOffset() {
	return serverTimeOffset;
}

//-----------------------------------------------------

void UserManager::BlockUser( BString user, bool report ) {

	if( IsUserBlocked(user) )
		return;
	blockList.Add( user );

	// fire off a message to change the status
	if( report ) {
		BMessage* blockMsg = new BMessage(BEAIM_SET_USER_BLOCKINESS);
		blockMsg->AddString( "userid", user.String() );
		blockMsg->AddBool( "block", true );
		blockMsg->AddBool( "alreadydone", true );
		blockMsg->AddInt32( "wtype", USER_OTHER_INFO );
		windows->ForwardIncomingMessage( blockMsg );
		windows->SendSingleWindowMessage( blockMsg, SW_BLOCKLIST_EDITOR );
		PostAppMessage( blockMsg );
	}
}

//-----------------------------------------------------

void UserManager::UnblockUser( BString user ) {

	unsigned i = 0;
	bool found = false;
	
	for( i = 0; i < blockList.Count(); ++i ) {
		if( CompareScreenNames( (char*)blockList[i].String(), (char*)user.String() ) ) {
			found = true;
			break;
		}
	}

	if( found ) {
		blockList.Delete(i);
		
		// fire off a message to change the status
		BMessage* blockMsg = new BMessage(BEAIM_SET_USER_BLOCKINESS);
		blockMsg->AddString( "userid", user.String() );
		blockMsg->AddBool( "block", false );
		blockMsg->AddBool( "alreadydone", true );
		blockMsg->AddInt32( "wtype", USER_OTHER_INFO );
		windows->ForwardIncomingMessage( blockMsg );
		windows->SendSingleWindowMessage( blockMsg, SW_BLOCKLIST_EDITOR );
		PostAppMessage( blockMsg );
	}
}

//-----------------------------------------------------

bool UserManager::IsUserBlocked( BString user ) {

	unsigned i = 0;
	for( i = 0; i < blockList.Count(); ++i ) {
		if( CompareScreenNames( (char*)blockList[i].String(), (char*)user.String() ) )
			return true;
	}
	return false;
}

//-----------------------------------------------------

bool UserManager::GetNextBlockedUser( BString& user, bool first ) {

	if( first )
		return blockList.First(user);
	else
		return blockList.Next(user);
}

//-----------------------------------------------------

void UserManager::WriteBlockedUsers( FILE* ptr ) {

	BString name;
	bool kg;

	if( !blockList.Count() )
		return;
		
	// write out all the away messages
	fprintf( ptr, "	<blocklist>\n" );
	kg = blockList.First(name);
	while( kg ) {
		fprintf( ptr, "		<blockeduser name=\"%s\"/>\n", name.String() );
		kg = blockList.Next(name);
	}
	fprintf( ptr, "	</blocklist>\n" );
}

//-----------------------------------------------------

void UserManager::WriteWindowPositions( FILE* ptr ) {

	winPosRect wp;
	bool kg;
	
	kg = windows->GetNextWindowPos( wp, true );
	if( !kg )
		return;
		
	fprintf( ptr, "	<windowpositions>\n" );
	while( kg ) {
		fprintf( ptr, "		<userwinpos name=\"%s\" ", wp.user.String() );
		fprintf( ptr, "x1=\"%ld\" y1=\"%ld\" ", int32(wp.frame.LeftTop().x), int32(wp.frame.LeftTop().y) );
		fprintf( ptr, "x2=\"%ld\" y2=\"%ld\" ", int32(wp.frame.RightBottom().x), int32(wp.frame.RightBottom().y) );
		fprintf( ptr, "div=\"%f\"/>\n", wp.divider );
		kg = windows->GetNextWindowPos( wp, false );
	}
	fprintf( ptr, "	</windowpositions>\n" );
}

//-----------------------------------------------------

int32 UserManager::WarningLevel() {
	return warningLevel;
}

//-----------------------------------------------------

void UserManager::SetWarningLevel( int32 wLevel ) {
	warningLevel = wLevel;
}

//-----------------------------------------------------
