#include <Screen.h>
#include <SupportKit.h>
#include <Application.h>
#include "WindowManager.h"
#include "ChatWindow.h"
#include "InfoWindow.h"
#include "GenericInput.h"
#include "BuddyList.h"
#include "MiscStuff.h"
#include "Globals.h"
#include "Say.h"

// window files
#include "ResultsWindow.h"
#include "AwayEditor.h"
#include "PrefsWindow.h"
#include "ProfileEditor.h"
#include "CustomAwayMsgEditor.h"
#include "ImporterWindow.h"
#include "BlockListEditor.h"
#include "AboutBox.h"

//---------------------------------------------------------------------

WindowManager::WindowManager() {
	
	//IMWindowRect = BRect( 25, 25, 400, 400 );
	IMWindowRect = BRect( 25, 45, 350, 350 );
	lock = new BLocker( "window_lock" );
	resWin = NULL;
	bList = NULL;
	signOn = NULL;
	profEd = NULL;
	awayEd = NULL;
	custEd = NULL;
	prefsWin = NULL;
	impWin = NULL;
	aboutBox = NULL;
	blockEd = NULL;
}

//---------------------------------------------------------------------

void WindowManager::ToggleHidden( bool forceShow ) {

	lock->Lock();
	bool show = forceShow ? false : !bList->IsMinimized();

	// toggle the minimization of the Buddy List window...
	if( bList && bList->Lock() ) {
		bList->Minimize(show);
		if( forceShow )
			bList->Activate();
		bList->Unlock();
	}

	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::CalcNewIMRect() {

	lock->Lock();
	BScreen* screen = new BScreen;
	
	// offset it... put it back to top-left if it doesn't fit onscreen
	IMWindowRect.OffsetBy( 20, 20 );
	if( !screen->Frame().Contains(IMWindowRect) )
		IMWindowRect = BRect( 25, 25, 400, 400 );
	
	delete screen;
	lock->Unlock();
}

//---------------------------------------------------------------------

BWindow* WindowManager::GetBuddyList() {

	BWindow* ret;
	lock->Lock();
	ret = bList;
	lock->Unlock();
	return ret;
}

//---------------------------------------------------------------------

// for DEBUG USE ONLY!!!!!!!
void WindowManager::MakeDataSenderWindow() {

	BRect frame(0,0,415,250);
	frame.OffsetBy( 100, 100 );

	DataSenderWindow* dSend = new DataSenderWindow( frame );
	dSend->Show();
}

//---------------------------------------------------------------------

void WindowManager::UserWindowClosed( BMessage* message )
{
	lock->Lock();

	char ScreenName[INTERNAL_NAME_MAX];
	strcpy( ScreenName, message->FindString("userid") );
	int wtype = (int)message->FindInt32("wtype");
	bool Found = false;
	unsigned int i;
	
	// only save the position for *message* windows...
	if( wtype == USER_MESSAGE_TYPE )
		SaveWindowPos( BString(ScreenName), message->FindRect("frame"), message->FindFloat("divider") );
	
	// Find the window in the list
	Found = false;
	for( i = 0; i < IMWindows.Count(); ++i ) {
		if( IMWindows[i].type == wtype &&
			CompareScreenNames((char*)ScreenName, (char*)IMWindows[i].name) )
		{
			Found = true;
			break;
		}
	}
	
	// Delete the record (if it was there)
	if( Found )
		IMWindows.Delete( i );

	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::SaveWindowPos( BString user, BRect frame, float divider ) {

	bool Found = false;
	unsigned int i;

	// find the correct position record
	for( i = 0; i < winPositions.Count(); ++i ) {
		if( CompareScreenNames( (char*)user.String(), (char*)winPositions[i].user.String()) )
		{
			Found = true;
			winPositions[i].frame = frame;
			winPositions[i].divider = divider;
			break;
		}
	}
	
	// not found? Add it
	if( !Found ) {
		winPosRect adder;
		adder.user = user;
		adder.frame = frame;
		adder.divider = divider;
		winPositions.Add(adder);
	}
}

//---------------------------------------------------------------------

void WindowManager::SwitchUserWindow( BMessage* message ) {

	lock->Lock();
	IMWindowItem item, target;
	bool Found = false, keepGoing;
	
	// true if we should go the the next one, false if to the prev
	bool forward = bool(message->what == BEAIM_NEXT_CHAT_WINDOW);
	
	// if the list is empty, just ignore it
	if( IMWindows.IsEmpty() )
		return;	

	// was this sent from a user window or somewhere else?
	if( message->HasString("userid") ) {

		char ScreenName[INTERNAL_NAME_MAX];
		strcpy( ScreenName, message->FindString("userid") );
		int wtype = (int)message->FindInt32("wtype");

		// Find the window in the list
		keepGoing = IMWindows.First( item );
		while( keepGoing ) {
			if( item.type == wtype &&
				CompareScreenNames( ScreenName, item.name ) )
			{
				Found = true;
				break;
			}
			keepGoing = IMWindows.Next( item );
		}
	}
	
	// if it wasn't there, use the first or last one
	if( !Found ) {
		if( forward )
			IMWindows.First( target );
		else
			IMWindows.Last( target );
		target.window->Activate();

		lock->Unlock();
		return;
	}

	// now look for the next (or previous) window
	if( forward ) {
		keepGoing = IMWindows.Next( target );
		if( !keepGoing )
			keepGoing = IMWindows.First( target );
	} else {
		keepGoing = IMWindows.Prev( target );
		if( !keepGoing )
			keepGoing = IMWindows.Last( target );
	}

	// activate the target window
	target.window->Activate();

	lock->Unlock();
}

//---------------------------------------------------------------------


void WindowManager::OpenBuddyList( BPoint* point ) {

	lock->Lock();
	BRect winRect, frame;
	BRect screenRect = BScreen().Frame();
	
	// get the window position from the global prefs
	winRect.left = float(user->ReadInt32( "buddylist.x1", -32000 ));
	winRect.top = float(user->ReadInt32( "buddylist.y1", -32000 ));
	winRect.right = float(user->ReadInt32( "buddylist.x2", -32000 ));
	winRect.bottom = float(user->ReadInt32( "buddylist.y2", -32000 ));
	
	// try to recreate the saved frame... if we can't, go with the default frame
	if( winRect.left == -32000 || winRect.top == -32000 ||
		winRect.right == -32000 || winRect.bottom == -32000 ) 
	{
		frame = BRect(0,0,158,340);
		frame.OffsetBy( screenRect.Width() - frame.Width() - 10, 29 );
	} else {
		frame = winRect;
		CorrectFrame( frame );
	}
	
	// create the buddylist window and show it
	bList = new BuddyList( frame );
	bList->Show();
	
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::UserWindowOpened( BMessage* message )
{
	lock->Lock();

	BWindow* newWindow;
	char name[INTERNAL_NAME_MAX];

	// Get the data from the BMessage and make a new record out of it
	message->FindPointer( "new_window", (void**)&newWindow );
	strcpy( name, message->FindString("userid") );
	IMWindowItem newRecord( name, (int)message->FindInt32("wtype"), newWindow );
	
	// Add that record to the list
	IMWindows.Add( newRecord );
	
	lock->Unlock();
}

//---------------------------------------------------------------------

BWindow* WindowManager::OpenUserWindow( BMessage* message )
{
	lock->Lock();
	BWindow *userWindow = NULL, *posWindow = NULL;

	BString userName;
	userName = BString(message->FindString("userid"));
	int wtype = (int)message->FindInt32("wtype");
	if( message->HasPointer("poswindow") )
		message->FindPointer( "poswindow", (void**)&posWindow );

	for( unsigned int i = 0; i < IMWindows.Count(); ++i ) {

		// The window the user wants is already open... activate it
		if( IMWindows[i].type == wtype &&
			CompareScreenNames(const_cast<char*>(userName.String()), IMWindows[i].name) )
		{
			BWindow* theWindow = IMWindows[i].window;
			theWindow->Activate();
			lock->Unlock();
			return theWindow;
		}
	}

	// Otherwise, make a new one
	switch( wtype ) {

		// a message (IM) window
		case USER_MESSAGE_TYPE:
			userWindow = MakeMessageWindow( userName );
			break;
			
		case USER_INFO_TYPE:
			userWindow = MakeInfoWindow( userName, posWindow );
			break;
	}

	// show it and quit
	userWindow->Show();
	lock->Unlock();
	return userWindow;
}

//---------------------------------------------------------------------

BWindow* WindowManager::MakeMessageWindow( BString userName ) {

	BRect screenRect = BScreen().Frame();
	unsigned int i = 0;
	bool Found = false;
	float divider = 0.8;
	BWindow* ret;
	BRect frame = IMWindowRect;

	// try and find a saved window position for this user
	for( i = 0; i < winPositions.Count(); ++i ) {
		if( CompareScreenNames( (char*)userName.String(), (char*)winPositions[i].user.String()) )
		{
			Found = true;
			userName = winPositions[i].user;
			frame = winPositions[i].frame;
			divider = winPositions[i].divider;
			break;
		}
	}

	// use one if we found one; otherwise, use the default
	if( !Found || !screenRect.Contains(frame) ) {
		ret = new ChatWindow( userName, IMWindowRect );
		CalcNewIMRect();
	} else
		ret = new ChatWindow( userName, frame, divider );
	
	return ret;
}

//---------------------------------------------------------------------

BWindow* WindowManager::MakeInfoWindow( BString userName, BWindow* posWindow )
{
	lock->Lock();
	BRect rect(0,0,330,286);

	// position it appropriately
	if( posWindow )
		MakeDialogFrame( rect, posWindow );
	else
		CenterFrame( rect );

	// make it	
	InfoWindow* ret = new InfoWindow( rect, userName );
	lock->Unlock();
	return ret;
}
		
//---------------------------------------------------------------------

void WindowManager::OpenSignOnWindow( BPoint* point ) {

	lock->Lock();
	BPoint winPos;
	BRect frame( 0, 0, 201, 205 );
	BRect screenRect = BScreen().Frame();
	
	// get the window position from the global prefs
	winPos.x = float(user->ReadInt32( "login.x", -32000, true ));
	winPos.y = float(user->ReadInt32( "login.y", -32000, true ));
	
	// try to recreate the saved frame... if we can't, go with the default frame
	if( winPos.x == -32000 || winPos.x == -32000 )
		frame.OffsetBy( screenRect.Width() - frame.Width() - 10, 29 );
	else {
		frame.OffsetTo( winPos );
		CorrectFrame( frame );
	}

	// create the signon window and show it
	signOn = new LoginWindow( frame );
	signOn->Show();

	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::CloseSignOnWindow( BPoint* point ) {

	lock->Lock();

	// lock the sign on window and close it
	signOn->Lock();
	signOn->Quit();
	
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::ForwardIncomingMessage( BMessage* message ) {

	lock->Lock();

	char ScreenName[INTERNAL_NAME_MAX];
	strcpy( ScreenName, message->FindString("userid") );
	bool Found = false;
	unsigned int i;
	int wtype = USER_OTHER_INFO;

	if( message->HasInt32("wtype") )
		wtype = (int)message->FindInt32("wtype");

	// Find the window in the list
	for( i = 0; i < IMWindows.Count(); ++i ) {
		printf( "comaprink: %s\n", IMWindows[i].name );
		if( (wtype == USER_OTHER_INFO || IMWindows[i].type == wtype) && 
			CompareScreenNames(ScreenName, IMWindows[i].name) )
		{
			Found = true;
			break;
		}
	}
	
	// Construct a BMessenger and forward the message
	if( Found ) {
		BMessenger sender( IMWindows[i].window );
		sender.SendMessage( message );
	} else if( wtype == USER_MESSAGE_TYPE ) {
		
		BWindow* newWindow = OpenUserWindow( message );
		BMessenger sender( newWindow );
		sender.SendMessage( message );	
	}
	
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::SendBuddyListMessage( BMessage* message ) {

	lock->Lock();
	bList->PostMessage( message );
	lock->Unlock();
}
		
//---------------------------------------------------------------------		

void WindowManager::SendSignOnMessage( BMessage* message ) {

	lock->Lock();
	signOn->PostMessage( message );
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::BroadcastMessage( BMessage* message ) {

	lock->Lock();
	SendBuddyListMessage( message );
	
	if( bList ) {
		bList->Lock();
		bList->PostMessage( message );
		bList->Unlock();
	}

	if( resWin ) {
		resWin->Lock();
		resWin->PostMessage( message );
		resWin->Unlock();
	}
	
	if( awayEd ) {
		awayEd->Lock();
		awayEd->PostMessage( message );
		awayEd->Unlock();
	}
	
	if( profEd ) {
		profEd->Lock();
		profEd->PostMessage( message );
		profEd->Unlock();
	}
	
	if( custEd ) {
		custEd->Lock();
		custEd->PostMessage( message );
		custEd->Unlock();
	}
	
	if( prefsWin ) {
		prefsWin->Lock();
		prefsWin->PostMessage( message );
		prefsWin->Unlock();
	}
	
	if( impWin ) {
		impWin->Lock();
		impWin->PostMessage( message );
		impWin->Unlock();
	}
	
	if( aboutBox ) {
		aboutBox->Lock();
		aboutBox->PostMessage( message );
		aboutBox->Unlock();
	}
	
	if( blockEd ) {
		blockEd->Lock();
		blockEd->PostMessage( message );
		blockEd->Unlock();
	}
	
	if( !message->HasBool("noappsend") )
		be_app->PostMessage( message );
	
	// send to all the open chat windows
	for( unsigned int i = 0; i < IMWindows.Count(); ++i ) {
		IMWindows[i].window->Lock();
		IMWindows[i].window->PostMessage( message );
		IMWindows[i].window->Unlock();
	}
		
	lock->Unlock();
}
		
//---------------------------------------------------------------------

void WindowManager::DisplayMessage( BMessage* msg ) {

	BString button = "OK";
	alert_type type = B_INFO_ALERT;

	if( msg->HasString("button") )
		button = BString(msg->FindString("button"));
	if( msg->HasBool("warning") )
		type = B_WARNING_ALERT;
		
	// play the sound if one is specified
	if( msg->HasInt32("sound") )
		sounds->PlaySound( msg->FindInt32("sound") );	
	
	(new BAlert("", (char*)msg->FindString("message"), button.String(), NULL, NULL, B_WIDTH_AS_USUAL, type))->Go();
}

//---------------------------------------------------------------------

void WindowManager::DisplayError( BMessage* ) {

	lock->Lock();
	//(new BAlert("", message.String(), "OK"))->Go();
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::CenterFrame( BRect& frame ) {

	lock->Lock();
	BRect centered;
	BScreen* screen = new BScreen;	
	
	centered.left = (screen->Frame().Width() - frame.Width()) / 2;
	centered.top = (screen->Frame().Height() - frame.Height()) / 2;
	centered.right = centered.left + frame.Width();
	centered.bottom = centered.top + frame.Height();
	frame = centered;
	
	delete screen;
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::MakeDialogFrame( BRect& frame, BWindow* window ) {
	
	lock->Lock();
	BPoint point;

	// make sure there IS a window	
	if( !window ) {
		lock->Unlock();
		return;
	}
	
	// get it's top left corner and call the real function
	point = window->Frame().LeftTop();
	MakeDialogFrame( frame, point );
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::MakeDialogFrame( BRect& frame, BPoint point ) {

	lock->Lock();
	
	// Move the rectangle sligtly down and right relative to point
	frame.OffsetTo( point.x + 20, point.y + 40 );
	CorrectFrame( frame );
		
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::CorrectFrame( BRect& frame ) {

	lock->Lock();
	//BScreen* screen = new BScreen;
	BRect screenRect = BScreen().Frame();
	float offx = 0, offy = 0;

	// if the the rect isn't totally onscreen, fix it
	if( !screenRect.Contains( frame ) ) {

		// offset by the amount that the box would be offscreen
		offx = frame.left + frame.Width() - screenRect.Width();
		offy = frame.top + frame.Height() - screenRect.Height();
		frame.OffsetBy( offx < 0 ? 0 : -offx - 20, offy < 0 ? 0 : -offy - 20 );
	}
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::OpenInputWindow( char* title, char* label, BMessage* msg, BWindow* window,
									 char* initial, int maxLen, bool toApp, char* fname )
{
	lock->Lock();

	// put the rect to window in the right spot
	BRect frame(0,0,200,100);
	if( window )
		MakeDialogFrame( frame, window );
	else
		CenterFrame( frame );
		
	// make the window
	GenericInputWindow* msgBox = new GenericInputWindow( frame, title, label, msg, window,
														 initial, maxLen, fname );

	// check and see if the message is supposed to go the the app (not a window)
	if( toApp )
		msgBox->SetTarget( APP_SIGNATURE );

	msgBox->Show();	
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::OpenSingleWindow( BMessage* msg, int32 whichWindow ) {

	lock->Lock();
	BRect winRect;
	BWindow* theWindow = NULL;
	BWindow* posWindow = NULL;
	
	switch( whichWindow ) {
		case 0: posWindow = bList;
			break;
		case 1: posWindow = signOn;
			break;
	}
	
	switch( msg->FindInt32("wtype") ) {
	
		case SW_PROFILE_EDITOR:
			theWindow = profEd;
			winRect = BRect(0,0,272,177);
			break;
			
		case SW_CUSTOM_AWAY_EDITOR:
			theWindow = custEd;
			winRect = BRect(0,0,272,177);
			break;
			
		case SW_PREFERENCES:
			theWindow = prefsWin;
			winRect = BRect(0,0,420,275);
			break;
			
		case SW_AWAY_EDITOR:
			theWindow = awayEd;
			winRect = BRect(0,0,389,200);
			break;
			
		case SW_EMAIL_RESULTS:
			theWindow = resWin;
			winRect = BRect(0,0,320,185);
			break;
			
		case SW_BUDDYLIST_IMPORTER:
			theWindow = impWin;
			winRect = BRect(0,0,340,199);
			break;
			
		case SW_ABOUT_BOX:
			theWindow = aboutBox;
			winRect = BRect(0,0,201,273);
			break;
			
		case SW_BLOCKLIST_EDITOR:
			theWindow = blockEd;
			winRect = BRect(0,0,278,127);
			break;
	}	
	
	// only make a new window if it's not already open
	if( !theWindow ) {

		// put the frame in the right spot
		if( posWindow )
			MakeDialogFrame( winRect, posWindow->Frame().LeftTop() );
		else
			CenterFrame( winRect );

		// now create the window
		switch( msg->FindInt32("wtype") ) {
		
			case SW_PROFILE_EDITOR:
				theWindow = profEd = new ProfileEditorWindow( winRect );
				break;
				
			case SW_CUSTOM_AWAY_EDITOR:
				theWindow = custEd = new CustomAwayMsgWindow( winRect );
				break;
				
			case SW_PREFERENCES:
				theWindow = prefsWin = new PrefsWindow( winRect, (bool)msg->FindBool("globalonly") );
				break;
				
			case SW_AWAY_EDITOR:
				theWindow = awayEd = new AwayEditorWindow( winRect );
				break;
				
			case SW_EMAIL_RESULTS:
				theWindow = resWin = new SearchResultsWindow( winRect );
				break;
				
			case SW_BUDDYLIST_IMPORTER:
				theWindow = impWin = new ImporterWindow( winRect );
				break;
				
			case SW_ABOUT_BOX:
				theWindow = aboutBox = new AboutWindow( winRect );
				break;
				
			case SW_BLOCKLIST_EDITOR:
				theWindow = blockEd = new BlockListEditorWindow( winRect );
				break;
		}
		
		// and activate it
		if( theWindow )
			theWindow->Show();
	}
	
	// set (or reset) the window
	if( theWindow ) {
		theWindow->Activate();	
		theWindow->PostMessage( msg );
	}
	lock->Unlock();
}

//---------------------------------------------------------------------

void WindowManager::SingleWindowClosed( BMessage* msg ) {

	switch( msg->FindInt32("wtype") ) {
	
		case SW_PROFILE_EDITOR:
			profEd = NULL;
			break;
			
		case SW_CUSTOM_AWAY_EDITOR:
			custEd = NULL;
			break;
			
		case SW_PREFERENCES:
			prefsWin = NULL;
			break;
			
		case SW_AWAY_EDITOR:
			awayEd = NULL;
			break;

		case SW_EMAIL_RESULTS:
			resWin = NULL;
			break;
			
		case SW_BUDDYLIST_IMPORTER:
			impWin = NULL;
			break;
			
		case SW_ABOUT_BOX:
			aboutBox = NULL;
			break;
			
		case SW_BLOCKLIST_EDITOR:
			blockEd = NULL;
			break;
	}
}

//---------------------------------------------------------------------

void WindowManager::SendSingleWindowMessage( BMessage* msg, int wtype ) {

	BWindow* theWindow = NULL;
	switch( wtype ) {
	
		case SW_PROFILE_EDITOR:
			theWindow = profEd;
			break;
			
		case SW_CUSTOM_AWAY_EDITOR:
			theWindow = custEd;
			break;
			
		case SW_PREFERENCES:
			theWindow = prefsWin;
			break;
			
		case SW_AWAY_EDITOR:
			theWindow = awayEd;
			break;

		case SW_EMAIL_RESULTS:
			theWindow = resWin;
			break;
			
		case SW_BUDDYLIST_IMPORTER:
			theWindow = impWin;
			break;
			
		case SW_ABOUT_BOX:
			theWindow = aboutBox;
			break;
			
		case SW_BLOCKLIST_EDITOR:
			theWindow = blockEd;
			break;
	}
	
	if( theWindow ) {
		theWindow->Lock();
		theWindow->PostMessage( msg );
		theWindow->Unlock();
	}
}

//---------------------------------------------------------------------

void WindowManager::CloseSingleWindow( BMessage* msg ) {

	BWindow* theWindow = NULL;
	switch( msg->FindInt32("wtype") ) {
	
		case SW_PROFILE_EDITOR:
			theWindow = profEd;
			break;
			
		case SW_CUSTOM_AWAY_EDITOR:
			theWindow = custEd;
			break;
			
		case SW_PREFERENCES:
			theWindow = prefsWin;
			break;
			
		case SW_AWAY_EDITOR:
			theWindow = awayEd;
			break;

		case SW_EMAIL_RESULTS:
			theWindow = resWin;
			break;
			
		case SW_BUDDYLIST_IMPORTER:
			theWindow = impWin;
			break;
			
		case SW_ABOUT_BOX:
			theWindow = aboutBox;
			break;
			
		case SW_BLOCKLIST_EDITOR:
			theWindow = blockEd;
			break;
	}
	
	if( theWindow )
		theWindow->PostMessage( new BMessage(B_QUIT_REQUESTED) );
}

//---------------------------------------------------------------------

void WindowManager::ForwardEmailSearchResults( BMessage* message ) {
	if( resWin )
		resWin->PostMessage( message );
}

//---------------------------------------------------------------------

bool WindowManager::GetNextWindowPos( winPosRect& wp, bool first ) {

	if( first )
		return winPositions.First(wp);
	else
		return winPositions.Next(wp);
}

//---------------------------------------------------------------------

void WindowManager::ClearWindowPositions() {
	winPositions.Clear();
}

//---------------------------------------------------------------------

// a horrendously inefficient procedure, since it has to traverse the 
// entire list whenever we're going to add a new winpos record. But
// I'll leave it there now for safety's sake...
void WindowManager::SetWindowPos( winPosRect& wp ) {
	winPositions.Add(wp);
}

//---------------------------------------------------------------------

void WindowManager::CloseDaWindows() {

	BRect frame;
	float div;
	
	BMessage* clsMessage = new BMessage(B_QUIT_REQUESTED);
	clsMessage->AddBool("dontreport", true);
	
	// close all the windows and save their positions
	for( unsigned int i = 0; i < IMWindows.Count(); ++i ) {
		IMWindows[i].window->Lock();
		frame = IMWindows[i].window->Frame();
		div = ((ChatWindow*)IMWindows[i].window)->Divider();
		IMWindows[i].window->PostMessage( clsMessage );
		IMWindows[i].window->Unlock();
		SaveWindowPos( BString(IMWindows[i].name), frame, div );
	}
	
	IMWindows.Clear();
	
	if( resWin ) {
		resWin->Lock();
		resWin->PostMessage( clsMessage );
		resWin->Unlock();
	}
	
	if( awayEd ) {
		awayEd->Lock();
		awayEd->PostMessage( clsMessage );
		awayEd->Unlock();
	}
	
	if( profEd ) {
		profEd->Lock();
		profEd->PostMessage( clsMessage );
		profEd->Unlock();
	}
	
	if( custEd ) {
		custEd->Lock();
		custEd->PostMessage( clsMessage );
		custEd->Unlock();
	}
	
	if( prefsWin ) {
		prefsWin->Lock();
		prefsWin->PostMessage( clsMessage );
		prefsWin->Unlock();
	}
	
	if( impWin ) {
		impWin->Lock();
		impWin->PostMessage( clsMessage );
		impWin->Unlock();
	}
	
	if( aboutBox ) {
		aboutBox->Lock();
		aboutBox->PostMessage( clsMessage );
		aboutBox->Unlock();
	}
	
	if( blockEd ) {
		blockEd->Lock();
		blockEd->PostMessage( clsMessage );
		blockEd->Unlock();
	}
}

//---------------------------------------------------------------------

